@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2020 Cppcheck team.
+ * Copyright (C) 2007-2022 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,8 +20,10 @@
 
 #include "analyzerinfo.h"
 #include "cmdlineparser.h"
+#include "color.h"
 #include "config.h"
 #include "cppcheck.h"
+#include "errortypes.h"
 #include "filelister.h"
 #include "importproject.h"
 #include "library.h"
@@ -34,12 +36,17 @@
 #include "utils.h"
 #include "checkunusedfunctions.h"
 
+#include <algorithm>
+#include <atomic>
 #include <csignal>
 #include <cstdio>
 #include <cstdlib> // EXIT_SUCCESS and EXIT_FAILURE
 #include <cstring>
+#include <functional>
 #include <iostream>
 #include <list>
+#include <memory>
+#include <type_traits>
 #include <utility>
 #include <vector>
 
@@ -51,12 +58,11 @@
 #   include <ucontext.h>
 
 #   undef _XOPEN_SOURCE
-#elif !defined(__OpenBSD__)
+#elif !defined(__OpenBSD__) && !defined(__HAIKU__)
 #   include <ucontext.h>
 #endif
 #ifdef __linux__
-#include <sys/syscall.h>
-#include <sys/types.h>
+#include <syscall.h>
 #endif
 #endif
 
@@ -66,11 +72,19 @@
 #include <execinfo.h>
 #endif
 
+#if defined(_WIN32)
 #if defined(_MSC_VER)
 #define USE_WINDOWS_SEH
-#include <DbgHelp.h>
-#include <TCHAR.H>
-#include <Windows.h>
+#endif
+#if defined (__MINGW32__)
+#   include <windows.h>
+#   include <dbghelp.h>
+#   include <tchar.h>
+#else
+#   include <Windows.h>
+#   include <DbgHelp.h>
+#   include <TCHAR.H>
+#endif
 #include <excpt.h>
 #endif
 
@@ -78,14 +92,12 @@
 /*static*/ FILE* CppCheckExecutor::mExceptionOutput = stdout;
 
 CppCheckExecutor::CppCheckExecutor()
-    : mSettings(nullptr), mLatestProgressOutputTime(0), mErrorOutput(nullptr), mBugHuntingReport(nullptr), mShowAllErrors(false)
-{
-}
+    : mSettings(nullptr), mLatestProgressOutputTime(0), mErrorOutput(nullptr), mShowAllErrors(false)
+{}
 
 CppCheckExecutor::~CppCheckExecutor()
 {
     delete mErrorOutput;
-    delete mBugHuntingReport;
 }
 
 bool CppCheckExecutor::parseFromArgs(CppCheck *cppcheck, int argc, const char* const argv[])
@@ -96,19 +108,23 @@ bool CppCheckExecutor::parseFromArgs(CppCheck *cppcheck, int argc, const char* c
 
     if (success) {
         if (parser.getShowVersion() && !parser.getShowErrorMessages()) {
-            const char * const extraVersion = CppCheck::extraVersion();
-            if (*extraVersion != 0)
-                std::cout << "Cppcheck " << CppCheck::version() << " ("
-                          << extraVersion << ')' << std::endl;
-            else
-                std::cout << "Cppcheck " << CppCheck::version() << std::endl;
+            if (!settings.cppcheckCfgProductName.empty()) {
+                std::cout << settings.cppcheckCfgProductName << std::endl;
+            } else {
+                const char * const extraVersion = CppCheck::extraVersion();
+                if (*extraVersion != 0)
+                    std::cout << "Cppcheck " << CppCheck::version() << " ("
+                              << extraVersion << ')' << std::endl;
+                else
+                    std::cout << "Cppcheck " << CppCheck::version() << std::endl;
+            }
         }
 
         if (parser.getShowErrorMessages()) {
             mShowAllErrors = true;
-            std::cout << ErrorLogger::ErrorMessage::getXMLHeader();
+            std::cout << ErrorMessage::getXMLHeader();
             cppcheck->getErrorMessages();
-            std::cout << ErrorLogger::ErrorMessage::getXMLFooter() << std::endl;
+            std::cout << ErrorMessage::getXMLFooter() << std::endl;
         }
 
         if (parser.exitAfterPrinting()) {
@@ -123,13 +139,13 @@ bool CppCheckExecutor::parseFromArgs(CppCheck *cppcheck, int argc, const char* c
     {
         for (std::list<std::string>::iterator iter = settings.includePaths.begin();
              iter != settings.includePaths.end();
-            ) {
+             ) {
             const std::string path(Path::toNativeSeparators(*iter));
             if (FileLister::isDirectory(path))
                 ++iter;
             else {
                 // If the include path is not found, warn user and remove the non-existing path from the list.
-                if (settings.isEnabled(Settings::INFORMATION))
+                if (settings.severity.isEnabled(Severity::information))
                     std::cout << "(information) Couldn't find path given by -I '" << path << '\'' << std::endl;
                 iter = settings.includePaths.erase(iter);
             }
@@ -158,13 +174,13 @@ bool CppCheckExecutor::parseFromArgs(CppCheck *cppcheck, int argc, const char* c
 #else
     const bool caseSensitive = true;
 #endif
-    if (!mSettings->project.fileSettings.empty() && !mSettings->fileFilter.empty()) {
+    if (!mSettings->project.fileSettings.empty() && !mSettings->fileFilters.empty()) {
         // filter only for the selected filenames from all project files
         std::list<ImportProject::FileSettings> newList;
 
         for (const ImportProject::FileSettings &fsetting : settings.project.fileSettings) {
-            if (matchglob(mSettings->fileFilter, fsetting.filename)) {
-                newList.push_back(fsetting);
+            if (matchglobs(mSettings->fileFilters, fsetting.filename)) {
+                newList.emplace_back(fsetting);
             }
         }
         if (!newList.empty())
@@ -176,8 +192,12 @@ bool CppCheckExecutor::parseFromArgs(CppCheck *cppcheck, int argc, const char* c
     } else if (!pathnames.empty()) {
         // Execute recursiveAddFiles() to each given file parameter
         const PathMatch matcher(ignored, caseSensitive);
-        for (const std::string &pathname : pathnames)
-            FileLister::recursiveAddFiles(mFiles, Path::toNativeSeparators(pathname), mSettings->library.markupExtensions(), matcher);
+        for (const std::string &pathname : pathnames) {
+            std::string err = FileLister::recursiveAddFiles(mFiles, Path::toNativeSeparators(pathname), mSettings->library.markupExtensions(), matcher);
+            if (!err.empty()) {
+                std::cout << "cppcheck: " << err << std::endl;
+            }
+        }
     }
 
     if (mFiles.empty() && settings.project.fileSettings.empty()) {
@@ -185,10 +205,10 @@ bool CppCheckExecutor::parseFromArgs(CppCheck *cppcheck, int argc, const char* c
         if (!ignored.empty())
             std::cout << "cppcheck: Maybe all paths were ignored?" << std::endl;
         return false;
-    } else if (!mSettings->fileFilter.empty() && settings.project.fileSettings.empty()) {
+    } else if (!mSettings->fileFilters.empty() && settings.project.fileSettings.empty()) {
         std::map<std::string, std::size_t> newMap;
         for (std::map<std::string, std::size_t>::const_iterator i = mFiles.begin(); i != mFiles.end(); ++i)
-            if (matchglob(mSettings->fileFilter, i->first)) {
+            if (matchglobs(mSettings->fileFilters, i->first)) {
                 newMap[i->first] = i->second;
             }
         mFiles = newMap;
@@ -209,7 +229,7 @@ int CppCheckExecutor::check(int argc, const char* const argv[])
 
     CheckUnusedFunctions::clear();
 
-    CppCheck cppCheck(*this, true);
+    CppCheck cppCheck(*this, true, executeCommand);
 
     const Settings& settings = cppCheck.settings();
     mSettings = &settings;
@@ -226,9 +246,9 @@ int CppCheckExecutor::check(int argc, const char* const argv[])
     int ret;
 
     if (cppCheck.settings().exceptionHandling)
-        ret = check_wrapper(cppCheck, argc, argv);
+        ret = check_wrapper(cppCheck);
     else
-        ret = check_internal(cppCheck, argc, argv);
+        ret = check_internal(cppCheck);
 
     mSettings = nullptr;
     return ret;
@@ -244,7 +264,7 @@ void CppCheckExecutor::setSettings(const Settings &settings)
  * \return size of array
  * */
 template<typename T, int size>
-std::size_t getArrayLength(const T(&)[size])
+std::size_t getArrayLength(const T (&)[size])
 {
     return size;
 }
@@ -277,6 +297,7 @@ static void print_stacktrace(FILE* output, bool demangling, int maxdepth, bool l
         char **symbolStringList = backtrace_symbols(callstackArray, currentdepth);
         if (symbolStringList) {
             fputs("Callstack:\n", output);
+            char demangle_buffer[2048]= {0};
             for (int i = offset; i < maxdepth; ++i) {
                 const char * const symbolString = symbolStringList[i];
                 char * realnameString = nullptr;
@@ -291,12 +312,11 @@ static void print_stacktrace(FILE* output, bool demangling, int maxdepth, bool l
                     if (plus && (plus>(firstBracketName+1))) {
                         char input_buffer[1024]= {0};
                         strncpy(input_buffer, firstBracketName+1, plus-firstBracketName-1);
-                        char output_buffer[2048]= {0};
-                        size_t length = getArrayLength(output_buffer);
+                        size_t length = getArrayLength(demangle_buffer);
                         int status=0;
                         // We're violating the specification - passing stack address instead of malloc'ed heap.
                         // Benefit is that no further heap is required, while there is sufficient stack...
-                        realnameString = abi::__cxa_demangle(input_buffer, output_buffer, &length, &status); // non-NULL on success
+                        realnameString = abi::__cxa_demangle(input_buffer, demangle_buffer, &length, &status); // non-NULL on success
                     }
                 }
                 const int ordinal=i-offset;
@@ -322,14 +342,18 @@ static void print_stacktrace(FILE* output, bool demangling, int maxdepth, bool l
     }
 #undef ADDRESSDISPLAYLENGTH
 #else
-    UNUSED(output);
-    UNUSED(demangling);
-    UNUSED(maxdepth);
-    UNUSED(lowMem);
+    (void)output;
+    (void)demangling;
+    (void)maxdepth;
+    (void)lowMem;
 #endif
 }
 
+#ifdef __USE_DYNAMIC_STACK_SIZE
+static const size_t MYSTACKSIZE = 16*1024+32768; // wild guess about a reasonable buffer
+#else
 static const size_t MYSTACKSIZE = 16*1024+SIGSTKSZ; // wild guess about a reasonable buffer
+#endif
 static char mytstack[MYSTACKSIZE]= {0}; // alternative stack for signal handler
 static bool bStackBelowHeap=false; // lame attempt to locate heap vs. stack address space. See CppCheckExecutor::check_wrapper()
 
@@ -356,7 +380,7 @@ static bool IsAddressOnStack(const void* ptr)
  */
 
 #define DECLARE_SIGNAL(x) std::make_pair(x, #x)
-typedef std::map<int, std::string> Signalmap_t;
+using Signalmap_t = std::map<int, std::string>;
 static const Signalmap_t listofsignals = {
     DECLARE_SIGNAL(SIGABRT),
     DECLARE_SIGNAL(SIGBUS),
@@ -389,7 +413,7 @@ static void CppcheckSignalHandler(int signo, siginfo_t * info, void * context)
         type = (int)uc->uc_mcontext.gregs[REG_ERR] & 2;
     }
 #else
-    UNUSED(context);
+    (void)context;
     killid = getpid();
 #endif
 
@@ -538,7 +562,7 @@ static void CppcheckSignalHandler(int signo, siginfo_t * info, void * context)
                 (type==0) ? "reading " : "writing ",
                 (unsigned long)info->si_addr,
                 (isAddressOnStack)?" Stackoverflow?":""
-               );
+                );
         break;
     case SIGUSR1:
         fputs("cppcheck received signal ", output);
@@ -579,15 +603,15 @@ namespace {
     };
     typedef BOOL (WINAPI *fpStackWalk64)(DWORD, HANDLE, HANDLE, LPSTACKFRAME64, PVOID, PREAD_PROCESS_MEMORY_ROUTINE64, PFUNCTION_TABLE_ACCESS_ROUTINE64, PGET_MODULE_BASE_ROUTINE64, PTRANSLATE_ADDRESS_ROUTINE64);
     fpStackWalk64 pStackWalk64;
-    typedef DWORD64(WINAPI *fpSymGetModuleBase64)(HANDLE, DWORD64);
+    typedef DWORD64 (WINAPI *fpSymGetModuleBase64)(HANDLE, DWORD64);
     fpSymGetModuleBase64 pSymGetModuleBase64;
     typedef BOOL (WINAPI *fpSymGetSymFromAddr64)(HANDLE, DWORD64, PDWORD64, PIMAGEHLP_SYMBOL64);
     fpSymGetSymFromAddr64 pSymGetSymFromAddr64;
     typedef BOOL (WINAPI *fpSymGetLineFromAddr64)(HANDLE, DWORD64, PDWORD, PIMAGEHLP_LINE64);
     fpSymGetLineFromAddr64 pSymGetLineFromAddr64;
-    typedef DWORD (WINAPI *fpUnDecorateSymbolName)(const TCHAR*, PTSTR, DWORD, DWORD) ;
+    typedef DWORD (WINAPI *fpUnDecorateSymbolName)(const TCHAR*, PTSTR, DWORD, DWORD);
     fpUnDecorateSymbolName pUnDecorateSymbolName;
-    typedef PVOID(WINAPI *fpSymFunctionTableAccess64)(HANDLE, DWORD64);
+    typedef PVOID (WINAPI *fpSymFunctionTableAccess64)(HANDLE, DWORD64);
     fpSymFunctionTableAccess64 pSymFunctionTableAccess64;
     typedef BOOL (WINAPI *fpSymInitialize)(HANDLE, PCSTR, BOOL);
     fpSymInitialize pSymInitialize;
@@ -620,9 +644,9 @@ namespace {
             hProcess,
             nullptr,
             TRUE
-        );
-        CONTEXT             context = *(ex->ContextRecord);
-        STACKFRAME64        stack= {0};
+            );
+        CONTEXT context = *(ex->ContextRecord);
+        STACKFRAME64 stack= {0};
 #ifdef _M_IX86
         stack.AddrPC.Offset    = context.Eip;
         stack.AddrPC.Mode      = AddrModeFlat;
@@ -647,18 +671,18 @@ namespace {
             BOOL result = pStackWalk64
                           (
 #ifdef _M_IX86
-                              IMAGE_FILE_MACHINE_I386,
+                IMAGE_FILE_MACHINE_I386,
 #else
-                              IMAGE_FILE_MACHINE_AMD64,
+                IMAGE_FILE_MACHINE_AMD64,
 #endif
-                              hProcess,
-                              hThread,
-                              &stack,
-                              &context,
-                              nullptr,
-                              pSymFunctionTableAccess64,
-                              pSymGetModuleBase64,
-                              nullptr
+                hProcess,
+                hThread,
+                &stack,
+                &context,
+                nullptr,
+                pSymFunctionTableAccess64,
+                pSymGetModuleBase64,
+                nullptr
                           );
             if (!result)  // official end...
                 break;
@@ -800,12 +824,12 @@ namespace {
  * TODO Check for multi-threading issues!
  *
  */
-int CppCheckExecutor::check_wrapper(CppCheck& cppcheck, int argc, const char* const argv[])
+int CppCheckExecutor::check_wrapper(CppCheck& cppcheck)
 {
 #ifdef USE_WINDOWS_SEH
     FILE *outputFile = stdout;
     __try {
-        return check_internal(cppcheck, argc, argv);
+        return check_internal(cppcheck);
     } __except (filterException(GetExceptionCode(), GetExceptionInformation())) {
         // reporting to stdout may not be helpful within a GUI application...
         fputs("Please report this to the cppcheck developers!\n", outputFile);
@@ -833,52 +857,45 @@ int CppCheckExecutor::check_wrapper(CppCheck& cppcheck, int argc, const char* co
     for (std::map<int, std::string>::const_iterator sig=listofsignals.begin(); sig!=listofsignals.end(); ++sig) {
         sigaction(sig->first, &act, nullptr);
     }
-    return check_internal(cppcheck, argc, argv);
+    return check_internal(cppcheck);
 #else
-    return check_internal(cppcheck, argc, argv);
+    return check_internal(cppcheck);
 #endif
 }
 
 /*
  * That is a method which gets called from check_wrapper
  * */
-int CppCheckExecutor::check_internal(CppCheck& cppcheck, int /*argc*/, const char* const argv[])
+int CppCheckExecutor::check_internal(CppCheck& cppcheck)
 {
     Settings& settings = cppcheck.settings();
     mSettings = &settings;
-    const bool std = tryLoadLibrary(settings.library, argv[0], "std.cfg");
+    const bool std = tryLoadLibrary(settings.library, settings.exename, "std.cfg");
 
     for (const std::string &lib : settings.libraries) {
-        if (!tryLoadLibrary(settings.library, argv[0], lib.c_str())) {
+        if (!tryLoadLibrary(settings.library, settings.exename, lib.c_str())) {
             const std::string msg("Failed to load the library " + lib);
-            const std::list<ErrorLogger::ErrorMessage::FileLocation> callstack;
-            ErrorLogger::ErrorMessage errmsg(callstack, emptyString, Severity::information, msg, "failedToLoadCfg", false);
+            const std::list<ErrorMessage::FileLocation> callstack;
+            ErrorMessage errmsg(callstack, emptyString, Severity::information, msg, "failedToLoadCfg", Certainty::normal);
             reportErr(errmsg);
             return EXIT_FAILURE;
         }
     }
 
-    bool posix = true;
-    if (settings.posix())
-        posix = tryLoadLibrary(settings.library, argv[0], "posix.cfg");
-    bool windows = true;
-    if (settings.isWindowsPlatform())
-        windows = tryLoadLibrary(settings.library, argv[0], "windows.cfg");
-
-    if (!std || !posix || !windows) {
-        const std::list<ErrorLogger::ErrorMessage::FileLocation> callstack;
-        const std::string msg("Failed to load " + std::string(!std ? "std.cfg" : !posix ? "posix.cfg" : "windows.cfg") + ". Your Cppcheck installation is broken, please re-install.");
+    if (!std) {
+        const std::list<ErrorMessage::FileLocation> callstack;
+        const std::string msg("Failed to load std.cfg. Your Cppcheck installation is broken, please re-install.");
 #ifdef FILESDIR
         const std::string details("The Cppcheck binary was compiled with FILESDIR set to \""
                                   FILESDIR "\" and will therefore search for "
                                   "std.cfg in " FILESDIR "/cfg.");
 #else
-        const std::string cfgfolder(Path::fromNativeSeparators(Path::getPathFromFilename(argv[0])) + "cfg");
+        const std::string cfgfolder(Path::fromNativeSeparators(Path::getPathFromFilename(settings.exename)) + "cfg");
         const std::string details("The Cppcheck binary was compiled without FILESDIR set. Either the "
                                   "std.cfg should be available in " + cfgfolder + " or the FILESDIR "
                                   "should be configured.");
 #endif
-        ErrorLogger::ErrorMessage errmsg(callstack, emptyString, Severity::information, msg+" "+details, "failedToLoadCfg", false);
+        ErrorMessage errmsg(callstack, emptyString, Severity::information, msg+" "+details, "failedToLoadCfg", Certainty::normal);
         reportErr(errmsg);
         return EXIT_FAILURE;
     }
@@ -891,14 +908,16 @@ int CppCheckExecutor::check_internal(CppCheck& cppcheck, int /*argc*/, const cha
     }
 
     if (settings.xml) {
-        reportErr(ErrorLogger::ErrorMessage::getXMLHeader());
+        reportErr(ErrorMessage::getXMLHeader());
     }
 
     if (!settings.buildDir.empty()) {
+        settings.loadSummaries();
+
         std::list<std::string> fileNames;
         for (std::map<std::string, std::size_t>::const_iterator i = mFiles.begin(); i != mFiles.end(); ++i)
-            fileNames.push_back(i->first);
-        AnalyzerInformation::writeFilesTxt(settings.buildDir, fileNames, settings.project.fileSettings);
+            fileNames.emplace_back(i->first);
+        AnalyzerInformation::writeFilesTxt(settings.buildDir, fileNames, settings.userDefines, settings.project.fileSettings);
     }
 
     unsigned int returnValue = 0;
@@ -960,7 +979,7 @@ int CppCheckExecutor::check_internal(CppCheck& cppcheck, int /*argc*/, const cha
 
     cppcheck.analyseWholeProgram(mSettings->buildDir, mFiles);
 
-    if (settings.isEnabled(Settings::INFORMATION) || settings.checkConfiguration) {
+    if (settings.severity.isEnabled(Severity::information) || settings.checkConfiguration) {
         const bool enableUnusedFunctionCheck = cppcheck.isUnusedFunctionCheckEnabled();
 
         if (settings.jointSuppressionReport) {
@@ -979,25 +998,25 @@ int CppCheckExecutor::check_internal(CppCheck& cppcheck, int /*argc*/, const cha
     if (!settings.checkConfiguration) {
         cppcheck.tooManyConfigsError("",0U);
 
-        if (settings.isEnabled(Settings::MISSING_INCLUDE) && (Preprocessor::missingIncludeFlag || Preprocessor::missingSystemIncludeFlag)) {
-            const std::list<ErrorLogger::ErrorMessage::FileLocation> callStack;
-            ErrorLogger::ErrorMessage msg(callStack,
-                                          emptyString,
-                                          Severity::information,
-                                          "Cppcheck cannot find all the include files (use --check-config for details)\n"
-                                          "Cppcheck cannot find all the include files. Cppcheck can check the code without the "
-                                          "include files found. But the results will probably be more accurate if all the include "
-                                          "files are found. Please check your project's include directories and add all of them "
-                                          "as include directories for Cppcheck. To see what files Cppcheck cannot find use "
-                                          "--check-config.",
-                                          Preprocessor::missingIncludeFlag ? "missingInclude" : "missingIncludeSystem",
-                                          false);
+        if (settings.checks.isEnabled(Checks::missingInclude) && (Preprocessor::missingIncludeFlag || Preprocessor::missingSystemIncludeFlag)) {
+            const std::list<ErrorMessage::FileLocation> callStack;
+            ErrorMessage msg(callStack,
+                             emptyString,
+                             Severity::information,
+                             "Cppcheck cannot find all the include files (use --check-config for details)\n"
+                             "Cppcheck cannot find all the include files. Cppcheck can check the code without the "
+                             "include files found. But the results will probably be more accurate if all the include "
+                             "files are found. Please check your project's include directories and add all of them "
+                             "as include directories for Cppcheck. To see what files Cppcheck cannot find use "
+                             "--check-config.",
+                             Preprocessor::missingIncludeFlag ? "missingInclude" : "missingIncludeSystem",
+                             Certainty::normal);
             reportInfo(msg);
         }
     }
 
     if (settings.xml) {
-        reportErr(ErrorLogger::ErrorMessage::getXMLFooter());
+        reportErr(ErrorMessage::getXMLFooter());
     }
 
     mSettings = nullptr;
@@ -1032,11 +1051,6 @@ static inline std::string ansiToOEM(const std::string &msg, bool doConvert)
 
 void CppCheckExecutor::reportErr(const std::string &errmsg)
 {
-    // Alert only about unique errors
-    if (mShownErrors.find(errmsg) != mShownErrors.end())
-        return;
-
-    mShownErrors.insert(errmsg);
     if (mErrorOutput)
         *mErrorOutput << errmsg << std::endl;
     else {
@@ -1044,9 +1058,9 @@ void CppCheckExecutor::reportErr(const std::string &errmsg)
     }
 }
 
-void CppCheckExecutor::reportOut(const std::string &outmsg)
+void CppCheckExecutor::reportOut(const std::string &outmsg, Color c)
 {
-    std::cout << ansiToOEM(outmsg, true) << std::endl;
+    std::cout << c << ansiToOEM(outmsg, true) << Color::Reset << std::endl;
 }
 
 void CppCheckExecutor::reportProgress(const std::string &filename, const char stage[], const std::size_t value)
@@ -1072,7 +1086,7 @@ void CppCheckExecutor::reportProgress(const std::string &filename, const char st
     }
 }
 
-void CppCheckExecutor::reportInfo(const ErrorLogger::ErrorMessage &msg)
+void CppCheckExecutor::reportInfo(const ErrorMessage &msg)
 {
     reportErr(msg);
 }
@@ -1085,28 +1099,25 @@ void CppCheckExecutor::reportStatus(std::size_t fileindex, std::size_t filecount
         oss << fileindex << '/' << filecount
             << " files checked " << percentDone
             << "% done";
-        std::cout << oss.str() << std::endl;
+        std::cout << Color::FgBlue << oss.str() << Color::Reset << std::endl;
     }
 }
 
-void CppCheckExecutor::reportErr(const ErrorLogger::ErrorMessage &msg)
+void CppCheckExecutor::reportErr(const ErrorMessage &msg)
 {
     if (mShowAllErrors) {
         reportOut(msg.toXML());
-    } else if (mSettings->xml) {
-        reportErr(msg.toXML());
-    } else {
-        reportErr(msg.toString(mSettings->verbose, mSettings->templateFormat, mSettings->templateLocation));
-    }
-}
-
-void CppCheckExecutor::bughuntingReport(const std::string &str)
-{
-    if (!mSettings || str.empty())
         return;
-    if (!mBugHuntingReport)
-        mBugHuntingReport = new std::ofstream(mSettings->bugHuntingReport);
-    (*mBugHuntingReport) << str << std::endl;
+    }
+
+    // Alert only about unique errors
+    if (!mShownErrors.insert(msg.toString(mSettings->verbose)).second)
+        return;
+
+    if (mSettings->xml)
+        reportErr(msg.toXML());
+    else
+        reportErr(msg.toString(mSettings->verbose, mSettings->templateFormat, mSettings->templateLocation));
 }
 
 void CppCheckExecutor::setExceptionOutput(FILE* exceptionOutput)
@@ -1119,46 +1130,83 @@ FILE* CppCheckExecutor::getExceptionOutput()
     return mExceptionOutput;
 }
 
-bool CppCheckExecutor::tryLoadLibrary(Library& destination, const char* basepath, const char* filename)
+bool CppCheckExecutor::tryLoadLibrary(Library& destination, const std::string& basepath, const char* filename)
 {
-    const Library::Error err = destination.load(basepath, filename);
+    const Library::Error err = destination.load(basepath.c_str(), filename);
 
-    if (err.errorcode == Library::UNKNOWN_ELEMENT)
+    if (err.errorcode == Library::ErrorCode::UNKNOWN_ELEMENT)
         std::cout << "cppcheck: Found unknown elements in configuration file '" << filename << "': " << err.reason << std::endl;
-    else if (err.errorcode != Library::OK) {
-        std::string errmsg;
+    else if (err.errorcode != Library::ErrorCode::OK) {
+        std::cout << "cppcheck: Failed to load library configuration file '" << filename << "'. ";
         switch (err.errorcode) {
-        case Library::OK:
+        case Library::ErrorCode::OK:
             break;
-        case Library::FILE_NOT_FOUND:
-            errmsg = "File not found";
+        case Library::ErrorCode::FILE_NOT_FOUND:
+            std::cout << "File not found";
             break;
-        case Library::BAD_XML:
-            errmsg = "Bad XML";
+        case Library::ErrorCode::BAD_XML:
+            std::cout << "Bad XML";
             break;
-        case Library::UNKNOWN_ELEMENT:
-            errmsg = "Unexpected element";
+        case Library::ErrorCode::UNKNOWN_ELEMENT:
+            std::cout << "Unexpected element";
             break;
-        case Library::MISSING_ATTRIBUTE:
-            errmsg = "Missing attribute";
+        case Library::ErrorCode::MISSING_ATTRIBUTE:
+            std::cout << "Missing attribute";
             break;
-        case Library::BAD_ATTRIBUTE_VALUE:
-            errmsg = "Bad attribute value";
+        case Library::ErrorCode::BAD_ATTRIBUTE_VALUE:
+            std::cout << "Bad attribute value";
             break;
-        case Library::UNSUPPORTED_FORMAT:
-            errmsg = "File is of unsupported format version";
+        case Library::ErrorCode::UNSUPPORTED_FORMAT:
+            std::cout << "File is of unsupported format version";
             break;
-        case Library::DUPLICATE_PLATFORM_TYPE:
-            errmsg = "Duplicate platform type";
+        case Library::ErrorCode::DUPLICATE_PLATFORM_TYPE:
+            std::cout << "Duplicate platform type";
             break;
-        case Library::PLATFORM_TYPE_REDEFINED:
-            errmsg = "Platform type redefined";
+        case Library::ErrorCode::PLATFORM_TYPE_REDEFINED:
+            std::cout << "Platform type redefined";
             break;
         }
         if (!err.reason.empty())
-            errmsg += " '" + err.reason + "'";
-        std::cout << "cppcheck: Failed to load library configuration file '" << filename << "'. " << errmsg << std::endl;
+            std::cout << " '" + err.reason + "'";
+        std::cout << std::endl;
         return false;
     }
     return true;
 }
+
+/**
+ * Execute a shell command and read the output from it. Returns true if command terminated successfully.
+ */
+// cppcheck-suppress passedByValue - "exe" copy needed in _WIN32 code
+bool CppCheckExecutor::executeCommand(std::string exe, std::vector<std::string> args, const std::string &redirect, std::string *output_)
+{
+    output_->clear();
+
+    std::string joinedArgs;
+    for (const std::string &arg : args) {
+        if (!joinedArgs.empty())
+            joinedArgs += " ";
+        if (arg.find(" ") != std::string::npos)
+            joinedArgs += '"' + arg + '"';
+        else
+            joinedArgs += arg;
+    }
+
+#ifdef _WIN32
+    // Extra quoutes are needed in windows if filename has space
+    if (exe.find(" ") != std::string::npos)
+        exe = "\"" + exe + "\"";
+    const std::string cmd = exe + " " + joinedArgs + " " + redirect;
+    std::unique_ptr<FILE, decltype(&_pclose)> pipe(_popen(cmd.c_str(), "r"), _pclose);
+#else
+    const std::string cmd = exe + " " + joinedArgs + " " + redirect;
+    std::unique_ptr<FILE, decltype(&pclose)> pipe(popen(cmd.c_str(), "r"), pclose);
+#endif
+    if (!pipe)
+        return false;
+    char buffer[1024];
+    while (fgets(buffer, sizeof(buffer), pipe.get()) != nullptr)
+        *output_ += buffer;
+    return true;
+}
+
