@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2019 Cppcheck team.
+ * Copyright (C) 2007-2022 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 /**
  *
  * @mainpage Cppcheck
- * @version 2.0
+ * @version 2.8
  *
  * @section overview_sec Overview
  * Cppcheck is a simple tool for static analysis of C/C++ code.
@@ -61,7 +61,14 @@
 
 #include "cppcheckexecutor.h"
 
+#ifdef NDEBUG
+#include "errortypes.h"
+
+#include <cstdlib>
+#include <exception>
 #include <iostream>
+#include <string>
+#endif
 
 #ifdef _WIN32
 #include <windows.h>
@@ -69,6 +76,12 @@
 static char exename[1024] = {0};
 #endif
 
+#if defined(__APPLE__)
+#include <mach-o/dyld.h>
+
+static char exename[1024] = {0};
+#endif
+
 /**
  * Main function of cppcheck
  *
@@ -88,7 +101,12 @@ int main(int argc, char* argv[])
     GetModuleFileNameA(nullptr, exename, sizeof(exename)/sizeof(exename[0])-1);
     argv[0] = exename;
 #endif
-
+#if defined(__APPLE__)
+    uint32_t size = sizeof(exename);
+    _NSGetExecutablePath(exename, &size);
+    argv[0] = exename;
+#endif
+// *INDENT-OFF*
 #ifdef NDEBUG
     try {
 #endif
@@ -103,20 +121,5 @@ int main(int argc, char* argv[])
     }
     return EXIT_FAILURE;
 #endif
+// *INDENT-ON*
 }
-
-
-// Warn about deprecated compilers
-#ifdef __clang__
-#   if ( __clang_major__ < 2 || ( __clang_major__  == 2 && __clang_minor__ < 9))
-#       warning "Using Clang 2.8 or earlier. Support for this version has been removed."
-#   endif
-#elif defined(__GNUC__)
-#   if (__GNUC__ < 4 || (__GNUC__ == 4 && __GNUC_MINOR__ < 6))
-#       warning "Using GCC 4.5 or earlier. Support for this version has been removed."
-#   endif
-#elif defined(_MSC_VER)
-#   if (_MSC_VER < 1800)
-#       pragma message("WARNING: Using Visual Studio 2012 or earlier. Support for this version has been removed.")
-#   endif
-#endif
