@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2020 Cppcheck team.
+ * Copyright (C) 2007-2022 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,26 +16,65 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <QApplication>
-#include <QString>
+#include "checkthread.h"
+
+#include "analyzerinfo.h"
+#include "common.h"
+#include "cppcheck.h"
+#include "erroritem.h"
+#include "errorlogger.h"
+#include "errortypes.h"
+#include "settings.h"
+#include "standards.h"
+#include "threadresult.h"
+
+#include <cstddef>
+#include <functional>
+#include <list>
+#include <map>
+#include <ostream>
+#include <set>
+#include <string>
+#include <vector>
+
 #include <QDebug>
 #include <QDir>
 #include <QFile>
-#include <QFileInfo>
-#include <QJsonDocument>
-#include <QJsonObject>
 #include <QProcess>
+#include <QRegularExpression>
 #include <QSettings>
-#include "checkthread.h"
-#include "erroritem.h"
-#include "threadresult.h"
-#include "cppcheck.h"
-#include "common.h"
+
+static bool executeCommand(std::string exe, std::vector<std::string> args, std::string redirect, std::string *output)
+{
+    output->clear();
+
+    QStringList args2;
+    for (const std::string &arg: args)
+        args2 << QString::fromStdString(arg);
+
+    QProcess process;
+    process.start(QString::fromStdString(exe), args2);
+    process.waitForFinished();
+
+    if (redirect == "2>&1") {
+        QString s1 = process.readAllStandardOutput();
+        QString s2 = process.readAllStandardError();
+        *output = (s1 + "\n" + s2).toStdString();
+    } else
+        *output = process.readAllStandardOutput().toStdString();
+
+    if (redirect.compare(0,3,"2> ") == 0) {
+        std::ofstream fout(redirect.substr(3));
+        fout << process.readAllStandardError().toStdString();
+    }
+    return process.exitCode() == 0;
+}
+
 
 CheckThread::CheckThread(ThreadResult &result) :
     mState(Ready),
     mResult(result),
-    mCppcheck(result, true),
+    mCppcheck(result, true, executeCommand),
     mAnalyseWholeProgram(false)
 {
     //ctor
@@ -60,6 +99,7 @@ void CheckThread::analyseWholeProgram(const QStringList &files)
     start();
 }
 
+// cppcheck-suppress unusedFunction - TODO: false positive
 void CheckThread::run()
 {
     mState = Running;
@@ -112,9 +152,7 @@ void CheckThread::run()
 
 void CheckThread::runAddonsAndTools(const ImportProject::FileSettings *fileSettings, const QString &fileName)
 {
-    QString dumpFile;
-
-    foreach (const QString addon, mAddonsAndTools) {
+    for (const QString& addon : mAddonsAndTools) {
         if (addon == CLANG_ANALYZER || addon == CLANG_TIDY) {
             if (!fileSettings)
                 continue;
@@ -127,17 +165,17 @@ void CheckThread::runAddonsAndTools(const ImportProject::FileSettings *fileSetti
                 args << ("-I" + QString::fromStdString(*incIt));
             for (std::list<std::string>::const_iterator i = fileSettings->systemIncludePaths.begin(); i != fileSettings->systemIncludePaths.end(); ++i)
                 args << "-isystem" << QString::fromStdString(*i);
-            foreach (QString def, QString::fromStdString(fileSettings->defines).split(";")) {
+            for (const QString& def : QString::fromStdString(fileSettings->defines).split(";")) {
                 args << ("-D" + def);
             }
-            foreach (const std::string& U, fileSettings->undefs) {
+            for (const std::string& U : fileSettings->undefs) {
                 args << QString::fromStdString("-U" + U);
             }
 
             const QString clangPath = CheckThread::clangTidyCmd();
             if (!clangPath.isEmpty()) {
                 QDir dir(clangPath + "/../lib/clang");
-                foreach (QString ver, dir.entryList()) {
+                for (QString ver : dir.entryList()) {
                     QString includePath = dir.absolutePath() + '/' + ver + "/include";
                     if (ver[0] != '.' && QDir(includePath).exists()) {
                         args << "-isystem" << includePath;
@@ -150,7 +188,7 @@ void CheckThread::runAddonsAndTools(const ImportProject::FileSettings *fileSetti
             // To create compile_commands.json in windows see:
             // https://bitsmaker.gitlab.io/post/clang-tidy-from-vs2015/
 
-            foreach (QString includePath, mClangIncludePaths) {
+            for (QString includePath : mClangIncludePaths) {
                 if (!includePath.isEmpty()) {
                     includePath.replace("\\", "/");
                     args << "-isystem" << includePath.trimmed();
@@ -163,22 +201,10 @@ void CheckThread::runAddonsAndTools(const ImportProject::FileSettings *fileSetti
             if (!fileSettings->standard.empty())
                 args << ("-std=" + QString::fromStdString(fileSettings->standard));
             else {
-                switch (mCppcheck.settings().standards.cpp) {
-                case Standards::CPP03:
-                    args << "-std=c++03";
-                    break;
-                case Standards::CPP11:
-                    args << "-std=c++11";
-                    break;
-                case Standards::CPP14:
-                    args << "-std=c++14";
-                    break;
-                case Standards::CPP17:
-                    args << "-std=c++17";
-                    break;
-                case Standards::CPP20:
-                    args << "-std=c++20";
-                    break;
+                // TODO: pass C or C++ standard based on file type
+                const std::string std = mCppcheck.settings().standards.getCPP();
+                if (!std.empty()) {
+                    args << ("-std=" + QString::fromStdString(std));
                 }
             }
 
@@ -195,7 +221,11 @@ void CheckThread::runAddonsAndTools(const ImportProject::FileSettings *fileSetti
                 process.start(clangCmd(),args2);
                 process.waitForFinished();
                 const QByteArray &ba = process.readAllStandardOutput();
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
+                const quint16 chksum = qChecksum(QByteArrayView(ba));
+#else
                 const quint16 chksum = qChecksum(ba.data(), ba.length());
+#endif
 
                 QFile f1(analyzerInfoFile + '.' + addon + "-E");
                 if (f1.open(QIODevice::ReadOnly | QIODevice::Text)) {
@@ -220,12 +250,12 @@ void CheckThread::runAddonsAndTools(const ImportProject::FileSettings *fileSetti
 
             if (addon == CLANG_ANALYZER) {
                 /*
-                // Using clang
-                args.insert(0,"--analyze");
-                args.insert(1, "-Xanalyzer");
-                args.insert(2, "-analyzer-output=text");
-                args << fileName;
-                */
+                   // Using clang
+                   args.insert(0,"--analyze");
+                   args.insert(1, "-Xanalyzer");
+                   args.insert(2, "-analyzer-output=text");
+                   args << fileName;
+                 */
                 // Using clang-tidy
                 args.insert(0,"-checks=-*,clang-analyzer-*");
                 args.insert(1, fileName);
@@ -239,7 +269,7 @@ void CheckThread::runAddonsAndTools(const ImportProject::FileSettings *fileSetti
             {
                 const QString cmd(clangTidyCmd());
                 QString debug(cmd.contains(" ") ? ('\"' + cmd + '\"') : cmd);
-                foreach (QString arg, args) {
+                for (const QString& arg : args) {
                     if (arg.contains(" "))
                         debug += " \"" + arg + '\"';
                     else
@@ -269,136 +299,22 @@ void CheckThread::runAddonsAndTools(const ImportProject::FileSettings *fileSetti
             }
 
             parseClangErrors(addon, fileName, errout);
-        } else {
-            const QString python = CheckThread::pythonCmd();
-            if (python.isEmpty())
-                continue;
-
-            const QString addonFilePath = CheckThread::getAddonFilePath(mDataDir, addon + ".py");
-            if (addonFilePath.isEmpty())
-                continue;
-
-            if (dumpFile.isEmpty()) {
-                const std::string buildDir = mCppcheck.settings().buildDir;
-                mCppcheck.settings().buildDir.clear();
-                mCppcheck.settings().dump = true;
-                if (!buildDir.empty()) {
-                    mCppcheck.settings().dumpFile = AnalyzerInformation::getAnalyzerInfoFile(buildDir, fileName.toStdString(), fileSettings ? fileSettings->cfg : std::string()) + ".dump";
-                    dumpFile = QString::fromStdString(mCppcheck.settings().dumpFile);
-                } else {
-                    dumpFile = fileName + ".dump";
-                }
-                if (fileSettings)
-                    mCppcheck.check(*fileSettings);
-                else
-                    mCppcheck.check(fileName.toStdString());
-                mCppcheck.settings().dump = false;
-                mCppcheck.settings().dumpFile.clear();
-                mCppcheck.settings().buildDir = buildDir;
-            }
-
-            QStringList args;
-            args << addonFilePath << "--cli" << dumpFile;
-            if (addon == "misra" && !mMisraFile.isEmpty() && QFileInfo(mMisraFile).exists()) {
-                if (mMisraFile.endsWith(".pdf", Qt::CaseInsensitive))
-                    args << "--misra-pdf=" + mMisraFile;
-                else
-                    args << "--rule-texts=" + mMisraFile;
-            }
-            qDebug() << python << args;
-
-            QProcess process;
-            QProcessEnvironment env = process.processEnvironment();
-            if (!env.contains("PYTHONHOME") && !python.startsWith("python")) {
-                env.insert("PYTHONHOME", QFileInfo(python).canonicalPath());
-                process.setProcessEnvironment(env);
-            }
-            process.start(python, args);
-            if (!process.waitForFinished()) {
-                const QString errMsg("ERROR: Process '" + python + " " + args.join(" ") +
-                                     "' did not finish successfully: " + process.errorString());
-                qWarning() << errMsg;
-                mResult.reportOut(errMsg.toStdString());
-            }
-            const QByteArray errout = process.readAllStandardError();
-            if (process.exitCode() != 0 && !errout.isEmpty()) {
-                const QString errMsg("ERROR: Process '" + python + " " + args.join(" ") +
-                                     "' failed with error code " +
-                                     QString::number(process.exitCode()) + ": '" +
-                                     process.errorString() +
-                                     "'\nError output: " + errout);
-                qWarning() << errMsg;
-                mResult.reportOut(errMsg.toStdString());
-            }
-            const QString output(process.readAllStandardOutput());
-            QFile f(dumpFile + '-' + addon + "-results");
-            if (f.open(QIODevice::WriteOnly | QIODevice::Text)) {
-                QTextStream out(&f);
-                out << output;
-                f.close();
-            }
-            parseAddonErrors(output, addon);
         }
     }
-
-    if (!dumpFile.isEmpty()) {
-        QFile::remove(dumpFile);
-    }
 }
 
 void CheckThread::stop()
 {
     mState = Stopping;
-    mCppcheck.terminate();
-}
-
-void CheckThread::parseAddonErrors(QString err, const QString &tool)
-{
-    Q_UNUSED(tool)
-    QTextStream in(&err, QIODevice::ReadOnly);
-    while (!in.atEnd()) {
-        QString line = in.readLine();
-        if (!line.startsWith("{"))
-            continue;
-
-        const QJsonDocument doc = QJsonDocument::fromJson(line.toLocal8Bit());
-        const QJsonObject obj = doc.object();
-
-        /*
-        msg = { 'file': location.file,
-                'linenr': location.linenr,
-                'column': location.column,
-                'severity': severity,
-                'message': message,
-                'addon': addon,
-                'errorId': errorId,
-                'extra': extra}
-        */
-
-        const std::string &filename = obj["file"].toString().toStdString();
-        const int lineNumber = obj["linenr"].toInt();
-        const int column = obj["column"].toInt();
-        const std::string severity = obj["severity"].toString().toStdString();
-        const std::string message = obj["message"].toString().toStdString();
-        const std::string id = (obj["addon"].toString() + "-" + obj["errorId"].toString()).toStdString();
-
-        std::list<ErrorLogger::ErrorMessage::FileLocation> callstack;
-        callstack.push_back(ErrorLogger::ErrorMessage::FileLocation(filename, lineNumber, column));
-        ErrorLogger::ErrorMessage errmsg(callstack, filename, Severity::fromString(severity), message, id, false);
-
-        if (isSuppressed(errmsg.toSuppressionsErrorMessage()))
-            continue;
-
-        mResult.reportErr(errmsg);
-    }
+    Settings::terminate();
 }
 
 void CheckThread::parseClangErrors(const QString &tool, const QString &file0, QString err)
 {
     QList<ErrorItem> errorItems;
     ErrorItem errorItem;
-    QRegExp r1("(.+):([0-9]+):([0-9]+): (note|warning|error|fatal error): (.*)");
-    QRegExp r2("(.*)\\[([a-zA-Z0-9\\-_\\.]+)\\]");
+    static const QRegularExpression r1("^(.+):([0-9]+):([0-9]+): (note|warning|error|fatal error): (.*)$");
+    static const QRegularExpression r2("^(.*)\\[([a-zA-Z0-9\\-_\\.]+)\\]$");
     QTextStream in(&err, QIODevice::ReadOnly);
     while (!in.atEnd()) {
         QString line = in.readLine();
@@ -417,43 +333,45 @@ void CheckThread::parseClangErrors(const QString &tool, const QString &file0, QS
             continue;
         }
 
-        if (!r1.exactMatch(line))
+        const QRegularExpressionMatch r1MatchRes = r1.match(line);
+        if (!r1MatchRes.hasMatch())
             continue;
-        if (r1.cap(4) != "note") {
+        if (r1MatchRes.captured(4) != "note") {
             errorItems.append(errorItem);
             errorItem = ErrorItem();
-            errorItem.file0 = r1.cap(1);
+            errorItem.file0 = r1MatchRes.captured(1);
         }
 
         errorItem.errorPath.append(QErrorPathItem());
-        errorItem.errorPath.last().file = r1.cap(1);
-        errorItem.errorPath.last().line = r1.cap(2).toInt();
-        errorItem.errorPath.last().column = r1.cap(3).toInt();
-        if (r1.cap(4) == "warning")
+        errorItem.errorPath.last().file = r1MatchRes.captured(1);
+        errorItem.errorPath.last().line = r1MatchRes.captured(2).toInt();
+        errorItem.errorPath.last().column = r1MatchRes.captured(3).toInt();
+        if (r1MatchRes.captured(4) == "warning")
             errorItem.severity = Severity::SeverityType::warning;
-        else if (r1.cap(4) == "error" || r1.cap(4) == "fatal error")
+        else if (r1MatchRes.captured(4) == "error" || r1MatchRes.captured(4) == "fatal error")
             errorItem.severity = Severity::SeverityType::error;
 
         QString message,id;
-        if (r2.exactMatch(r1.cap(5))) {
-            message = r2.cap(1);
-            const QString id1(r2.cap(2));
+        const QRegularExpressionMatch r2MatchRes = r2.match(r1MatchRes.captured(5));
+        if (r2MatchRes.hasMatch()) {
+            message = r2MatchRes.captured(1);
+            const QString id1(r2MatchRes.captured(2));
             if (id1.startsWith("clang"))
                 id = id1;
             else
-                id = tool + '-' + r2.cap(2);
+                id = tool + '-' + r2MatchRes.captured(2);
             if (tool == CLANG_TIDY) {
                 if (id1.startsWith("performance"))
                     errorItem.severity = Severity::SeverityType::performance;
                 else if (id1.startsWith("portability"))
                     errorItem.severity = Severity::SeverityType::portability;
-                else if (id1.startsWith("cert") || (id1.startsWith("misc") && !id1.contains("unused")))
+                else if (id1.startsWith("misc") && !id1.contains("unused"))
                     errorItem.severity = Severity::SeverityType::warning;
                 else
                     errorItem.severity = Severity::SeverityType::style;
             }
         } else {
-            message = r1.cap(5);
+            message = r1MatchRes.captured(5);
             id = CLANG_ANALYZER;
         }
 
@@ -466,7 +384,7 @@ void CheckThread::parseClangErrors(const QString &tool, const QString &file0, QS
     }
     errorItems.append(errorItem);
 
-    foreach (const ErrorItem &e, errorItems) {
+    for (const ErrorItem &e : errorItems) {
         if (e.errorPath.isEmpty())
             continue;
         Suppressions::ErrorMessage errorMessage;
@@ -478,21 +396,21 @@ void CheckThread::parseClangErrors(const QString &tool, const QString &file0, QS
         if (isSuppressed(errorMessage))
             continue;
 
-        std::list<ErrorLogger::ErrorMessage::FileLocation> callstack;
-        foreach (const QErrorPathItem &path, e.errorPath) {
-            callstack.push_back(ErrorLogger::ErrorMessage::FileLocation(path.file.toStdString(), path.info.toStdString(), path.line, path.column));
+        std::list<ErrorMessage::FileLocation> callstack;
+        for (const QErrorPathItem &path : e.errorPath) {
+            callstack.push_back(ErrorMessage::FileLocation(path.file.toStdString(), path.info.toStdString(), path.line, path.column));
         }
         const std::string f0 = file0.toStdString();
         const std::string msg = e.message.toStdString();
         const std::string id = e.errorId.toStdString();
-        ErrorLogger::ErrorMessage errmsg(callstack, f0, e.severity, msg, id, false);
+        ErrorMessage errmsg(callstack, f0, e.severity, msg, id, Certainty::normal);
         mResult.reportErr(errmsg);
     }
 }
 
 bool CheckThread::isSuppressed(const Suppressions::ErrorMessage &errorMessage) const
 {
-    foreach (const Suppressions::Suppression &suppression, mSuppressions) {
+    for (const Suppressions::Suppression &suppression : mSuppressions) {
         if (suppression.isSuppressed(errorMessage))
             return true;
     }
@@ -548,45 +466,3 @@ QString CheckThread::clangTidyCmd()
 
     return QString();
 }
-
-QString CheckThread::pythonCmd()
-{
-    QString path = QSettings().value(SETTINGS_PYTHON_PATH).toString();
-    if (!path.isEmpty())
-        return path;
-
-    path = "python";
-#ifdef Q_OS_WIN
-    path += ".exe";
-#endif
-
-    QProcess process;
-    process.start(path, QStringList() << "--version");
-    process.waitForFinished();
-    if (process.exitCode() == 0)
-        return path;
-
-    return QString();
-}
-
-QString CheckThread::getAddonFilePath(const QString &dataDir, const QString &addonFile)
-{
-    const QStringList paths = QStringList() << "/" << "/addons/" << "/../addons/";
-
-    if (!dataDir.isEmpty()) {
-        foreach (const QString p, paths) {
-            const QString filePath(dataDir + p + addonFile);
-            if (QFileInfo(filePath).exists())
-                return filePath;
-        }
-    }
-
-    const QString appPath = QApplication::applicationDirPath();
-    foreach (const QString p, paths) {
-        const QString filePath(appPath + p + addonFile);
-        if (QFileInfo(filePath).exists())
-            return filePath;
-    }
-
-    return QString();
-}
