@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2020 Cppcheck team.
+ * Copyright (C) 2007-2022 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,31 +19,44 @@
 #ifndef PROJECT_FILE_H
 #define PROJECT_FILE_H
 
+#include "settings.h"
+#include "suppressions.h"
+
+#include <cstddef>
 #include <map>
+
 #include <QObject>
 #include <QString>
 #include <QStringList>
-#include <QXmlStreamReader>
 
-#include "suppressions.h"
-
-#include <settings.h>
+class QXmlStreamReader;
+class QXmlStreamWriter;
 
 /// @addtogroup GUI
 /// @{
 
 
 /**
-* @brief A class that reads and writes project files.
-* The project files contain project-specific settings for checking. For
-* example a list of include paths.
-*/
+ * @brief A class that reads and writes project files.
+ * The project files contain project-specific settings for checking. For
+ * example a list of include paths.
+ */
 class ProjectFile : public QObject {
     Q_OBJECT
 
 public:
     explicit ProjectFile(QObject *parent = nullptr);
     explicit ProjectFile(const QString &filename, QObject *parent = nullptr);
+    ~ProjectFile() override {
+        if (this == mActiveProject) mActiveProject = nullptr;
+    }
+
+    static ProjectFile* getActiveProject() {
+        return mActiveProject;
+    }
+    void setActiveProject() {
+        mActiveProject = this;
+    }
 
     /**
      * @brief Read the project file.
@@ -88,57 +101,57 @@ public:
     }
 
     /**
-    * @brief Get list of include directories.
-    * @return list of directories.
-    */
+     * @brief Get list of include directories.
+     * @return list of directories.
+     */
     QStringList getIncludeDirs() const {
         return ProjectFile::fromNativeSeparators(mIncludeDirs);
     }
 
     /**
-    * @brief Get list of defines.
-    * @return list of defines.
-    */
+     * @brief Get list of defines.
+     * @return list of defines.
+     */
     QStringList getDefines() const {
         return mDefines;
     }
 
     /**
-    * @brief Get list of undefines.
-    * @return list of undefines.
-    */
+     * @brief Get list of undefines.
+     * @return list of undefines.
+     */
     QStringList getUndefines() const {
         return mUndefines;
     }
 
     /**
-    * @brief Get list of paths to check.
-    * @return list of paths.
-    */
+     * @brief Get list of paths to check.
+     * @return list of paths.
+     */
     QStringList getCheckPaths() const {
         return ProjectFile::fromNativeSeparators(mPaths);
     }
 
     /**
-    * @brief Get list of paths to exclude from the check.
-    * @return list of paths.
-    */
+     * @brief Get list of paths to exclude from the check.
+     * @return list of paths.
+     */
     QStringList getExcludedPaths() const {
         return ProjectFile::fromNativeSeparators(mExcludedPaths);
     }
 
     /**
-    * @brief Get list of paths to exclude from the check.
-    * @return list of paths.
-    */
+     * @brief Get list of paths to exclude from the check.
+     * @return list of paths.
+     */
     QStringList getVsConfigurations() const {
         return mVsConfigurations;
     }
 
     /**
-    * @brief Get list libraries.
-    * @return list of libraries.
-    */
+     * @brief Get list libraries.
+     * @return list of libraries.
+     */
     QStringList getLibraries() const {
         return mLibraries;
     }
@@ -152,25 +165,32 @@ public:
     }
 
     /**
-    * @brief Get "raw" suppressions.
-    * @return list of suppressions.
-    */
+     * @brief Get "raw" suppressions.
+     * @return list of suppressions.
+     */
     QList<Suppressions::Suppression> getSuppressions() const {
         return mSuppressions;
     }
 
     /**
-    * @brief Get list addons.
-    * @return list of addons.
-    */
+     * @brief Get list addons.
+     * @return list of addons.
+     */
     QStringList getAddons() const {
         return mAddons;
     }
 
     /**
-    * @brief Get list of addons and tools.
-    * @return list of addons and tools.
-    */
+     * @brief Get path to addon python script
+     * @param filesDir Data files folder set by --data-dir
+     * @param addon addon i.e. "misra" to lookup
+     */
+    static QString getAddonFilePath(QString filesDir, const QString &addon);
+
+    /**
+     * @brief Get list of addons and tools.
+     * @return list of addons and tools.
+     */
     QStringList getAddonsAndTools() const;
 
     bool getClangAnalyzer() const {
@@ -201,22 +221,26 @@ public:
         mMaxCtuDepth = maxCtuDepth;
     }
 
-    const std::map<std::string,std::string>& getFunctionContracts() const {
-        return mFunctionContracts;
+    int getMaxTemplateRecursion() const {
+        return mMaxTemplateRecursion;
+    }
+
+    void setMaxTemplateRecursion(int maxTemplateRecursion) {
+        mMaxTemplateRecursion = maxTemplateRecursion;
     }
 
     /**
-    * @brief Get filename for the project file.
-    * @return file name.
-    */
+     * @brief Get filename for the project file.
+     * @return file name.
+     */
     QString getFilename() const {
         return mFilename;
     }
 
     /**
-    * @brief Set project root path.
-    * @param rootpath new project root path.
-    */
+     * @brief Set project root path.
+     * @param rootpath new project root path.
+     */
     void setRootPath(const QString &rootpath) {
         mRootPath = rootpath;
     }
@@ -269,9 +293,6 @@ public:
      */
     void setLibraries(const QStringList &libraries);
 
-    /** Set contract for a function */
-    void setFunctionContract(QString function, QString expects);
-
     /**
      * @brief Set platform.
      * @param platform platform.
@@ -284,6 +305,9 @@ public:
      */
     void setSuppressions(const QList<Suppressions::Suppression> &suppressions);
 
+    /** Add suppression */
+    void addSuppression(const Suppressions::Suppression &suppression);
+
     /**
      * @brief Set list of addons.
      * @param addons List of addons.
@@ -303,6 +327,12 @@ public:
         mTags = tags;
     }
 
+    /** Set tags for a warning */
+    void setWarningTags(std::size_t hash, const QString& tags);
+
+    /** Get tags for a warning */
+    QString getWarningTags(std::size_t hash) const;
+
     /**
      * @brief Write project file (to disk).
      * @param filename Filename to use.
@@ -339,8 +369,6 @@ public:
     /** Use Clang parser */
     bool clangParser;
 
-    /** Bug hunting */
-    bool bugHunting;
 protected:
 
     /**
@@ -385,12 +413,6 @@ protected:
      */
     void readExcludes(QXmlStreamReader &reader);
 
-    /**
-     * @brief Read function contracts.
-     * @param reader XML stream reader.
-     */
-    void readFunctionContracts(QXmlStreamReader &reader);
-
     /**
      * @brief Read lists of Visual Studio configurations
      * @param reader XML stream reader.
@@ -410,11 +432,17 @@ protected:
     void readSuppressions(QXmlStreamReader &reader);
 
     /**
-      * @brief Read string list
-      * @param stringlist   destination string list
-      * @param reader       XML stream reader
-      * @param elementname  elementname for each string
-      */
+     * @brief Read tag warnings, what warnings are tagged with a specific tag
+     * @param reader XML stream reader.
+     */
+    void readTagWarnings(QXmlStreamReader &reader, const QString &tag);
+
+    /**
+     * @brief Read string list
+     * @param stringlist   destination string list
+     * @param reader       XML stream reader
+     * @param elementname  elementname for each string
+     */
     void readStringList(QStringList &stringlist, QXmlStreamReader &reader, const char elementname[]);
 
     /**
@@ -500,8 +528,6 @@ private:
      */
     QStringList mLibraries;
 
-    std::map<std::string, std::string> mFunctionContracts;
-
     /**
      * @brief Platform
      */
@@ -524,15 +550,24 @@ private:
     bool mClangTidy;
 
     /**
-     * @brief Warning tags
+     * @brief Tags
      */
     QStringList mTags;
 
+    /**
+     * @brief Warning tags
+     */
+    std::map<std::size_t, QString> mWarningTags;
+
     /** Max CTU depth */
     int mMaxCtuDepth;
 
+    /** Max template instantiation recursion */
+    int mMaxTemplateRecursion;
+
     QStringList mCheckUnknownFunctionReturn;
 
+    static ProjectFile *mActiveProject;
 };
 /// @}
 #endif  // PROJECT_FILE_H
