@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2020 Cppcheck team.
+ * Copyright (C) 2007-2022 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,8 +19,14 @@
 //---------------------------------------------------------------------------
 
 #include "check.h"
-#include "settings.h"
 
+#include "errorlogger.h"
+#include "settings.h"
+#include "token.h"
+#include "tokenize.h"
+#include "valueflow.h"
+
+#include <cctype>
 #include <iostream>
 
 //---------------------------------------------------------------------------
@@ -37,22 +43,35 @@ Check::Check(const std::string &aname)
     instances().push_back(this);
 }
 
-void Check::reportError(const ErrorLogger::ErrorMessage &errmsg)
+void Check::reportError(const ErrorMessage &errmsg)
 {
     std::cout << errmsg.toXML() << std::endl;
 }
 
-bool Check::wrongData(const Token *tok, bool condition, const char *str)
+
+void Check::reportError(const std::list<const Token *> &callstack, Severity::SeverityType severity, const std::string &id, const std::string &msg, const CWE &cwe, Certainty::CertaintyLevel certainty)
 {
-#if defined(DACA2) || defined(UNSTABLE)
-    if (condition) {
+    const ErrorMessage errmsg(callstack, mTokenizer ? &mTokenizer->list : nullptr, severity, id, msg, cwe, certainty);
+    if (mErrorLogger)
+        mErrorLogger->reportErr(errmsg);
+    else
+        reportError(errmsg);
+}
+
+void Check::reportError(const ErrorPath &errorPath, Severity::SeverityType severity, const char id[], const std::string &msg, const CWE &cwe, Certainty::CertaintyLevel certainty)
+{
+    const ErrorMessage errmsg(errorPath, mTokenizer ? &mTokenizer->list : nullptr, severity, id, msg, cwe, certainty);
+    if (mErrorLogger)
+        mErrorLogger->reportErr(errmsg);
+    else
+        reportError(errmsg);
+}
+
+bool Check::wrongData(const Token *tok, const char *str)
+{
+    if (mSettings->daca)
         reportError(tok, Severity::debug, "DacaWrongData", "Wrong data detected by condition " + std::string(str));
-    }
-#else
-    (void)tok;
-    (void)str;
-#endif
-    return condition;
+    return true;
 }
 
 std::list<Check *> &Check::instances()
@@ -93,4 +112,4 @@ ErrorPath Check::getErrorPath(const Token* errtok, const ValueFlow::Value* value
         errorPath.emplace_back(errtok, bug);
     }
     return errorPath;
-}
\ No newline at end of file
+}
