@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2020 Cppcheck team.
+ * Copyright (C) 2007-2022 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,18 +25,18 @@
 #include "astutils.h"
 #include "checkmemoryleak.h"  // <- CheckMemoryLeak::memoryLeak
 #include "checknullpointer.h" // <- CheckNullPointer::isPointerDeRef
-#include "errorlogger.h"
 #include "mathlib.h"
 #include "settings.h"
+#include "errortypes.h"
 #include "symboldatabase.h"
 #include "token.h"
 #include "tokenize.h"
 
-#include <cstddef>
 #include <iostream>
 #include <list>
-#include <stack>
+#include <memory>
 #include <utility>
+#include <vector>
 
 //---------------------------------------------------------------------------
 
@@ -52,10 +52,12 @@ static const CWE CWE415(415U);
 static const int NEW_ARRAY = -2;
 static const int NEW = -1;
 
+static const std::vector<std::pair<std::string, std::string>> alloc_failed_conds {{"==", "0"}, {"<", "0"}, {"==", "-1"}, {"<=", "-1"}};
+static const std::vector<std::pair<std::string, std::string>> alloc_success_conds {{"!=", "0"}, {">", "0"}, {"!=", "-1"}, {">=", "0"}};
 
 /**
  * @brief Is variable type some class with automatic deallocation?
- * @param vartok variable token
+ * @param var variable token
  * @return true unless it can be seen there is no automatic deallocation
  */
 static bool isAutoDealloc(const Variable *var)
@@ -75,6 +77,16 @@ static bool isAutoDealloc(const Variable *var)
     return true;
 }
 
+static bool isVarTokComparison(const Token * tok, const Token ** vartok,
+                               const std::vector<std::pair<std::string, std::string>>& ops)
+{
+    for (const auto & op : ops) {
+        if (astIsVariableComparison(tok, op.first, op.second, vartok))
+            return true;
+    }
+    return false;
+}
+
 //---------------------------------------------------------------------------
 
 void VarInfo::print()
@@ -101,6 +113,9 @@ void VarInfo::print()
         case NOALLOC:
             status = "noalloc";
             break;
+        case REALLOC:
+            status = "realloc";
+            break;
         default:
             status = "?";
             break;
@@ -111,6 +126,7 @@ void VarInfo::print()
                   << "possibleUsage='" << strusage << "' "
                   << "conditionalAlloc=" << (conditionalAlloc.find(it->first) != conditionalAlloc.end() ? "yes" : "no") << " "
                   << "referenced=" << (referenced.find(it->first) != referenced.end() ? "yes" : "no") << " "
+                  << "reallocedFrom=" << it->second.reallocedFromType
                   << std::endl;
     }
 }
@@ -148,12 +164,12 @@ void CheckLeakAutoVar::deallocUseError(const Token *tok, const std::string &varn
 void CheckLeakAutoVar::deallocReturnError(const Token *tok, const Token *deallocTok, const std::string &varname)
 {
     const std::list<const Token *> locations = { deallocTok, tok };
-    reportError(locations, Severity::error, "deallocret", "$symbol:" + varname + "\nReturning/dereferencing '$symbol' after it is deallocated / released", CWE672, false);
+    reportError(locations, Severity::error, "deallocret", "$symbol:" + varname + "\nReturning/dereferencing '$symbol' after it is deallocated / released", CWE672, Certainty::normal);
 }
 
 void CheckLeakAutoVar::configurationInfo(const Token* tok, const std::string &functionName)
 {
-    if (mSettings->checkLibrary && mSettings->isEnabled(Settings::INFORMATION)) {
+    if (mSettings->checkLibrary && mSettings->severity.isEnabled(Severity::information)) {
         reportError(tok,
                     Severity::information,
                     "checkLibraryUseIgnore",
@@ -166,9 +182,9 @@ void CheckLeakAutoVar::doubleFreeError(const Token *tok, const Token *prevFreeTo
     const std::list<const Token *> locations = { prevFreeTok, tok };
 
     if (Library::isresource(type))
-        reportError(locations, Severity::error, "doubleFree", "$symbol:" + varname + "\nResource handle '$symbol' freed twice.", CWE415, false);
+        reportError(locations, Severity::error, "doubleFree", "$symbol:" + varname + "\nResource handle '$symbol' freed twice.", CWE415, Certainty::normal);
     else
-        reportError(locations, Severity::error, "doubleFree", "$symbol:" + varname + "\nMemory pointed to by '$symbol' is freed twice.", CWE415, false);
+        reportError(locations, Severity::error, "doubleFree", "$symbol:" + varname + "\nMemory pointed to by '$symbol' is freed twice.", CWE415, Certainty::normal);
 }
 
 
@@ -191,22 +207,6 @@ void CheckLeakAutoVar::check()
         VarInfo varInfo;
 
         checkScope(scope->bodyStart, &varInfo, notzero, 0);
-
-        varInfo.conditionalAlloc.clear();
-
-        // Clear reference arguments from varInfo..
-        std::map<int, VarInfo::AllocInfo>::iterator it = varInfo.alloctype.begin();
-        while (it != varInfo.alloctype.end()) {
-            const Variable *var = symbolDatabase->getVariableFromVarId(it->first);
-            if (!var ||
-                (var->isArgument() && var->isReference()) ||
-                (!var->isArgument() && !var->isLocal()))
-                varInfo.alloctype.erase(it++);
-            else
-                ++it;
-        }
-
-        ret(scope->bodyEnd, varInfo);
     }
 }
 
@@ -259,12 +259,12 @@ static bool isLocalVarNoAutoDealloc(const Token *varTok, const bool isCpp)
 }
 
 /** checks if nameToken is a name of a function in a function call:
-*     func(arg)
-* or
-*     func<temp1_arg>(arg)
-* @param nameToken Function name token
-* @return opening parenthesis token or NULL if not a function call
-*/
+ *     func(arg)
+ * or
+ *     func<temp1_arg>(arg)
+ * @param nameToken Function name token
+ * @return opening parenthesis token or NULL if not a function call
+ */
 
 static const Token * isFunctionCall(const Token * nameToken)
 {
@@ -328,6 +328,14 @@ void CheckLeakAutoVar::checkScope(const Token * const startToken,
         if (!tok || tok == endToken)
             break;
 
+        if (Token::Match(tok, "const %type%"))
+            tok = tok->tokAt(2);
+
+        while (tok->str() == "(")
+            tok = tok->next();
+        while (tok->isUnaryOp("*") && tok->astOperand1()->isUnaryOp("&"))
+            tok = tok->astOperand1()->astOperand1();
+
         // parse statement, skip to last member
         const Token *varTok = tok;
         while (Token::Match(varTok, "%name% ::|. %name% !!("))
@@ -339,9 +347,22 @@ void CheckLeakAutoVar::checkScope(const Token * const startToken,
         while (Token::Match(ftok, "%name% :: %name%"))
             ftok = ftok->tokAt(2);
 
+        auto isAssignment = [](const Token* varTok) -> const Token* {
+            if (varTok->varId()) {
+                const Token* top = varTok;
+                while (top->astParent()) {
+                    top = top->astParent();
+                    if (!Token::Match(top, "(|*|&|."))
+                        break;
+                }
+                if (top->str() == "=" && succeeds(top, varTok))
+                    return top;
+            }
+            return nullptr;
+        };
+
         // assignment..
-        if (Token::Match(varTok, "%var% =")) {
-            const Token* const tokAssignOp = varTok->next();
+        if (const Token* const tokAssignOp = isAssignment(varTok)) {
 
             // taking address of another variable..
             if (Token::Match(tokAssignOp, "= %var% [+;]")) {
@@ -463,57 +484,49 @@ void CheckLeakAutoVar::checkScope(const Token * const startToken,
                 VarInfo varInfo1(*varInfo);  // VarInfo for if code
                 VarInfo varInfo2(*varInfo);  // VarInfo for else code
 
+                // Skip expressions before commas
+                const Token * astOperand2AfterCommas = tok->next()->astOperand2();
+                while (Token::simpleMatch(astOperand2AfterCommas, ","))
+                    astOperand2AfterCommas = astOperand2AfterCommas->astOperand2();
+
                 // Recursively scan variable comparisons in condition
-                std::stack<const Token *> tokens;
-                tokens.push(tok->next()->astOperand2());
-                while (!tokens.empty()) {
-                    const Token *tok3 = tokens.top();
-                    tokens.pop();
+                visitAstNodes(astOperand2AfterCommas, [&](const Token *tok3) {
                     if (!tok3)
-                        continue;
+                        return ChildrenToVisit::none;
                     if (tok3->str() == "&&" || tok3->str() == "||") {
                         // FIXME: handle && ! || better
-                        tokens.push(tok3->astOperand1());
-                        tokens.push(tok3->astOperand2());
-                        continue;
+                        return ChildrenToVisit::op1_and_op2;
                     }
                     if (tok3->str() == "(" && Token::Match(tok3->astOperand1(), "UNLIKELY|LIKELY")) {
-                        tokens.push(tok3->astOperand2());
-                        continue;
+                        return ChildrenToVisit::op2;
                     } else if (tok3->str() == "(" && Token::Match(tok3->previous(), "%name%")) {
                         const std::vector<const Token *> params = getArguments(tok3->previous());
                         for (const Token *par : params) {
                             if (!par->isComparisonOp())
                                 continue;
                             const Token *vartok = nullptr;
-                            if (astIsVariableComparison(par, "!=", "0", &vartok) ||
-                                astIsVariableComparison(par, "==", "0", &vartok) ||
-                                astIsVariableComparison(par, "<", "0", &vartok) ||
-                                astIsVariableComparison(par, ">", "0", &vartok) ||
-                                astIsVariableComparison(par, "==", "-1", &vartok) ||
-                                astIsVariableComparison(par, "!=", "-1", &vartok)) {
+                            if (isVarTokComparison(par, &vartok, alloc_success_conds) ||
+                                (isVarTokComparison(par, &vartok, alloc_failed_conds))) {
                                 varInfo1.erase(vartok->varId());
                                 varInfo2.erase(vartok->varId());
                             }
                         }
-                        continue;
+                        return ChildrenToVisit::none;
                     }
 
                     const Token *vartok = nullptr;
-                    if (astIsVariableComparison(tok3, "!=", "0", &vartok)) {
+                    if (isVarTokComparison(tok3, &vartok, alloc_success_conds)) {
+                        varInfo2.reallocToAlloc(vartok->varId());
                         varInfo2.erase(vartok->varId());
-                        if (notzero.find(vartok->varId()) != notzero.end())
+                        if (astIsVariableComparison(tok3, "!=", "0", &vartok) &&
+                            (notzero.find(vartok->varId()) != notzero.end()))
                             varInfo2.clear();
-                    } else if (astIsVariableComparison(tok3, "==", "0", &vartok)) {
-                        varInfo1.erase(vartok->varId());
-                    } else if (astIsVariableComparison(tok3, "<", "0", &vartok)) {
-                        varInfo1.erase(vartok->varId());
-                    } else if (astIsVariableComparison(tok3, ">", "0", &vartok)) {
-                        varInfo2.erase(vartok->varId());
-                    } else if (astIsVariableComparison(tok3, "==", "-1", &vartok)) {
+                    } else if (isVarTokComparison(tok3, &vartok, alloc_failed_conds)) {
+                        varInfo1.reallocToAlloc(vartok->varId());
                         varInfo1.erase(vartok->varId());
                     }
-                }
+                    return ChildrenToVisit::none;
+                });
 
                 checkScope(closingParenthesis->next(), &varInfo1, notzero, recursiveCount);
                 closingParenthesis = closingParenthesis->linkAt(1);
@@ -637,7 +650,7 @@ void CheckLeakAutoVar::checkScope(const Token * const startToken,
 
             // Handle scopes that might be noreturn
             if (allocation.status == VarInfo::NOALLOC && Token::simpleMatch(tok, ") ; }")) {
-                const std::string &functionName(tok->link()->previous()->str());
+                const std::string functionName(mSettings->library.getFunctionName(tok->link()->previous()));
                 bool unknown = false;
                 if (mTokenizer->isScopeNoReturn(tok->tokAt(2), &unknown)) {
                     if (!unknown)
@@ -730,6 +743,7 @@ void CheckLeakAutoVar::checkScope(const Token * const startToken,
             changeAllocStatus(varInfo, allocation, vtok, vtok);
         }
     }
+    ret(endToken, *varInfo, true);
 }
 
 
@@ -753,10 +767,13 @@ const Token * CheckLeakAutoVar::checkTokenInsideExpression(const Token * const t
                         break;
                     rhs = rhs->astParent();
                 }
+                while (rhs->isCast()) {
+                    rhs = rhs->astOperand1();
+                }
                 if (rhs->varId() == tok->varId()) {
                     // simple assignment
                     varInfo->erase(tok->varId());
-                } else if (rhs->str() == "(" && mSettings->library.returnValue(rhs->astOperand1()) != emptyString) {
+                } else if (rhs->str() == "(" && !mSettings->library.returnValue(rhs->astOperand1()).empty()) {
                     // #9298, assignment through return value of a function
                     const std::string &returnValue = mSettings->library.returnValue(rhs->astOperand1());
                     if (returnValue.compare(0, 3, "arg") == 0) {
@@ -800,15 +817,18 @@ void CheckLeakAutoVar::changeAllocStatusIfRealloc(std::map<int, VarInfo::AllocIn
     const Library::AllocFunc* f = mSettings->library.getReallocFuncInfo(fTok);
     if (f && f->arg == -1 && f->reallocArg > 0 && f->reallocArg <= numberOfArguments(fTok)) {
         const Token* argTok = getArguments(fTok).at(f->reallocArg - 1);
-        VarInfo::AllocInfo& argAlloc = alloctype[argTok->varId()];
+        if (alloctype.find(argTok->varId()) != alloctype.end()) {
+            VarInfo::AllocInfo& argAlloc = alloctype[argTok->varId()];
+            if (argAlloc.type != 0 && argAlloc.type != f->groupId)
+                mismatchError(fTok, argAlloc.allocTok, argTok->str());
+            argAlloc.status = VarInfo::REALLOC;
+            argAlloc.allocTok = fTok;
+        }
         VarInfo::AllocInfo& retAlloc = alloctype[retTok->varId()];
-        if (argAlloc.type != 0 && argAlloc.type != f->groupId)
-            mismatchError(fTok, argAlloc.allocTok, argTok->str());
-        argAlloc.status = VarInfo::DEALLOC;
-        argAlloc.allocTok = fTok;
         retAlloc.type = f->groupId;
         retAlloc.status = VarInfo::ALLOC;
         retAlloc.allocTok = fTok;
+        retAlloc.reallocedFromType = argTok->varId();
     }
 }
 
@@ -825,7 +845,8 @@ void CheckLeakAutoVar::changeAllocStatus(VarInfo *varInfo, const VarInfo::AllocI
                 varInfo->erase(arg->varId());
         } else if (var->second.managed()) {
             doubleFreeError(tok, var->second.allocTok, arg->str(), allocation.type);
-        } else if (var->second.type != allocation.type) {
+            var->second.status = allocation.status;
+        } else if (var->second.type != allocation.type && var->second.type != 0) {
             // mismatching allocation and deallocation
             mismatchError(tok, var->second.allocTok, arg->str());
             varInfo->erase(arg->varId());
@@ -835,7 +856,7 @@ void CheckLeakAutoVar::changeAllocStatus(VarInfo *varInfo, const VarInfo::AllocI
             var->second.type = allocation.type;
             var->second.allocTok = allocation.allocTok;
         }
-    } else if (allocation.status != VarInfo::NOALLOC) {
+    } else if (allocation.status != VarInfo::NOALLOC && allocation.status != VarInfo::OWNED && !Token::simpleMatch(tok->astTop(), "return")) {
         alloctype[arg->varId()].status = VarInfo::DEALLOC;
         alloctype[arg->varId()].allocTok = tok;
     }
@@ -844,7 +865,7 @@ void CheckLeakAutoVar::changeAllocStatus(VarInfo *varInfo, const VarInfo::AllocI
 void CheckLeakAutoVar::functionCall(const Token *tokName, const Token *tokOpeningPar, VarInfo *varInfo, const VarInfo::AllocInfo& allocation, const Library::AllocFunc* af)
 {
     // Ignore function call?
-    if (mSettings->library.isLeakIgnore(tokName->str()))
+    if (mSettings->library.isLeakIgnore(mSettings->library.getFunctionName(tokName)))
         return;
     if (mSettings->library.getReallocFuncInfo(tokName))
         return;
@@ -856,11 +877,19 @@ void CheckLeakAutoVar::functionCall(const Token *tokName, const Token *tokOpenin
     }
 
     int argNr = 1;
-    for (const Token *arg = tokFirstArg; arg; arg = arg->nextArgument()) {
-        if (mTokenizer->isCPP() && arg->str() == "new") {
-            arg = arg->next();
-            if (Token::simpleMatch(arg, "( std :: nothrow )"))
-                arg = arg->tokAt(5);
+    for (const Token *funcArg = tokFirstArg; funcArg; funcArg = funcArg->nextArgument()) {
+        const Token* arg = funcArg;
+        if (mTokenizer->isCPP()) {
+            int tokAdvance = 0;
+            if (arg->str() == "new")
+                tokAdvance = 1;
+            else if (Token::simpleMatch(arg, "* new"))
+                tokAdvance = 2;
+            if (tokAdvance > 0) {
+                arg = arg->tokAt(tokAdvance);
+                if (Token::simpleMatch(arg, "( std :: nothrow )"))
+                    arg = arg->tokAt(5);
+            }
         }
 
         // Skip casts
@@ -954,15 +983,16 @@ void CheckLeakAutoVar::leakIfAllocated(const Token *vartok,
     }
 }
 
-void CheckLeakAutoVar::ret(const Token *tok, const VarInfo &varInfo)
+void CheckLeakAutoVar::ret(const Token *tok, VarInfo &varInfo, const bool isEndOfScope)
 {
     const std::map<int, VarInfo::AllocInfo> &alloctype = varInfo.alloctype;
     const std::map<int, std::string> &possibleUsage = varInfo.possibleUsage;
+    std::vector<int> toRemove;
 
     const SymbolDatabase *symbolDatabase = mTokenizer->getSymbolDatabase();
     for (std::map<int, VarInfo::AllocInfo>::const_iterator it = alloctype.begin(); it != alloctype.end(); ++it) {
-        // don't warn if variable is conditionally allocated
-        if (!it->second.managed() && varInfo.conditionalAlloc.find(it->first) != varInfo.conditionalAlloc.end())
+        // don't warn if variable is conditionally allocated, unless it leaves the scope
+        if (!isEndOfScope && !it->second.managed() && varInfo.conditionalAlloc.find(it->first) != varInfo.conditionalAlloc.end())
             continue;
 
         // don't warn if there is a reference of the variable
@@ -972,6 +1002,9 @@ void CheckLeakAutoVar::ret(const Token *tok, const VarInfo &varInfo)
         const int varid = it->first;
         const Variable *var = symbolDatabase->getVariableFromVarId(varid);
         if (var) {
+            // don't warn if we leave an inner scope
+            if (isEndOfScope && var->scope() && tok != var->scope()->bodyEnd)
+                continue;
             bool used = false;
             for (const Token *tok2 = tok; tok2; tok2 = tok2->next()) {
                 if (tok2->str() == ";")
@@ -991,7 +1024,7 @@ void CheckLeakAutoVar::ret(const Token *tok, const VarInfo &varInfo)
                     tok2 = tok3->tokAt(4);
                 else
                     continue;
-                if (Token::Match(tok2, "[});,]")) {
+                if (Token::Match(tok2, "[});,+]")) {
                     used = true;
                     break;
                 }
@@ -1009,6 +1042,9 @@ void CheckLeakAutoVar::ret(const Token *tok, const VarInfo &varInfo)
                     configurationInfo(tok, use->second);
                 }
             }
+            toRemove.push_back(varid);
         }
     }
+    for (int varId : toRemove)
+        varInfo.erase(varId);
 }
