@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2016 Cppcheck team.
+ * Copyright (C) 2007-2022 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,10 +19,15 @@
 
 //---------------------------------------------------------------------------
 #include "checksizeof.h"
-#include "symboldatabase.h"
-#include <algorithm>
-#include <cctype>
 
+#include "errortypes.h"
+#include "settings.h"
+#include "symboldatabase.h"
+#include "token.h"
+#include "tokenize.h"
+
+#include <map>
+#include <vector>
 
 //---------------------------------------------------------------------------
 
@@ -32,20 +37,19 @@ namespace {
 }
 
 // CWE IDs used:
-static const struct CWE CWE398(398U);	// Indicator of Poor Code Quality
-
+static const struct CWE CWE398(398U);   // Indicator of Poor Code Quality
+static const struct CWE CWE467(467U);   // Use of sizeof() on a Pointer Type
+static const struct CWE CWE682(682U);   // Incorrect Calculation
 //---------------------------------------------------------------------------
 //---------------------------------------------------------------------------
 void CheckSizeof::checkSizeofForNumericParameter()
 {
-    if (!_settings->isEnabled("warning"))
+    if (!mSettings->severity.isEnabled(Severity::warning))
         return;
 
-    const SymbolDatabase *symbolDatabase = _tokenizer->getSymbolDatabase();
-    const std::size_t functions = symbolDatabase->functionScopes.size();
-    for (std::size_t i = 0; i < functions; ++i) {
-        const Scope * scope = symbolDatabase->functionScopes[i];
-        for (const Token* tok = scope->classStart->next(); tok != scope->classEnd; tok = tok->next()) {
+    const SymbolDatabase *symbolDatabase = mTokenizer->getSymbolDatabase();
+    for (const Scope * scope : symbolDatabase->functionScopes) {
+        for (const Token* tok = scope->bodyStart->next(); tok != scope->bodyEnd; tok = tok->next()) {
             if (Token::Match(tok, "sizeof ( %num% )") ||
                 Token::Match(tok, "sizeof %num%")) {
                 sizeofForNumericParameterError(tok);
@@ -60,7 +64,7 @@ void CheckSizeof::sizeofForNumericParameterError(const Token *tok)
                 "sizeofwithnumericparameter", "Suspicious usage of 'sizeof' with a numeric constant as parameter.\n"
                 "It is unusual to use a constant value with sizeof. For example, 'sizeof(10)'"
                 " returns 4 (in 32-bit systems) or 8 (in 64-bit systems) instead of 10. 'sizeof('A')'"
-                " and 'sizeof(char)' can return different results.");
+                " and 'sizeof(char)' can return different results.", CWE682, Certainty::normal);
 }
 
 
@@ -68,13 +72,11 @@ void CheckSizeof::sizeofForNumericParameterError(const Token *tok)
 //---------------------------------------------------------------------------
 void CheckSizeof::checkSizeofForArrayParameter()
 {
-    if (!_settings->isEnabled("warning"))
+    if (!mSettings->severity.isEnabled(Severity::warning))
         return;
-    const SymbolDatabase *symbolDatabase = _tokenizer->getSymbolDatabase();
-    const std::size_t functions = symbolDatabase->functionScopes.size();
-    for (std::size_t i = 0; i < functions; ++i) {
-        const Scope * scope = symbolDatabase->functionScopes[i];
-        for (const Token* tok = scope->classStart->next(); tok != scope->classEnd; tok = tok->next()) {
+    const SymbolDatabase *symbolDatabase = mTokenizer->getSymbolDatabase();
+    for (const Scope * scope : symbolDatabase->functionScopes) {
+        for (const Token* tok = scope->bodyStart->next(); tok != scope->bodyEnd; tok = tok->next()) {
             if (Token::Match(tok, "sizeof ( %var% )") ||
                 Token::Match(tok, "sizeof %var% !![")) {
                 const Token* varTok = tok->next();
@@ -102,20 +104,18 @@ void CheckSizeof::sizeofForArrayParameterError(const Token *tok)
                 "         return sizeof(a);\n"
                 "     }\n"
                 "returns 4 (in 32-bit systems) or 8 (in 64-bit systems) instead of 100 (the "
-                "size of the array in bytes)."
-               );
+                "size of the array in bytes).", CWE467, Certainty::normal
+                );
 }
 
 void CheckSizeof::checkSizeofForPointerSize()
 {
-    if (!_settings->isEnabled("warning"))
+    if (!mSettings->severity.isEnabled(Severity::warning))
         return;
 
-    const SymbolDatabase *symbolDatabase = _tokenizer->getSymbolDatabase();
-    const std::size_t functions = symbolDatabase->functionScopes.size();
-    for (std::size_t i = 0; i < functions; ++i) {
-        const Scope * scope = symbolDatabase->functionScopes[i];
-        for (const Token* tok = scope->classStart; tok != scope->classEnd; tok = tok->next()) {
+    const SymbolDatabase *symbolDatabase = mTokenizer->getSymbolDatabase();
+    for (const Scope * scope : symbolDatabase->functionScopes) {
+        for (const Token* tok = scope->bodyStart; tok != scope->bodyEnd; tok = tok->next()) {
             const Token* tokSize;
             const Token* tokFunc;
             const Token *variable = nullptr;
@@ -151,13 +151,36 @@ void CheckSizeof::checkSizeofForPointerSize()
                 continue;
             }
 
-            if (tokFunc && tokFunc->str() == "calloc")
+            if (tokSize && tokFunc->str() == "calloc")
                 tokSize = tokSize->nextArgument();
 
-            if (tokFunc && tokSize) {
-                for (const Token* tok2 = tokSize; tok2 != tokFunc->linkAt(1); tok2 = tok2->next()) {
-                    if (Token::simpleMatch(tok2, "/ sizeof"))
+            if (tokSize) {
+                const Token * const paramsListEndTok = tokFunc->linkAt(1);
+                for (const Token* tok2 = tokSize; tok2 != paramsListEndTok; tok2 = tok2->next()) {
+                    if (Token::simpleMatch(tok2, "/ sizeof")) {
+                        // Allow division with sizeof(char)
+                        if (Token::simpleMatch(tok2->next(), "sizeof (")) {
+                            const Token *sztok = tok2->tokAt(2)->astOperand2();
+                            const ValueType *vt = ((sztok != nullptr) ? sztok->valueType() : nullptr);
+                            if (vt && vt->type == ValueType::CHAR && vt->pointer == 0)
+                                continue;
+                        }
+                        auto hasMultiplication = [](const Token* parTok) -> bool {
+                            while (parTok) { // Allow division if followed by multiplication
+                                if (parTok->isArithmeticalOp() && parTok->str() == "*") {
+                                    for (const Token* szTok : { parTok->astOperand1(), parTok->astOperand2() })
+                                        if (Token::simpleMatch(szTok, "(") && Token::simpleMatch(szTok->previous(), "sizeof"))
+                                            return true;
+                                }
+                                parTok = parTok->astParent();
+                            }
+                            return false;
+                        };
+                        if (hasMultiplication(tok2->astParent()))
+                            continue;
+
                         divideBySizeofError(tok2, tokFunc->str());
+                    }
                 }
             }
 
@@ -170,24 +193,27 @@ void CheckSizeof::checkSizeofForPointerSize()
             while (Token::Match(variable2, "%var% ::|."))
                 variable2 = variable2->tokAt(2);
 
+            if (!variable)
+                continue;
+
             // Ensure the variables are in the symbol database
             // Also ensure the variables are pointers
             // Only keep variables which are pointers
             const Variable *var = variable->variable();
             if (!var || !var->isPointer() || var->isArray()) {
-                variable = 0;
+                variable = nullptr;
             }
 
             if (variable2) {
                 var = variable2->variable();
                 if (!var || !var->isPointer() || var->isArray()) {
-                    variable2 = 0;
+                    variable2 = nullptr;
                 }
             }
 
             // If there are no pointer variable at this point, there is
             // no need to continue
-            if (variable == 0 && variable2 == 0) {
+            if (variable == nullptr && variable2 == nullptr) {
                 continue;
             }
 
@@ -235,23 +261,23 @@ void CheckSizeof::sizeofForPointerError(const Token *tok, const std::string &var
                 "Size of pointer '" + varname + "' used instead of size of its data.\n"
                 "Size of pointer '" + varname + "' used instead of size of its data. "
                 "This is likely to lead to a buffer overflow. You probably intend to "
-                "write 'sizeof(*" + varname + ")'.");
+                "write 'sizeof(*" + varname + ")'.", CWE467, Certainty::normal);
 }
 
 void CheckSizeof::divideBySizeofError(const Token *tok, const std::string &memfunc)
 {
     reportError(tok, Severity::warning, "sizeofDivisionMemfunc",
-                "Division by result of sizeof(). " + memfunc + "() expects a size in bytes, did you intend to multiply instead?");
+                "Division by result of sizeof(). " + memfunc + "() expects a size in bytes, did you intend to multiply instead?", CWE682, Certainty::normal);
 }
 
 //-----------------------------------------------------------------------------
 //-----------------------------------------------------------------------------
 void CheckSizeof::sizeofsizeof()
 {
-    if (!_settings->isEnabled("warning"))
+    if (!mSettings->severity.isEnabled(Severity::warning))
         return;
 
-    for (const Token *tok = _tokenizer->tokens(); tok; tok = tok->next()) {
+    for (const Token *tok = mTokenizer->tokens(); tok; tok = tok->next()) {
         if (Token::Match(tok, "sizeof (| sizeof")) {
             sizeofsizeofError(tok);
             tok = tok->next();
@@ -265,19 +291,61 @@ void CheckSizeof::sizeofsizeofError(const Token *tok)
                 "sizeofsizeof", "Calling 'sizeof' on 'sizeof'.\n"
                 "Calling sizeof for 'sizeof looks like a suspicious code and "
                 "most likely there should be just one 'sizeof'. The current "
-                "code is equivalent to 'sizeof(size_t)'");
+                "code is equivalent to 'sizeof(size_t)'", CWE682, Certainty::normal);
 }
 
 //-----------------------------------------------------------------------------
 
 void CheckSizeof::sizeofCalculation()
 {
-    if (!_settings->isEnabled("warning"))
+    if (!mSettings->severity.isEnabled(Severity::warning))
         return;
 
-    const bool printInconclusive = _settings->inconclusive;
+    const bool printInconclusive = mSettings->certainty.isEnabled(Certainty::inconclusive);
 
-    for (const Token *tok = _tokenizer->tokens(); tok; tok = tok->next()) {
+    for (const Token *tok = mTokenizer->tokens(); tok; tok = tok->next()) {
+        if (!Token::simpleMatch(tok, "sizeof ("))
+            continue;
+
+        // ignore if the `sizeof` result is cast to void inside a macro, i.e. the calculation is
+        // expected to be parsed but skipped, such as in a disabled custom ASSERT() macro
+        if (tok->isExpandedMacro() && tok->previous()) {
+            const Token *cast_end = (tok->previous()->str() == "(") ? tok->previous() : tok;
+            if (Token::simpleMatch(cast_end->tokAt(-3), "( void )") ||
+                Token::simpleMatch(cast_end->tokAt(-4), "static_cast < void >")) {
+                continue;
+            }
+        }
+
+        const Token *argument = tok->next()->astOperand2();
+        if (!argument || !argument->isCalculation())
+            continue;
+
+        bool inconclusive = false;
+        if (argument->isExpandedMacro())
+            inconclusive = true;
+        else if (tok->next()->isExpandedMacro())
+            inconclusive = true;
+
+        if (!inconclusive || printInconclusive)
+            sizeofCalculationError(argument, inconclusive);
+    }
+}
+
+void CheckSizeof::sizeofCalculationError(const Token *tok, bool inconclusive)
+{
+    reportError(tok, Severity::warning,
+                "sizeofCalculation", "Found calculation inside sizeof().", CWE682, inconclusive ? Certainty::inconclusive : Certainty::normal);
+}
+
+//-----------------------------------------------------------------------------
+
+void CheckSizeof::sizeofFunction()
+{
+    if (!mSettings->severity.isEnabled(Severity::warning))
+        return;
+
+    for (const Token *tok = mTokenizer->tokens(); tok; tok = tok->next()) {
         if (Token::simpleMatch(tok, "sizeof (")) {
 
             // ignore if the `sizeof` result is cast to void inside a macro, i.e. the calculation is
@@ -285,22 +353,29 @@ void CheckSizeof::sizeofCalculation()
             if (tok->isExpandedMacro() && tok->previous()) {
                 const Token *cast_end = (tok->previous()->str() == "(") ? tok->previous() : tok;
                 if (Token::simpleMatch(cast_end->tokAt(-3), "( void )") ||
-                    Token::simpleMatch(cast_end->tokAt(-1), "static_cast<void>")) {
+                    Token::simpleMatch(cast_end->tokAt(-4), "static_cast < void >")) {
                     continue;
                 }
             }
 
-            const Token *argument = tok->next()->astOperand2();
-            if (argument && argument->isCalculation() && (!argument->isExpandedMacro() || printInconclusive))
-                sizeofCalculationError(argument, argument->isExpandedMacro());
+            if (const Token *argument = tok->next()->astOperand2()) {
+                const Token *checkToken = argument->previous();
+                if (checkToken->tokType() == Token::eName)
+                    break;
+                const Function * fun = checkToken->function();
+                // Don't report error if the function is overloaded
+                if (fun && fun->nestedIn->functionMap.count(checkToken->str()) == 1) {
+                    sizeofFunctionError(tok);
+                }
+            }
         }
     }
 }
 
-void CheckSizeof::sizeofCalculationError(const Token *tok, bool inconclusive)
+void CheckSizeof::sizeofFunctionError(const Token *tok)
 {
     reportError(tok, Severity::warning,
-                "sizeofCalculation", "Found calculation inside sizeof().", CWE(0U), inconclusive);
+                "sizeofFunctionCall", "Found function call inside sizeof().", CWE682, Certainty::normal);
 }
 
 //-----------------------------------------------------------------------------
@@ -308,19 +383,28 @@ void CheckSizeof::sizeofCalculationError(const Token *tok, bool inconclusive)
 //-----------------------------------------------------------------------------
 void CheckSizeof::suspiciousSizeofCalculation()
 {
-    if (!_settings->isEnabled("warning") || !_settings->inconclusive)
+    if (!mSettings->severity.isEnabled(Severity::warning) || !mSettings->certainty.isEnabled(Certainty::inconclusive))
         return;
 
-    // TODO: Use AST here. This should be possible as soon as sizeof without brackets is correctly parsed
-    for (const Token *tok = _tokenizer->tokens(); tok; tok = tok->next()) {
+    for (const Token *tok = mTokenizer->tokens(); tok; tok = tok->next()) {
         if (Token::simpleMatch(tok, "sizeof (")) {
-            const Token* const end = tok->linkAt(1);
-            const Variable* var = end->previous()->variable();
-            if (end->strAt(-1) == "*" || (var && var->isPointer() && !var->isArray())) {
-                if (end->strAt(1) == "/")
-                    divideSizeofError(tok);
-            } else if (Token::simpleMatch(end, ") * sizeof") && end->next()->astOperand1() == tok->next())
-                multiplySizeofError(tok);
+            const Token* lPar = tok->astParent();
+            if (lPar && lPar->str() == "(") {
+                const Token* const rPar = lPar->link();
+                const Token* varTok = lPar->astOperand2();
+                int derefCount = 0;
+                while (Token::Match(varTok, "[|*")) {
+                    ++derefCount;
+                    varTok = varTok->astOperand1();
+                }
+                if (lPar->astParent() && lPar->astParent()->str() == "/") {
+                    const Variable* var = varTok ? varTok->variable() : nullptr;
+                    if (var && var->isPointer() && !var->isArray() && !(var->valueType() && var->valueType()->pointer <= derefCount))
+                        divideSizeofError(tok);
+                }
+                else if (Token::simpleMatch(rPar, ") * sizeof") && rPar->next()->astOperand1() == tok->next())
+                    multiplySizeofError(tok);
+            }
         }
     }
 }
@@ -328,7 +412,7 @@ void CheckSizeof::suspiciousSizeofCalculation()
 void CheckSizeof::multiplySizeofError(const Token *tok)
 {
     reportError(tok, Severity::warning,
-                "multiplySizeof", "Multiplying sizeof() with sizeof() indicates a logic error.", CWE(0U), true);
+                "multiplySizeof", "Multiplying sizeof() with sizeof() indicates a logic error.", CWE682, Certainty::inconclusive);
 }
 
 void CheckSizeof::divideSizeofError(const Token *tok)
@@ -336,16 +420,16 @@ void CheckSizeof::divideSizeofError(const Token *tok)
     reportError(tok, Severity::warning,
                 "divideSizeof", "Division of result of sizeof() on pointer type.\n"
                 "Division of result of sizeof() on pointer type. sizeof() returns the size of the pointer, "
-                "not the size of the memory area it points to.", CWE(0U), true);
+                "not the size of the memory area it points to.", CWE682, Certainty::inconclusive);
 }
 
 void CheckSizeof::sizeofVoid()
 {
-    if (!_settings->isEnabled("portability"))
+    if (!mSettings->severity.isEnabled(Severity::portability))
         return;
 
-    for (const Token *tok = _tokenizer->tokens(); tok; tok = tok->next()) {
-        if (Token::simpleMatch(tok, "sizeof ( )")) { // "sizeof(void)" gets simplified to sizeof ( )
+    for (const Token *tok = mTokenizer->tokens(); tok; tok = tok->next()) {
+        if (Token::simpleMatch(tok, "sizeof ( void )")) {
             sizeofVoidError(tok);
         } else if (Token::simpleMatch(tok, "sizeof (") && tok->next()->astOperand2()) {
             const ValueType *vt = tok->next()->astOperand2()->valueType();
@@ -355,16 +439,16 @@ void CheckSizeof::sizeofVoid()
             // only warn for: 'void *' - 'integral'
             const ValueType *vt1  = tok->astOperand1() ? tok->astOperand1()->valueType() : nullptr;
             const ValueType *vt2  = tok->astOperand2() ? tok->astOperand2()->valueType() : nullptr;
-            bool op1IsvoidPointer = (vt1 && vt1->type == ValueType::Type::VOID && vt1->pointer == 1U);
-            bool op2IsIntegral    = (vt2 && vt2->isIntegral() && vt2->pointer == 0U);
+            const bool op1IsvoidPointer = (vt1 && vt1->type == ValueType::Type::VOID && vt1->pointer == 1U);
+            const bool op2IsIntegral    = (vt2 && vt2->isIntegral() && vt2->pointer == 0U);
             if (op1IsvoidPointer && op2IsIntegral)
                 arithOperationsOnVoidPointerError(tok, tok->astOperand1()->expressionString(), vt1->str());
         } else if (Token::Match(tok, "+|++|--|+=|-=")) { // Arithmetic operations on variable of type "void*"
             const ValueType *vt1 = tok->astOperand1() ? tok->astOperand1()->valueType() : nullptr;
             const ValueType *vt2 = tok->astOperand2() ? tok->astOperand2()->valueType() : nullptr;
 
-            bool voidpointer1 = (vt1 && vt1->type == ValueType::Type::VOID && vt1->pointer == 1U);
-            bool voidpointer2 = (vt2 && vt2->type == ValueType::Type::VOID && vt2->pointer == 1U);
+            const bool voidpointer1 = (vt1 && vt1->type == ValueType::Type::VOID && vt1->pointer == 1U);
+            const bool voidpointer2 = (vt2 && vt2->type == ValueType::Type::VOID && vt2->pointer == 1U);
 
             if (voidpointer1)
                 arithOperationsOnVoidPointerError(tok, tok->astOperand1()->expressionString(), vt1->str());
@@ -379,19 +463,19 @@ void CheckSizeof::sizeofVoidError(const Token *tok)
 {
     const std::string message = "Behaviour of 'sizeof(void)' is not covered by the ISO C standard.";
     const std::string verbose = message + " A value for 'sizeof(void)' is defined only as part of a GNU C extension, which defines 'sizeof(void)' to be 1.";
-    reportError(tok, Severity::portability, "sizeofVoid", message + "\n" + verbose, CWE398, false);
+    reportError(tok, Severity::portability, "sizeofVoid", message + "\n" + verbose, CWE682, Certainty::normal);
 }
 
 void CheckSizeof::sizeofDereferencedVoidPointerError(const Token *tok, const std::string &varname)
 {
     const std::string message = "'*" + varname + "' is of type 'void', the behaviour of 'sizeof(void)' is not covered by the ISO C standard.";
     const std::string verbose = message + " A value for 'sizeof(void)' is defined only as part of a GNU C extension, which defines 'sizeof(void)' to be 1.";
-    reportError(tok, Severity::portability, "sizeofDereferencedVoidPointer", message + "\n" + verbose);
+    reportError(tok, Severity::portability, "sizeofDereferencedVoidPointer", message + "\n" + verbose, CWE682, Certainty::normal);
 }
 
 void CheckSizeof::arithOperationsOnVoidPointerError(const Token* tok, const std::string &varname, const std::string &vartype)
 {
-    const std::string message = "'" + varname + "' is of type '" + vartype + "'. When using void pointers in calculations, the behaviour is undefined.";
+    const std::string message = "'$symbol' is of type '" + vartype + "'. When using void pointers in calculations, the behaviour is undefined.";
     const std::string verbose = message + " Arithmetic operations on 'void *' is a GNU C extension, which defines the 'sizeof(void)' to be 1.";
-    reportError(tok, Severity::portability, "arithOperationsOnVoidPointer", message + "\n" + verbose);
+    reportError(tok, Severity::portability, "arithOperationsOnVoidPointer", "$symbol:" + varname + '\n' + message + '\n' + verbose, CWE467, Certainty::normal);
 }
