@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2016 Cppcheck team.
+ * Copyright (C) 2007-2022 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,11 +22,15 @@
 #define checksizeofH
 //---------------------------------------------------------------------------
 
-#include "config.h"
 #include "check.h"
+#include "config.h"
 
-class Function;
-class Variable;
+#include <string>
+
+class ErrorLogger;
+class Settings;
+class Token;
+class Tokenizer;
 
 /// @addtogroup Checks
 /// @{
@@ -37,21 +41,20 @@ class Variable;
 class CPPCHECKLIB CheckSizeof : public Check {
 public:
     /** @brief This constructor is used when registering the CheckClass */
-    CheckSizeof() : Check(myName()) {
-    }
+    CheckSizeof() : Check(myName()) {}
 
     /** @brief This constructor is used when running checks. */
     CheckSizeof(const Tokenizer* tokenizer, const Settings* settings, ErrorLogger* errorLogger)
-        : Check(myName(), tokenizer, settings, errorLogger) {
-    }
+        : Check(myName(), tokenizer, settings, errorLogger) {}
 
     /** @brief Run checks against the normal token list */
-    void runChecks(const Tokenizer* tokenizer, const Settings* settings, ErrorLogger* errorLogger) {
+    void runChecks(const Tokenizer* tokenizer, const Settings* settings, ErrorLogger* errorLogger) override {
         CheckSizeof checkSizeof(tokenizer, settings, errorLogger);
 
         // Checks
         checkSizeof.sizeofsizeof();
         checkSizeof.sizeofCalculation();
+        checkSizeof.sizeofFunction();
         checkSizeof.suspiciousSizeofCalculation();
         checkSizeof.checkSizeofForArrayParameter();
         checkSizeof.checkSizeofForPointerSize();
@@ -59,16 +62,15 @@ public:
         checkSizeof.sizeofVoid();
     }
 
-    /** @brief Run checks against the simplified token list */
-    void runSimplifiedChecks(const Tokenizer*, const Settings*, ErrorLogger*) {
-    }
-
     /** @brief %Check for 'sizeof sizeof ..' */
     void sizeofsizeof();
 
     /** @brief %Check for calculations inside sizeof */
     void sizeofCalculation();
 
+    /** @brief %Check for function call inside sizeof */
+    void sizeofFunction();
+
     /** @brief %Check for suspicious calculations with sizeof results */
     void suspiciousSizeofCalculation();
 
@@ -88,6 +90,7 @@ private:
     // Error messages..
     void sizeofsizeofError(const Token* tok);
     void sizeofCalculationError(const Token* tok, bool inconclusive);
+    void sizeofFunctionError(const Token* tok);
     void multiplySizeofError(const Token* tok);
     void divideSizeofError(const Token* tok);
     void sizeofForArrayParameterError(const Token* tok);
@@ -98,33 +101,35 @@ private:
     void sizeofDereferencedVoidPointerError(const Token *tok, const std::string &varname);
     void arithOperationsOnVoidPointerError(const Token* tok, const std::string &varname, const std::string &vartype);
 
-    void getErrorMessages(ErrorLogger* errorLogger, const Settings* settings) const {
-        CheckSizeof c(0, settings, errorLogger);
+    void getErrorMessages(ErrorLogger* errorLogger, const Settings* settings) const override {
+        CheckSizeof c(nullptr, settings, errorLogger);
 
-        c.sizeofForArrayParameterError(0);
-        c.sizeofForPointerError(0, "varname");
-        c.divideBySizeofError(0, "memset");
-        c.sizeofForNumericParameterError(0);
-        c.sizeofsizeofError(0);
-        c.sizeofCalculationError(0, false);
-        c.multiplySizeofError(0);
-        c.divideSizeofError(0);
-        c.sizeofVoidError(0);
-        c.sizeofDereferencedVoidPointerError(0, "varname");
-        c.arithOperationsOnVoidPointerError(0, "varname", "vartype");
+        c.sizeofForArrayParameterError(nullptr);
+        c.sizeofForPointerError(nullptr, "varname");
+        c.divideBySizeofError(nullptr, "memset");
+        c.sizeofForNumericParameterError(nullptr);
+        c.sizeofsizeofError(nullptr);
+        c.sizeofCalculationError(nullptr, false);
+        c.sizeofFunctionError(nullptr);
+        c.multiplySizeofError(nullptr);
+        c.divideSizeofError(nullptr);
+        c.sizeofVoidError(nullptr);
+        c.sizeofDereferencedVoidPointerError(nullptr, "varname");
+        c.arithOperationsOnVoidPointerError(nullptr, "varname", "vartype");
     }
 
     static std::string myName() {
         return "Sizeof";
     }
 
-    std::string classInfo() const {
+    std::string classInfo() const override {
         return "sizeof() usage checks\n"
                "- sizeof for array given as function argument\n"
                "- sizeof for numeric given as function argument\n"
                "- using sizeof(pointer) instead of the size of pointed data\n"
                "- look for 'sizeof sizeof ..'\n"
                "- look for calculations inside sizeof()\n"
+               "- look for function calls inside sizeof()\n"
                "- look for suspicious calculations with sizeof()\n"
                "- using 'sizeof(void)' which is undefined\n";
     }
