@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2020 Cppcheck team.
+ * Copyright (C) 2007-2022 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,20 +20,28 @@
 
 #include "path.h"
 #include "settings.h"
+#include "standards.h"
 #include "suppressions.h"
-#include "tinyxml2.h"
 #include "token.h"
 #include "tokenize.h"
 #include "utils.h"
+
+#include <algorithm>
+#include <cstdlib>
+#include <cstring>
+#include <fstream> // IWYU pragma: keep
+#include <iostream>
+#include <iterator>
+
+#include <utility>
+
+#include <tinyxml2.h>
+
+#include <simplecpp.h>
+
 #define PICOJSON_USE_INT64
 #include <picojson.h>
 
-#include <cstring>
-#include <fstream>
-#include <utility>
-#include <sstream>
-
-
 ImportProject::ImportProject()
 {
     projectType = Type::UNKNOWN;
@@ -77,16 +85,6 @@ void ImportProject::ignoreOtherConfigs(const std::string &cfg)
     }
 }
 
-void ImportProject::ignoreOtherPlatforms(cppcheck::Platform::PlatformType platformType)
-{
-    for (std::list<FileSettings>::iterator it = fileSettings.begin(); it != fileSettings.end();) {
-        if (it->platformType != cppcheck::Platform::Unspecified && it->platformType != platformType)
-            fileSettings.erase(it++);
-        else
-            ++it;
-    }
-}
-
 void ImportProject::FileSettings::setDefines(std::string defs)
 {
     while (defs.find(";%(") != std::string::npos) {
@@ -151,22 +149,22 @@ static bool simplifyPathWithVariables(std::string &s, std::map<std::string, std:
 
 void ImportProject::FileSettings::setIncludePaths(const std::string &basepath, const std::list<std::string> &in, std::map<std::string, std::string, cppcheck::stricmp> &variables)
 {
-    std::list<std::string> listInc;
-    // only parse each includePath once - so remove duplicates
-    std::list<std::string> uniqueIncludePaths = in;
-    uniqueIncludePaths.sort();
-    uniqueIncludePaths.unique();
-
-    for (const std::string &it : uniqueIncludePaths) {
-        if (it.empty())
+    std::set<std::string> found;
+    // NOLINTNEXTLINE(performance-unnecessary-copy-initialization)
+    const std::list<std::string> copyIn(in);
+    includePaths.clear();
+    for (const std::string &ipath : copyIn) {
+        if (ipath.empty())
             continue;
-        if (it.compare(0,2,"%(")==0)
+        if (ipath.compare(0,2,"%(")==0)
+            continue;
+        std::string s(Path::fromNativeSeparators(ipath));
+        if (!found.insert(s).second)
             continue;
-        std::string s(Path::fromNativeSeparators(it));
         if (s[0] == '/' || (s.size() > 1U && s.compare(1,2,":/") == 0)) {
             if (!endsWith(s,'/'))
                 s += '/';
-            listInc.push_back(s);
+            includePaths.push_back(s);
             continue;
         }
 
@@ -181,9 +179,8 @@ void ImportProject::FileSettings::setIncludePaths(const std::string &basepath, c
         }
         if (s.empty())
             continue;
-        listInc.push_back(s + '/');
+        includePaths.push_back(s + '/');
     }
-    includePaths.swap(listInc);
 }
 
 ImportProject::Type ImportProject::import(const std::string &filename, Settings *settings)
@@ -196,41 +193,87 @@ ImportProject::Type ImportProject::import(const std::string &filename, Settings
     if (!mPath.empty() && !endsWith(mPath,'/'))
         mPath += '/';
 
-    if (endsWith(filename, ".json", 5)) {
-        importCompileCommands(fin);
-        return ImportProject::Type::COMPILE_DB;
-    } else if (endsWith(filename, ".sln", 4)) {
-        importSln(fin,mPath);
-        return ImportProject::Type::VS_SLN;
-    } else if (endsWith(filename, ".vcxproj", 8)) {
+    const std::vector<std::string> fileFilters =
+        settings ? settings->fileFilters : std::vector<std::string>();
+
+    if (endsWith(filename, ".json")) {
+        if (importCompileCommands(fin)) {
+            setRelativePaths(filename);
+            return ImportProject::Type::COMPILE_DB;
+        }
+    } else if (endsWith(filename, ".sln")) {
+        if (importSln(fin, mPath, fileFilters)) {
+            setRelativePaths(filename);
+            return ImportProject::Type::VS_SLN;
+        }
+    } else if (endsWith(filename, ".vcxproj")) {
         std::map<std::string, std::string, cppcheck::stricmp> variables;
-        importVcxproj(filename, variables, emptyString);
-        return ImportProject::Type::VS_VCXPROJ;
-    } else if (endsWith(filename, ".bpr", 4)) {
-        importBcb6Prj(filename);
-        return ImportProject::Type::BORLAND;
-    } else if (settings && endsWith(filename, ".cppcheck", 9)) {
-        return importCppcheckGuiProject(fin, settings) ? ImportProject::Type::CPPCHECK_GUI : ImportProject::Type::MISSING;
+        if (importVcxproj(filename, variables, emptyString, fileFilters)) {
+            setRelativePaths(filename);
+            return ImportProject::Type::VS_VCXPROJ;
+        }
+    } else if (endsWith(filename, ".bpr")) {
+        if (importBcb6Prj(filename)) {
+            setRelativePaths(filename);
+            return ImportProject::Type::BORLAND;
+        }
+    } else if (settings && endsWith(filename, ".cppcheck")) {
+        if (importCppcheckGuiProject(fin, settings)) {
+            setRelativePaths(filename);
+            return ImportProject::Type::CPPCHECK_GUI;
+        }
+    } else {
+        return ImportProject::Type::UNKNOWN;
     }
-    return ImportProject::Type::UNKNOWN;
+    return ImportProject::Type::FAILURE;
 }
 
 static std::string readUntil(const std::string &command, std::string::size_type *pos, const char until[])
 {
     std::string ret;
+    bool escapedString = false;
     bool str = false;
-    while (*pos < command.size() && (str || !std::strchr(until, command[*pos]))) {
-        if (command[*pos] == '\\')
-            ++*pos;
-        if (*pos < command.size())
-            ret += command[(*pos)++];
-        if (endsWith(ret, '\"'))
+    bool escape = false;
+    for (; *pos < command.size() && (str || !std::strchr(until, command[*pos])); (*pos)++) {
+        if (escape)
+            escape = false;
+        else if (command[*pos] == '\\') {
+            if (str)
+                escape = true;
+            else if (command[*pos + 1] == '"') {
+                if (escapedString)
+                    return ret + "\\\"";
+                escapedString = true;
+                ret += "\\\"";
+                (*pos)++;
+                continue;
+            }
+        } else if (command[*pos] == '\"')
             str = !str;
+        ret += command[*pos];
     }
     return ret;
 }
 
-void ImportProject::FileSettings::parseCommand(const std::string &command)
+static std::string unescape(const std::string &in)
+{
+    std::string out;
+    bool escape = false;
+    for (char c: in) {
+        if (escape) {
+            escape = false;
+            if (!std::strchr("\\\"\'",c))
+                out += "\\";
+            out += c;
+        } else if (c == '\\')
+            escape = true;
+        else
+            out += c;
+    }
+    return out;
+}
+
+void ImportProject::FileSettings::parseCommand(std::string command)
 {
     std::string defs;
 
@@ -253,32 +296,30 @@ void ImportProject::FileSettings::parseCommand(const std::string &command)
         }
         const std::string fval = readUntil(command, &pos, " =");
         if (F=='D') {
-            const std::string defval = readUntil(command, &pos, " ");
+            std::string defval = readUntil(command, &pos, " ");
             defs += fval;
+            if (defval.size() >= 3 && defval.compare(0,2,"=\"")==0 && defval.back()=='\"')
+                defval = "=" + unescape(defval.substr(2, defval.size() - 3));
+            else if (defval.size() >= 5 && defval.compare(0, 3, "=\\\"") == 0 && endsWith(defval, "\\\""))
+                defval = "=\"" + unescape(defval.substr(3, defval.size() - 5)) + "\"";
             if (!defval.empty())
                 defs += defval;
             defs += ';';
         } else if (F=='U')
             undefs.insert(fval);
         else if (F=='I') {
-            if (std::find(includePaths.begin(), includePaths.end(), fval) == includePaths.end())
-                includePaths.push_back(fval);
+            std::string i = fval;
+            if (i.size() > 1 && i[0] == '\"' && i.back() == '\"')
+                i = unescape(i.substr(1, i.size() - 2));
+            if (std::find(includePaths.begin(), includePaths.end(), i) == includePaths.end())
+                includePaths.push_back(i);
         } else if (F=='s' && fval.compare(0,2,"td") == 0) {
             ++pos;
             const std::string stdval = readUntil(command, &pos, " ");
             standard = stdval;
+            // TODO: use simplecpp::DUI::std instead of specifying it manually
             if (standard.compare(0, 3, "c++") || standard.compare(0, 5, "gnu++")) {
-                std::string stddef;
-                if (standard == "c++98" || standard == "gnu++98" || standard == "c++03" || standard == "gnu++03") {
-                    stddef = "199711L";
-                } else if (standard == "c++11" || standard == "gnu++11"|| standard == "c++0x" || standard == "gnu++0x") {
-                    stddef = "201103L";
-                } else if (standard == "c++14" || standard == "gnu++14" || standard == "c++1y" || standard == "gnu++1y") {
-                    stddef = "201402L";
-                } else if (standard == "c++17" || standard == "gnu++17" || standard == "c++1z" || standard == "gnu++1z") {
-                    stddef = "201703L";
-                }
-
+                const std::string stddef = simplecpp::getCppStdString(standard);
                 if (stddef.empty()) {
                     // TODO: log error
                     continue;
@@ -288,21 +329,7 @@ void ImportProject::FileSettings::parseCommand(const std::string &command)
                 defs += stddef;
                 defs += ";";
             } else if (standard.compare(0, 1, "c") || standard.compare(0, 3, "gnu")) {
-                if (standard == "c90" || standard == "iso9899:1990" || standard == "gnu90" || standard == "iso9899:199409") {
-                    // __STDC_VERSION__ is not set for C90 although the macro was added in the 1994 amendments
-                    continue;
-                }
-
-                std::string stddef;
-
-                if (standard == "c99" || standard == "iso9899:1999" || standard == "gnu99") {
-                    stddef = "199901L";
-                } else if (standard == "c11" || standard == "iso9899:2011" || standard == "gnu11" || standard == "c1x" || standard == "gnu1x") {
-                    stddef = "201112L";
-                } else if (standard == "c17") {
-                    stddef = "201710L";
-                }
-
+                const std::string stddef = simplecpp::getCStdString(standard);
                 if (stddef.empty()) {
                     // TODO: log error
                     continue;
@@ -340,12 +367,14 @@ void ImportProject::FileSettings::parseCommand(const std::string &command)
     setDefines(defs);
 }
 
-void ImportProject::importCompileCommands(std::istream &istr)
+bool ImportProject::importCompileCommands(std::istream &istr)
 {
     picojson::value compileCommands;
     istr >> compileCommands;
-    if (!compileCommands.is<picojson::array>())
-        return;
+    if (!compileCommands.is<picojson::array>()) {
+        printError("compilation database is not a JSON array");
+        return false;
+    }
 
     for (const picojson::value &fileInfo : compileCommands.get<picojson::array>()) {
         picojson::object obj = fileInfo.get<picojson::object>();
@@ -358,26 +387,38 @@ void ImportProject::importCompileCommands(std::istream &istr)
 
         const std::string directory = dirpath;
 
-        std::ostringstream comm;
-        if (obj.find("arguments") != obj.end()) {
-            if (obj[ "arguments" ].is< picojson::array >()) {
-                for (const picojson::value& arg : obj[ "arguments" ].get< picojson::array >()) {
-                    if (arg.is< std::string >()) {
-                        comm << arg.get< std::string >() << " ";
+        std::string command;
+        if (obj.count("arguments")) {
+            if (obj["arguments"].is<picojson::array>()) {
+                for (const picojson::value& arg : obj["arguments"].get<picojson::array>()) {
+                    if (arg.is<std::string>()) {
+                        std::string str = arg.get<std::string>();
+                        if (str.find(" ") != std::string::npos)
+                            str = "\"" + str + "\"";
+                        command += str + " ";
                     }
                 }
             } else {
-                return;
+                printError("'arguments' field in compilation database entry is not a JSON array");
+                return false;
             }
-        } else if (obj.find("command") != obj.end()) {
-            if (obj[ "command" ].is< std::string >()) {
-                comm << obj[ "command" ].get< std::string >();
+        } else if (obj.count("command")) {
+            if (obj["command"].is<std::string>()) {
+                command = obj["command"].get<std::string>();
+            } else {
+                printError("'command' field in compilation database entry is not a string");
+                return false;
             }
         } else {
-            return;
+            printError("no 'arguments' or 'command' field found in compilation database entry");
+            return false;
+        }
+
+        if (!obj.count("file") || !obj["file"].is<std::string>()) {
+            printError("skip compilation database entry because it does not have a proper 'file' field");
+            continue;
         }
 
-        const std::string command = comm.str();
         const std::string file = Path::fromNativeSeparators(obj["file"].get<std::string>());
 
         // Accept file?
@@ -385,28 +426,52 @@ void ImportProject::importCompileCommands(std::istream &istr)
             continue;
 
         struct FileSettings fs;
-        if (Path::isAbsolute(file) || Path::fileExists(file))
-            fs.filename = file;
-        else {
-            std::string path = directory;
-            if (!path.empty() && !endsWith(path,'/'))
-                path += '/';
-            path += file;
-            fs.filename = Path::simplifyPath(path);
+        if (Path::isAbsolute(file))
+            fs.filename = Path::simplifyPath(file);
+#ifdef _WIN32
+        else if (file[0] == '/' && directory.size() > 2 && std::isalpha(directory[0]) && directory[1] == ':')
+            // directory: C:\foo\bar
+            // file: /xy/z.c
+            // => c:/xy/z.c
+            fs.filename = Path::simplifyPath(directory.substr(0,2) + file);
+#endif
+        else
+            fs.filename = Path::simplifyPath(directory + file);
+        if (!sourceFileExists(fs.filename)) {
+            printError("'" + fs.filename + "' from compilation database does not exist");
+            return false;
         }
         fs.parseCommand(command); // read settings; -D, -I, -U, -std, -m*, -f*
         std::map<std::string, std::string, cppcheck::stricmp> variables;
         fs.setIncludePaths(directory, fs.includePaths, variables);
         fileSettings.push_back(fs);
     }
+
+    return true;
 }
 
-void ImportProject::importSln(std::istream &istr, const std::string &path)
+bool ImportProject::importSln(std::istream &istr, const std::string &path, const std::vector<std::string> &fileFilters)
 {
+    std::string line;
+
+    if (!std::getline(istr,line)) {
+        printError("Visual Studio solution file is empty");
+        return false;
+    }
+
+    if (line.find("Microsoft Visual Studio Solution File") != 0) {
+        // Skip BOM
+        if (!std::getline(istr, line) || line.find("Microsoft Visual Studio Solution File") != 0) {
+            printError("Visual Studio solution file header not found");
+            return false;
+        }
+    }
+
     std::map<std::string,std::string,cppcheck::stricmp> variables;
     variables["SolutionDir"] = path;
 
-    std::string line;
+    bool found = false;
+
     while (std::getline(istr,line)) {
         if (line.compare(0,8,"Project(")!=0)
             continue;
@@ -419,8 +484,19 @@ void ImportProject::importSln(std::istream &istr, const std::string &path)
         std::string vcxproj(line.substr(pos1+1, pos-pos1+7));
         if (!Path::isAbsolute(vcxproj))
             vcxproj = path + vcxproj;
-        importVcxproj(Path::fromNativeSeparators(vcxproj), variables, emptyString);
+        if (!importVcxproj(Path::fromNativeSeparators(vcxproj), variables, emptyString, fileFilters)) {
+            printError("failed to load '" + vcxproj + "' from Visual Studio solution");
+            return false;
+        }
+        found = true;
     }
+
+    if (!found) {
+        printError("no projects found in Visual Studio solution file");
+        return false;
+    }
+
+    return true;
 }
 
 namespace {
@@ -459,6 +535,7 @@ namespace {
             for (const tinyxml2::XMLElement *e1 = idg->FirstChildElement(); e1; e1 = e1->NextSiblingElement()) {
                 if (std::strcmp(e1->Name(), "ClCompile") != 0)
                     continue;
+                enhancedInstructionSet = "StreamingSIMDExtensions2";
                 for (const tinyxml2::XMLElement *e = e1->FirstChildElement(); e; e = e->NextSiblingElement()) {
                     if (e->GetText()) {
                         if (std::strcmp(e->Name(), "PreprocessorDefinitions") == 0)
@@ -467,6 +544,17 @@ namespace {
                             if (!additionalIncludePaths.empty())
                                 additionalIncludePaths += ';';
                             additionalIncludePaths += e->GetText();
+                        } else if (std::strcmp(e->Name(), "LanguageStandard") == 0) {
+                            if (std::strcmp(e->GetText(), "stdcpp14") == 0)
+                                cppstd = Standards::CPP14;
+                            else if (std::strcmp(e->GetText(), "stdcpp17") == 0)
+                                cppstd = Standards::CPP17;
+                            else if (std::strcmp(e->GetText(), "stdcpp20") == 0)
+                                cppstd = Standards::CPP20;
+                            else if (std::strcmp(e->GetText(), "stdcpplatest") == 0)
+                                cppstd = Standards::CPPLatest;
+                        } else if (std::strcmp(e->Name(), "EnableEnhancedInstructionSet") == 0) {
+                            enhancedInstructionSet = e->GetText();
                         }
                     }
                 }
@@ -504,8 +592,10 @@ namespace {
             return false;
         }
         std::string condition;
+        std::string enhancedInstructionSet;
         std::string preprocessorDefinitions;
         std::string additionalIncludePaths;
+        Standards::cppstd_t cppstd = Standards::CPPLatest;
     };
 }
 
@@ -602,7 +692,7 @@ static void loadVisualStudioProperties(const std::string &props, std::map<std::s
     }
 }
 
-void ImportProject::importVcxproj(const std::string &filename, std::map<std::string, std::string, cppcheck::stricmp> &variables, const std::string &additionalIncludeDirectories)
+bool ImportProject::importVcxproj(const std::string &filename, std::map<std::string, std::string, cppcheck::stricmp> &variables, const std::string &additionalIncludeDirectories, const std::vector<std::string> &fileFilters)
 {
     variables["ProjectDir"] = Path::simplifyPath(Path::getPathFromFilename(filename));
 
@@ -615,11 +705,15 @@ void ImportProject::importVcxproj(const std::string &filename, std::map<std::str
 
     tinyxml2::XMLDocument doc;
     const tinyxml2::XMLError error = doc.LoadFile(filename.c_str());
-    if (error != tinyxml2::XML_SUCCESS)
-        return;
+    if (error != tinyxml2::XML_SUCCESS) {
+        printError(std::string("Visual Studio project file is not a valid XML - ") + tinyxml2::XMLDocument::ErrorIDToName(error));
+        return false;
+    }
     const tinyxml2::XMLElement * const rootnode = doc.FirstChildElement();
-    if (rootnode == nullptr)
-        return;
+    if (rootnode == nullptr) {
+        printError("Visual Studio project file has no XML root node");
+        return false;
+    }
     for (const tinyxml2::XMLElement *node = rootnode->FirstChildElement(); node; node = node->NextSiblingElement()) {
         if (std::strcmp(node->Name(), "ItemGroup") == 0) {
             const char *labelAttribute = node->Attribute("Label");
@@ -661,11 +755,15 @@ void ImportProject::importVcxproj(const std::string &filename, std::map<std::str
     }
 
     for (const std::string &c : compileList) {
+        const std::string cfilename = Path::simplifyPath(Path::isAbsolute(c) ? c : Path::getPathFromFilename(filename) + c);
+        if (!fileFilters.empty() && !matchglobs(fileFilters, cfilename))
+            continue;
+
         for (const ProjectConfiguration &p : projectConfigurationList) {
 
             if (!guiProject.checkVsConfigs.empty()) {
                 bool doChecking = false;
-                for (std::string config : guiProject.checkVsConfigs)
+                for (const std::string& config : guiProject.checkVsConfigs)
                     if (config == p.configuration) {
                         doChecking = true;
                         break;
@@ -675,7 +773,7 @@ void ImportProject::importVcxproj(const std::string &filename, std::map<std::str
             }
 
             FileSettings fs;
-            fs.filename = Path::simplifyPath(Path::isAbsolute(c) ? c : Path::getPathFromFilename(filename) + c);
+            fs.filename = cfilename;
             fs.cfg = p.name;
             fs.msc = true;
             fs.useMfc = useOfMfc;
@@ -690,7 +788,18 @@ void ImportProject::importVcxproj(const std::string &filename, std::map<std::str
             for (const ItemDefinitionGroup &i : itemDefinitionGroupList) {
                 if (!i.conditionIsTrue(p))
                     continue;
+                fs.standard = Standards::getCPP(i.cppstd);
                 fs.defines += ';' + i.preprocessorDefinitions;
+                if (i.enhancedInstructionSet == "StreamingSIMDExtensions")
+                    fs.defines += ";__SSE__";
+                else if (i.enhancedInstructionSet == "StreamingSIMDExtensions2")
+                    fs.defines += ";__SSE2__";
+                else if (i.enhancedInstructionSet == "AdvancedVectorExtensions")
+                    fs.defines += ";__AVX__";
+                else if (i.enhancedInstructionSet == "AdvancedVectorExtensions2")
+                    fs.defines += ";__AVX2__";
+                else if (i.enhancedInstructionSet == "AdvancedVectorExtensions512")
+                    fs.defines += ";__AVX512__";
                 additionalIncludePaths += ';' + i.additionalIncludePaths;
             }
             fs.setDefines(fs.defines);
@@ -698,17 +807,23 @@ void ImportProject::importVcxproj(const std::string &filename, std::map<std::str
             fileSettings.push_back(fs);
         }
     }
+
+    return true;
 }
 
-void ImportProject::importBcb6Prj(const std::string &projectFilename)
+bool ImportProject::importBcb6Prj(const std::string &projectFilename)
 {
     tinyxml2::XMLDocument doc;
     const tinyxml2::XMLError error = doc.LoadFile(projectFilename.c_str());
-    if (error != tinyxml2::XML_SUCCESS)
-        return;
+    if (error != tinyxml2::XML_SUCCESS) {
+        printError(std::string("Borland project file is not a valid XML - ") + tinyxml2::XMLDocument::ErrorIDToName(error));
+        return false;
+    }
     const tinyxml2::XMLElement * const rootnode = doc.FirstChildElement();
-    if (rootnode == nullptr)
-        return;
+    if (rootnode == nullptr) {
+        printError("Borland project file has no XML root node");
+        return false;
+    }
 
     const std::string& projectDir = Path::simplifyPath(Path::getPathFromFilename(projectFilename));
 
@@ -958,6 +1073,8 @@ void ImportProject::importBcb6Prj(const std::string &projectFilename)
         fs.filename = Path::simplifyPath(Path::isAbsolute(c) ? c : projectDir + c);
         fileSettings.push_back(fs);
     }
+
+    return true;
 }
 
 static std::string joinRelativePath(const std::string &path1, const std::string &path2)
@@ -1000,16 +1117,21 @@ bool ImportProject::importCppcheckGuiProject(std::istream &istr, Settings *setti
 {
     tinyxml2::XMLDocument doc;
     const std::string xmldata = istream_to_string(istr);
-    if (doc.Parse(xmldata.data(), xmldata.size()) != tinyxml2::XML_SUCCESS)
+    const tinyxml2::XMLError error = doc.Parse(xmldata.data(), xmldata.size());
+    if (error != tinyxml2::XML_SUCCESS) {
+        printError(std::string("Cppcheck GUI project file is not a valid XML - ") + tinyxml2::XMLDocument::ErrorIDToName(error));
         return false;
+    }
     const tinyxml2::XMLElement * const rootnode = doc.FirstChildElement();
-    if (rootnode == nullptr || strcmp(rootnode->Name(), CppcheckXml::ProjectElementName) != 0)
+    if (rootnode == nullptr || strcmp(rootnode->Name(), CppcheckXml::ProjectElementName) != 0) {
+        printError("Cppcheck GUI project file has no XML root node");
         return false;
+    }
 
     const std::string &path = mPath;
 
     std::list<std::string> paths;
-    std::list<std::string> suppressions;
+    std::list<Suppressions::Suppression> suppressions;
     Settings temp;
 
     guiProject.analyzeAllVsConfigs.clear();
@@ -1019,7 +1141,7 @@ bool ImportProject::importCppcheckGuiProject(std::istream &istr, Settings *setti
             temp.basePaths.push_back(joinRelativePath(path, node->Attribute(CppcheckXml::RootPathNameAttrib)));
             temp.relativePaths = true;
         } else if (strcmp(node->Name(), CppcheckXml::BugHunting) == 0)
-            temp.bugHunting = true;
+            ;
         else if (strcmp(node->Name(), CppcheckXml::BuildDirElementName) == 0)
             temp.buildDir = joinRelativePath(path, node->GetText() ? node->GetText() : "");
         else if (strcmp(node->Name(), CppcheckXml::IncludeDirElementName) == 0)
@@ -1035,22 +1157,32 @@ bool ImportProject::importCppcheckGuiProject(std::istream &istr, Settings *setti
             paths = readXmlStringList(node, path, CppcheckXml::PathName, CppcheckXml::PathNameAttrib);
         else if (strcmp(node->Name(), CppcheckXml::ExcludeElementName) == 0)
             guiProject.excludedPaths = readXmlStringList(node, "", CppcheckXml::ExcludePathName, CppcheckXml::ExcludePathNameAttrib);
-        else if (strcmp(node->Name(), CppcheckXml::FunctionContracts) == 0) {
-            for (const tinyxml2::XMLElement *child = node->FirstChildElement(); child; child = child->NextSiblingElement()) {
-                if (strcmp(child->Name(), CppcheckXml::FunctionContract) == 0) {
-                    const char *function = child->Attribute(CppcheckXml::ContractFunction);
-                    const char *expects = child->Attribute(CppcheckXml::ContractExpects);
-                    if (function && expects)
-                        temp.functionContracts[function] = expects;
-                }
-            }
-        } else if (strcmp(node->Name(), CppcheckXml::IgnoreElementName) == 0)
+        else if (strcmp(node->Name(), CppcheckXml::FunctionContracts) == 0)
+            ;
+        else if (strcmp(node->Name(), CppcheckXml::VariableContractsElementName) == 0)
+            ;
+        else if (strcmp(node->Name(), CppcheckXml::IgnoreElementName) == 0)
             guiProject.excludedPaths = readXmlStringList(node, "", CppcheckXml::IgnorePathName, CppcheckXml::IgnorePathNameAttrib);
         else if (strcmp(node->Name(), CppcheckXml::LibrariesElementName) == 0)
             guiProject.libraries = readXmlStringList(node, "", CppcheckXml::LibraryElementName, nullptr);
-        else if (strcmp(node->Name(), CppcheckXml::SuppressionsElementName) == 0)
-            suppressions = readXmlStringList(node, "", CppcheckXml::SuppressionElementName, nullptr);
-        else if (strcmp(node->Name(), CppcheckXml::VSConfigurationElementName) == 0)
+        else if (strcmp(node->Name(), CppcheckXml::SuppressionsElementName) == 0) {
+            for (const tinyxml2::XMLElement *child = node->FirstChildElement(); child; child = child->NextSiblingElement()) {
+                if (strcmp(child->Name(), CppcheckXml::SuppressionElementName) != 0)
+                    continue;
+                auto read = [](const char *s, const char *def) {
+                    return s ? s : def;
+                };
+                Suppressions::Suppression s;
+                s.errorId = read(child->GetText(), "");
+                s.fileName = read(child->Attribute("fileName"), "");
+                if (!s.fileName.empty())
+                    s.fileName = joinRelativePath(path, s.fileName);
+                s.lineNumber = child->IntAttribute("lineNumber", Suppressions::Suppression::NO_LINE);
+                s.symbolName = read(child->Attribute("symbolName"), "");
+                std::istringstream(read(child->Attribute("hash"), "0")) >> s.hash;
+                suppressions.push_back(s);
+            }
+        } else if (strcmp(node->Name(), CppcheckXml::VSConfigurationElementName) == 0)
             guiProject.checkVsConfigs = readXmlStringList(node, "", CppcheckXml::VSConfigurationName, nullptr);
         else if (strcmp(node->Name(), CppcheckXml::PlatformElementName) == 0)
             guiProject.platform = node->GetText();
@@ -1074,6 +1206,8 @@ bool ImportProject::importCppcheckGuiProject(std::istream &istr, Settings *setti
             temp.checkUnusedTemplates = (strcmp(node->GetText(), "true") == 0);
         else if (strcmp(node->Name(), CppcheckXml::MaxCtuDepthElementName) == 0)
             temp.maxCtuDepth = std::atoi(node->GetText());
+        else if (strcmp(node->Name(), CppcheckXml::MaxTemplateRecursionElementName) == 0)
+            temp.maxTemplateRecursion = std::atoi(node->GetText());
         else if (strcmp(node->Name(), CppcheckXml::CheckUnknownFunctionReturn) == 0)
             ; // TODO
         else if (strcmp(node->Name(), Settings::SafeChecks::XmlRootName) == 0) {
@@ -1089,7 +1223,9 @@ bool ImportProject::importCppcheckGuiProject(std::istream &istr, Settings *setti
                 else
                     return false;
             }
-        } else
+        } else if (strcmp(node->Name(), CppcheckXml::TagWarningsElementName) == 0)
+            ; // TODO
+        else
             return false;
     }
     settings->basePaths = temp.basePaths;
@@ -1104,14 +1240,14 @@ bool ImportProject::importCppcheckGuiProject(std::istream &istr, Settings *setti
 
     for (const std::string &p : paths)
         guiProject.pathNames.push_back(p);
-    for (const std::string &supp : suppressions)
-        settings->nomsg.addSuppressionLine(supp);
+    for (const Suppressions::Suppression &supp : suppressions)
+        settings->nomsg.addSuppression(supp);
     settings->checkHeaders = temp.checkHeaders;
     settings->checkUnusedTemplates = temp.checkUnusedTemplates;
     settings->maxCtuDepth = temp.maxCtuDepth;
+    settings->maxTemplateRecursion = temp.maxTemplateRecursion;
     settings->safeChecks = temp.safeChecks;
-    settings->bugHunting = temp.bugHunting;
-    settings->functionContracts = temp.functionContracts;
+
     return true;
 }
 
@@ -1146,5 +1282,27 @@ void ImportProject::selectOneVsConfig(Settings::PlatformType platform)
 
 std::list<std::string> ImportProject::getVSConfigs()
 {
-    return std::list<std::string> (mAllVSConfigs.begin(), mAllVSConfigs.end());
+    return std::list<std::string>(mAllVSConfigs.begin(), mAllVSConfigs.end());
+}
+
+void ImportProject::setRelativePaths(const std::string &filename)
+{
+    if (Path::isAbsolute(filename))
+        return;
+    const std::vector<std::string> basePaths{Path::fromNativeSeparators(Path::getCurrentPath())};
+    for (auto &fs: fileSettings) {
+        fs.filename = Path::getRelativePath(fs.filename, basePaths);
+        for (auto &includePath: fs.includePaths)
+            includePath = Path::getRelativePath(includePath, basePaths);
+    }
+}
+
+void ImportProject::printError(const std::string &message)
+{
+    std::cout << "cppcheck: error: " << message << std::endl;
+}
+
+bool ImportProject::sourceFileExists(const std::string &file)
+{
+    return Path::fileExists(file);
 }
