@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2020 Cppcheck team.
+ * Copyright (C) 2007-2022 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,14 +18,18 @@
 
 
 #include "mathlib.h"
-#include "errorlogger.h"
+#include "errortypes.h"
 #include "utils.h"
 
 #include <cctype>
 #include <cmath>
 #include <cstdlib>
+#include <exception>
 #include <limits>
 #include <locale>
+#include <stdexcept>
+
+#include <simplecpp.h>
 
 #if defined(_MSC_VER) && _MSC_VER <= 1700  // VS2012 doesn't have std::isinf and std::isnan
 #define ISINF(x)      (!_finite(x))
@@ -44,7 +48,7 @@ MathLib::value::value(const std::string &s) :
     mIntValue(0), mDoubleValue(0), mIsUnsigned(false)
 {
     if (MathLib::isFloat(s)) {
-        mType = MathLib::value::FLOAT;
+        mType = MathLib::value::Type::FLOAT;
         mDoubleValue = MathLib::toDoubleNumber(s);
         return;
     }
@@ -52,7 +56,7 @@ MathLib::value::value(const std::string &s) :
     if (!MathLib::isInt(s))
         throw InternalError(nullptr, "Invalid value: " + s);
 
-    mType = MathLib::value::INT;
+    mType = MathLib::value::Type::INT;
     mIntValue = MathLib::toLongNumber(s);
 
     if (isIntHex(s) && mIntValue < 0)
@@ -65,12 +69,12 @@ MathLib::value::value(const std::string &s) :
             if (c == 'u' || c == 'U')
                 mIsUnsigned = true;
             else if (c == 'l' || c == 'L') {
-                if (mType == MathLib::value::INT)
-                    mType = MathLib::value::LONG;
-                else if (mType == MathLib::value::LONG)
-                    mType = MathLib::value::LONGLONG;
+                if (mType == MathLib::value::Type::INT)
+                    mType = MathLib::value::Type::LONG;
+                else if (mType == MathLib::value::Type::LONG)
+                    mType = MathLib::value::Type::LONGLONG;
             } else if (i > 2U && c == '4' && s[i-1] == '6' && s[i-2] == 'i')
-                mType = MathLib::value::LONGLONG;
+                mType = MathLib::value::Type::LONGLONG;
         }
     }
 }
@@ -78,7 +82,7 @@ MathLib::value::value(const std::string &s) :
 std::string MathLib::value::str() const
 {
     std::ostringstream ostr;
-    if (mType == MathLib::value::FLOAT) {
+    if (mType == MathLib::value::Type::FLOAT) {
         if (ISNAN(mDoubleValue))
             return "nan.0";
         if (ISINF(mDoubleValue))
@@ -102,9 +106,9 @@ std::string MathLib::value::str() const
         ostr << static_cast<biguint>(mIntValue) << "U";
     else
         ostr << mIntValue;
-    if (mType == MathLib::value::LONG)
+    if (mType == MathLib::value::Type::LONG)
         ostr << "L";
-    else if (mType == MathLib::value::LONGLONG)
+    else if (mType == MathLib::value::Type::LONGLONG)
         ostr << "LL";
     return ostr.str();
 }
@@ -121,7 +125,7 @@ void MathLib::value::promote(const MathLib::value &v)
     } else if (!isFloat()) {
         mIsUnsigned = false;
         mDoubleValue = mIntValue;
-        mType = MathLib::value::FLOAT;
+        mType = MathLib::value::Type::FLOAT;
     }
 }
 
@@ -294,25 +298,22 @@ MathLib::biguint MathLib::toULongNumber(const std::string & str)
 {
     // hexadecimal numbers:
     if (isIntHex(str)) {
-        if (str[0] == '-') {
-            biguint ret = 0;
-            std::istringstream istr(str);
-            istr >> std::hex >> ret;
+        try {
+            const biguint ret = std::stoull(str, nullptr, 16);
             return ret;
-        } else {
-            unsigned long long ret = 0;
-            std::istringstream istr(str);
-            istr >> std::hex >> ret;
-            return (biguint)ret;
+        } catch (const std::out_of_range& e) {
+            throw InternalError(nullptr, "Internal Error. MathLib::toULongNumber: out_of_range: " + str + " (" + e.what() +")");
         }
     }
 
     // octal numbers:
     if (isOct(str)) {
-        biguint ret = 0;
-        std::istringstream istr(str);
-        istr >> std::oct >> ret;
-        return ret;
+        try {
+            const biguint ret = std::stoull(str, nullptr, 8);
+            return ret;
+        } catch (const std::out_of_range& e) {
+            throw InternalError(nullptr, "Internal Error. MathLib::toULongNumber: out_of_range: " + str + " (" + e.what() +")");
+        }
     }
 
     // binary numbers:
@@ -340,13 +341,15 @@ MathLib::biguint MathLib::toULongNumber(const std::string & str)
             return static_cast<biguint>(doubleval);
     }
 
-    biguint ret = 0;
-    std::istringstream istr(str);
-    istr >> ret;
-    return ret;
+    try {
+        const biguint ret = std::stoull(str, nullptr, 10);
+        return ret;
+    } catch (const std::out_of_range& e) {
+        throw InternalError(nullptr, "Internal Error. MathLib::toULongNumber: out_of_range: " + str + " (" + e.what() +")");
+    }
 }
 
-static unsigned int encodeMultiChar(const std::string& str)
+unsigned int MathLib::encodeMultiChar(const std::string& str)
 {
     unsigned int retval = 0;
     for (char it : str) {
@@ -355,157 +358,26 @@ static unsigned int encodeMultiChar(const std::string& str)
     return retval;
 }
 
-static bool isoctal(int c)
-{
-    return c>='0' && c<='7';
-}
-
-MathLib::bigint MathLib::characterLiteralToLongNumber(const std::string& str)
-{
-    if (str.empty())
-        return 0; // <- only possible in unit testing
-
-    // '\xF6'
-    if (str.size() == 4 && str.compare(0,2,"\\x")==0 && std::isxdigit(str[2]) && std::isxdigit(str[3])) {
-        return std::strtoul(str.substr(2).c_str(), nullptr, 16);
-    }
-
-    // '\123'
-    if (str.size() == 4 && str[0] == '\\' && isoctal(str[1]) && isoctal(str[2]) && isoctal(str[3])) {
-        return (char)std::strtoul(str.substr(1).c_str(), nullptr, 8);
-    }
-
-    // C99 6.4.4.4
-    // The value of an integer character constant containing more than one character (e.g., 'ab'),
-    // or containing a character or escape sequence that does not map to a single-byte execution character,
-    // is implementation-defined.
-    // clang and gcc seem to use the following encoding: 'AB' as (('A' << 8) | 'B')
-    const std::string& normStr = normalizeCharacterLiteral(str);
-    return encodeMultiChar(normStr);
-}
-
-std::string MathLib::normalizeCharacterLiteral(const std::string& iLiteral)
-{
-    std::string normalizedLiteral;
-    const std::string::size_type iLiteralLen = iLiteral.size();
-    for (std::string::size_type idx = 0; idx < iLiteralLen ; ++idx) {
-        if (iLiteral[idx] != '\\') {
-            normalizedLiteral.push_back(iLiteral[idx]);
-            continue;
-        }
-        ++idx;
-        if (idx == iLiteralLen) {
-            throw InternalError(nullptr, "Internal Error. MathLib::normalizeCharacterLiteral: Unhandled char constant '" + iLiteral + "'.");
-        }
-        switch (iLiteral[idx]) {
-        case 'x':
-            // Hexa-decimal number: skip \x and interpret the next two characters
-        {
-            if (++idx == iLiteralLen)
-                throw InternalError(nullptr, "Internal Error. MathLib::normalizeCharacterLiteral: Unhandled char constant '" + iLiteral + "'.");
-            std::string tempBuf;
-            tempBuf.push_back(iLiteral[idx]);
-            if (++idx != iLiteralLen)
-                tempBuf.push_back(iLiteral[idx]);
-            normalizedLiteral.push_back(static_cast<char>(MathLib::toULongNumber("0x" + tempBuf)));
-            continue;
-        }
-        case 'u':
-        case 'U':
-            // Unicode string; just skip the \u or \U
-            if (idx + 1 == iLiteralLen)
-                throw InternalError(nullptr, "Internal Error. MathLib::characterLiteralToLongNumber: Unhandled char constant '" + iLiteral + "'.");
-            continue;
-        }
-        // Single digit octal number
-        if (1 == iLiteralLen - idx) {
-            switch (iLiteral[idx]) {
-            case '0':
-            case '1':
-            case '2':
-            case '3':
-            case '4':
-            case '5':
-            case '6':
-            case '7':
-                normalizedLiteral.push_back(iLiteral[idx]-'0');
-                break;
-            case 'a':
-                normalizedLiteral.push_back('\a');
-                break;
-            case 'b':
-                normalizedLiteral.push_back('\b');
-                break;
-            case 'e':
-                normalizedLiteral.push_back(0x1B); // clang, gcc, tcc interpnormalizedLiteral this as 0x1B - escape character
-                break;
-            case 'f':
-                normalizedLiteral.push_back('\f');
-                break;
-            case 'n':
-                normalizedLiteral.push_back('\n');
-                break;
-            case 'r':
-                normalizedLiteral.push_back('\r');
-                break;
-            case 't':
-                normalizedLiteral.push_back('\t');
-                break;
-            case 'v':
-                normalizedLiteral.push_back('\v');
-                break;
-            case '\\':
-            case '\?':
-            case '\'':
-            case '\"':
-                normalizedLiteral.push_back(iLiteral[idx]);
-                break;
-            default:
-                throw InternalError(nullptr, "Internal Error. MathLib::normalizeCharacterLiteral: Unhandled char constant '" + iLiteral + "'.");
-            }
-            continue;
-        }
-        // 2-3 digit octal number
-        if (!MathLib::isOctalDigit(iLiteral[idx]))
-            throw InternalError(nullptr, "Internal Error. MathLib::normalizeCharacterLiteral: Unhandled char constant '" + iLiteral + "'.");
-        std::string tempBuf;
-        tempBuf.push_back(iLiteral[idx]);
-        ++idx;
-        if (MathLib::isOctalDigit(iLiteral[idx])) {
-            tempBuf.push_back(iLiteral[idx]);
-            ++idx;
-            if (MathLib::isOctalDigit(iLiteral[idx])) {
-                tempBuf.push_back(iLiteral[idx]);
-            }
-        }
-        normalizedLiteral.push_back(static_cast<char>(MathLib::toLongNumber("0" + tempBuf)));
-    }
-    return normalizedLiteral;
-}
-
 MathLib::bigint MathLib::toLongNumber(const std::string & str)
 {
     // hexadecimal numbers:
     if (isIntHex(str)) {
-        if (str[0] == '-') {
-            bigint ret = 0;
-            std::istringstream istr(str);
-            istr >> std::hex >> ret;
-            return ret;
-        } else {
-            unsigned long long ret = 0;
-            std::istringstream istr(str);
-            istr >> std::hex >> ret;
+        try {
+            const biguint ret = std::stoull(str, nullptr, 16);
             return (bigint)ret;
+        } catch (const std::out_of_range& e) {
+            throw InternalError(nullptr, "Internal Error. MathLib::toLongNumber: out_of_range: " + str + " (" + e.what() +")");
         }
     }
 
     // octal numbers:
     if (isOct(str)) {
-        bigint ret = 0;
-        std::istringstream istr(str);
-        istr >> std::oct >> ret;
-        return ret;
+        try {
+            const biguint ret = std::stoull(str, nullptr, 8);
+            return ret;
+        } catch (const std::out_of_range& e) {
+            throw InternalError(nullptr, "Internal Error. MathLib::toLongNumber: out_of_range: " + str + " (" + e.what() +")");
+        }
     }
 
     // binary numbers:
@@ -535,21 +407,15 @@ MathLib::bigint MathLib::toLongNumber(const std::string & str)
             return static_cast<bigint>(doubleval);
     }
 
-    if (isCharLiteral(str)) {
-        return characterLiteralToLongNumber(getCharLiteral(str));
-    }
+    if (isCharLiteral(str))
+        return simplecpp::characterLiteralToLL(str);
 
-    if (str[0] == '-') {
-        bigint ret = 0;
-        std::istringstream istr(str);
-        istr >> ret;
+    try {
+        const biguint ret = std::stoull(str, nullptr, 10);
         return ret;
+    } catch (const std::out_of_range& e) {
+        throw InternalError(nullptr, "Internal Error. MathLib::toLongNumber: out_of_range: " + str + " (" + e.what() +")");
     }
-
-    biguint ret = 0;
-    std::istringstream istr(str);
-    istr >> ret;
-    return ret;
 }
 
 // in-place conversion of (sub)string to double. Requires no heap.
@@ -569,7 +435,7 @@ static double myStod(const std::string& str, std::string::const_iterator from, s
     int distance;
     if (std::string::npos == decimalsep) {
         distance = to - it;
-    } else  if (decimalsep > (to - str.begin()))
+    } else if (decimalsep > (to - str.begin()))
         return 0.; // error handling??
     else
         distance = int(decimalsep)-(from - str.begin());
@@ -586,10 +452,9 @@ static double myStod(const std::string& str, std::string::const_iterator from, s
             --distance;
         result += digitval(*it)* std::pow(base, distance);
     }
-    return (positivesign)?result:-result;
+    return positivesign ? result : -result;
 }
 
-
 // Assuming a limited support of built-in hexadecimal floats (see C99, C++17) that is a fall-back implementation.
 // Performance has been optimized WRT to heap activity, however the calculation part is not optimized.
 static double floatHexToDoubleNumber(const std::string& str)
@@ -597,21 +462,26 @@ static double floatHexToDoubleNumber(const std::string& str)
     const std::size_t p = str.find_first_of("pP",3);
     const double factor1 = myStod(str, str.begin() + 2, str.begin()+p, 16);
     const bool suffix = (str.back() == 'f') || (str.back() == 'F') || (str.back() == 'l') || (str.back() == 'L');
-    const double exponent = myStod(str, str.begin() + p + 1, (suffix)?str.end()-1:str.end(), 10);
+    const double exponent = myStod(str, str.begin() + p + 1, suffix ? str.end()-1:str.end(), 10);
     const double factor2 = std::pow(2, exponent);
     return factor1 * factor2;
 }
 
 double MathLib::toDoubleNumber(const std::string &str)
 {
-    if (isCharLiteral(str))
-        return characterLiteralToLongNumber(getCharLiteral(str));
+    if (isCharLiteral(str)) {
+        try {
+            return simplecpp::characterLiteralToLL(str);
+        } catch (const std::exception& e) {
+            throw InternalError(nullptr, "Internal Error. MathLib::toLongNumber: characterLiteralToLL(" + str + ") => " + e.what());
+        }
+    }
     if (isIntHex(str))
         return static_cast<double>(toLongNumber(str));
     // nullcheck
     if (isNullValue(str))
         return 0.0;
-#ifdef __clang__
+#ifdef _LIBCPP_VERSION
     if (isFloat(str)) // Workaround libc++ bug at http://llvm.org/bugs/show_bug.cgi?id=17782
         // TODO : handle locale
         return std::strtod(str.c_str(), nullptr);
@@ -626,7 +496,7 @@ double MathLib::toDoubleNumber(const std::string &str)
     return ret;
 }
 
-template<> std::string MathLib::toString(double value)
+template<> std::string MathLib::toString<double>(double value)
 {
     std::ostringstream result;
     result.precision(12);
@@ -746,70 +616,70 @@ bool MathLib::isPositive(const std::string &str)
 
 static bool isValidIntegerSuffixIt(std::string::const_iterator it, std::string::const_iterator end, bool supportMicrosoftExtensions=true)
 {
-    enum { START, SUFFIX_U, SUFFIX_UL, SUFFIX_ULL, SUFFIX_L, SUFFIX_LU, SUFFIX_LL, SUFFIX_LLU, SUFFIX_I, SUFFIX_I6, SUFFIX_I64, SUFFIX_UI, SUFFIX_UI6, SUFFIX_UI64 } state = START;
+    enum class Status { START, SUFFIX_U, SUFFIX_UL, SUFFIX_ULL, SUFFIX_L, SUFFIX_LU, SUFFIX_LL, SUFFIX_LLU, SUFFIX_I, SUFFIX_I6, SUFFIX_I64, SUFFIX_UI, SUFFIX_UI6, SUFFIX_UI64 } state = Status::START;
     for (; it != end; ++it) {
         switch (state) {
-        case START:
+        case Status::START:
             if (*it == 'u' || *it == 'U')
-                state = SUFFIX_U;
+                state = Status::SUFFIX_U;
             else if (*it == 'l' || *it == 'L')
-                state = SUFFIX_L;
+                state = Status::SUFFIX_L;
             else if (supportMicrosoftExtensions && (*it == 'i' || *it == 'I'))
-                state = SUFFIX_I;
+                state = Status::SUFFIX_I;
             else
                 return false;
             break;
-        case SUFFIX_U:
+        case Status::SUFFIX_U:
             if (*it == 'l' || *it == 'L')
-                state = SUFFIX_UL; // UL
+                state = Status::SUFFIX_UL; // UL
             else if (supportMicrosoftExtensions && (*it == 'i' || *it == 'I'))
-                state = SUFFIX_UI;
+                state = Status::SUFFIX_UI;
             else
                 return false;
             break;
-        case SUFFIX_UL:
+        case Status::SUFFIX_UL:
             if (*it == 'l' || *it == 'L')
-                state = SUFFIX_ULL; // ULL
+                state = Status::SUFFIX_ULL; // ULL
             else
                 return false;
             break;
-        case SUFFIX_L:
+        case Status::SUFFIX_L:
             if (*it == 'u' || *it == 'U')
-                state = SUFFIX_LU; // LU
+                state = Status::SUFFIX_LU; // LU
             else if (*it == 'l' || *it == 'L')
-                state = SUFFIX_LL; // LL
+                state = Status::SUFFIX_LL; // LL
             else
                 return false;
             break;
-        case SUFFIX_LU:
+        case Status::SUFFIX_LU:
             return false;
-        case SUFFIX_LL:
+        case Status::SUFFIX_LL:
             if (*it == 'u' || *it == 'U')
-                state = SUFFIX_LLU; // LLU
+                state = Status::SUFFIX_LLU; // LLU
             else
                 return false;
             break;
-        case SUFFIX_I:
+        case Status::SUFFIX_I:
             if (*it == '6')
-                state = SUFFIX_I6;
+                state = Status::SUFFIX_I6;
             else
                 return false;
             break;
-        case SUFFIX_I6:
+        case Status::SUFFIX_I6:
             if (*it == '4')
-                state = SUFFIX_I64;
+                state = Status::SUFFIX_I64;
             else
                 return false;
             break;
-        case SUFFIX_UI:
+        case Status::SUFFIX_UI:
             if (*it == '6')
-                state = SUFFIX_UI6;
+                state = Status::SUFFIX_UI6;
             else
                 return false;
             break;
-        case SUFFIX_UI6:
+        case Status::SUFFIX_UI6:
             if (*it == '4')
-                state = SUFFIX_UI64;
+                state = Status::SUFFIX_UI64;
             else
                 return false;
             break;
@@ -817,17 +687,18 @@ static bool isValidIntegerSuffixIt(std::string::const_iterator it, std::string::
             return false;
         }
     }
-    return ((state == SUFFIX_U) ||
-            (state == SUFFIX_L) ||
-            (state == SUFFIX_UL) ||
-            (state == SUFFIX_LU) ||
-            (state == SUFFIX_LL) ||
-            (state == SUFFIX_ULL) ||
-            (state == SUFFIX_LLU) ||
-            (state == SUFFIX_I64) ||
-            (state == SUFFIX_UI64));
+    return ((state == Status::SUFFIX_U) ||
+            (state == Status::SUFFIX_L) ||
+            (state == Status::SUFFIX_UL) ||
+            (state == Status::SUFFIX_LU) ||
+            (state == Status::SUFFIX_LL) ||
+            (state == Status::SUFFIX_ULL) ||
+            (state == Status::SUFFIX_LLU) ||
+            (state == Status::SUFFIX_I64) ||
+            (state == Status::SUFFIX_UI64));
 }
 
+// cppcheck-suppress unusedFunction
 bool MathLib::isValidIntegerSuffix(const std::string& str, bool supportMicrosoftExtensions)
 {
     return isValidIntegerSuffixIt(str.begin(), str.end(), supportMicrosoftExtensions);
@@ -881,9 +752,9 @@ bool MathLib::isOct(const std::string& str)
 
 bool MathLib::isIntHex(const std::string& str)
 {
-    enum Status {
+    enum class Status {
         START, HEX_0, HEX_X, DIGIT
-    } state = START;
+    } state = Status::START;
     if (str.empty())
         return false;
     std::string::const_iterator it = str.begin();
@@ -891,40 +762,40 @@ bool MathLib::isIntHex(const std::string& str)
         ++it;
     for (; it != str.end(); ++it) {
         switch (state) {
-        case START:
+        case Status::START:
             if (*it == '0')
-                state = HEX_0;
+                state = Status::HEX_0;
             else
                 return false;
             break;
-        case HEX_0:
+        case Status::HEX_0:
             if (*it == 'x' || *it == 'X')
-                state = HEX_X;
+                state = Status::HEX_X;
             else
                 return false;
             break;
-        case HEX_X:
+        case Status::HEX_X:
             if (isxdigit(static_cast<unsigned char>(*it)))
-                state = DIGIT;
+                state = Status::DIGIT;
             else
                 return false;
             break;
-        case DIGIT:
+        case Status::DIGIT:
             if (isxdigit(static_cast<unsigned char>(*it)))
-                ; //  state = DIGIT;
+                ; //  state = Status::DIGIT;
             else
                 return isValidIntegerSuffixIt(it,str.end());
             break;
         }
     }
-    return DIGIT==state;
+    return Status::DIGIT == state;
 }
 
 bool MathLib::isFloatHex(const std::string& str)
 {
-    enum Status {
+    enum class Status {
         START, HEX_0, HEX_X, WHOLE_NUMBER_DIGIT, POINT, FRACTION, EXPONENT_P, EXPONENT_SIGN, EXPONENT_DIGITS, EXPONENT_SUFFIX
-    } state = START;
+    } state = Status::START;
     if (str.empty())
         return false;
     std::string::const_iterator it = str.begin();
@@ -932,72 +803,72 @@ bool MathLib::isFloatHex(const std::string& str)
         ++it;
     for (; it != str.end(); ++it) {
         switch (state) {
-        case START:
+        case Status::START:
             if (*it == '0')
-                state = HEX_0;
+                state = Status::HEX_0;
             else
                 return false;
             break;
-        case HEX_0:
+        case Status::HEX_0:
             if (*it == 'x' || *it == 'X')
-                state = HEX_X;
+                state = Status::HEX_X;
             else
                 return false;
             break;
-        case HEX_X:
+        case Status::HEX_X:
             if (isxdigit(static_cast<unsigned char>(*it)))
-                state = WHOLE_NUMBER_DIGIT;
+                state = Status::WHOLE_NUMBER_DIGIT;
             else if (*it == '.')
-                state = POINT;
+                state = Status::POINT;
             else
                 return false;
             break;
-        case WHOLE_NUMBER_DIGIT:
+        case Status::WHOLE_NUMBER_DIGIT:
             if (isxdigit(static_cast<unsigned char>(*it)))
-                ; // state = WHOLE_NUMBER_DIGITS;
+                ; // state = Status::WHOLE_NUMBER_DIGITS;
             else if (*it=='.')
-                state = FRACTION;
+                state = Status::FRACTION;
             else if (*it=='p' || *it=='P')
-                state = EXPONENT_P;
+                state = Status::EXPONENT_P;
             else
                 return false;
             break;
-        case POINT:
-        case FRACTION:
+        case Status::POINT:
+        case Status::FRACTION:
             if (isxdigit(static_cast<unsigned char>(*it)))
-                state=FRACTION;
+                state = Status::FRACTION;
             else if (*it == 'p' || *it == 'P')
-                state = EXPONENT_P;
+                state = Status::EXPONENT_P;
             else
                 return false;
             break;
-        case EXPONENT_P:
+        case Status::EXPONENT_P:
             if (isdigit(static_cast<unsigned char>(*it)))
-                state = EXPONENT_DIGITS;
+                state = Status::EXPONENT_DIGITS;
             else if (*it == '+' || *it == '-')
-                state = EXPONENT_SIGN;
+                state = Status::EXPONENT_SIGN;
             else
                 return false;
             break;
-        case EXPONENT_SIGN:
+        case Status::EXPONENT_SIGN:
             if (isdigit(static_cast<unsigned char>(*it)))
-                state = EXPONENT_DIGITS;
+                state = Status::EXPONENT_DIGITS;
             else
                 return false;
             break;
-        case EXPONENT_DIGITS:
+        case Status::EXPONENT_DIGITS:
             if (isdigit(static_cast<unsigned char>(*it)))
-                ; //  state = EXPONENT_DIGITS;
+                ; //  state = Status::EXPONENT_DIGITS;
             else if (*it == 'f' || *it == 'F' || *it == 'l' || *it == 'L')
-                state = EXPONENT_SUFFIX;
+                state = Status::EXPONENT_SUFFIX;
             else
                 return false;
             break;
-        case EXPONENT_SUFFIX:
+        case Status::EXPONENT_SUFFIX:
             return false;
         }
     }
-    return (EXPONENT_DIGITS==state) || (EXPONENT_SUFFIX == state);
+    return (Status::EXPONENT_DIGITS == state) || (Status::EXPONENT_SUFFIX == state);
 }
 
 
@@ -1012,9 +883,9 @@ bool MathLib::isFloatHex(const std::string& str)
  **/
 bool MathLib::isBin(const std::string& str)
 {
-    enum Status {
+    enum class Status {
         START, GNU_BIN_PREFIX_0, GNU_BIN_PREFIX_B, DIGIT
-    } state = START;
+    } state = Status::START;
     if (str.empty())
         return false;
     std::string::const_iterator it = str.begin();
@@ -1022,40 +893,40 @@ bool MathLib::isBin(const std::string& str)
         ++it;
     for (; it != str.end(); ++it) {
         switch (state) {
-        case START:
+        case Status::START:
             if (*it == '0')
-                state = GNU_BIN_PREFIX_0;
+                state = Status::GNU_BIN_PREFIX_0;
             else
                 return false;
             break;
-        case GNU_BIN_PREFIX_0:
+        case Status::GNU_BIN_PREFIX_0:
             if (*it == 'b' || *it == 'B')
-                state = GNU_BIN_PREFIX_B;
+                state = Status::GNU_BIN_PREFIX_B;
             else
                 return false;
             break;
-        case GNU_BIN_PREFIX_B:
+        case Status::GNU_BIN_PREFIX_B:
             if (*it == '0' || *it == '1')
-                state = DIGIT;
+                state = Status::DIGIT;
             else
                 return false;
             break;
-        case DIGIT:
+        case Status::DIGIT:
             if (*it == '0' || *it == '1')
-                ; //  state = DIGIT;
+                ; //  state = Status::DIGIT;
             else
                 return isValidIntegerSuffixIt(it,str.end());
             break;
         }
     }
-    return state == DIGIT;
+    return state == Status::DIGIT;
 }
 
 bool MathLib::isDec(const std::string & str)
 {
-    enum Status {
+    enum class Status {
         START, DIGIT
-    } state = START;
+    } state = Status::START;
     if (str.empty())
         return false;
     std::string::const_iterator it = str.begin();
@@ -1063,21 +934,21 @@ bool MathLib::isDec(const std::string & str)
         ++it;
     for (; it != str.end(); ++it) {
         switch (state) {
-        case START:
+        case Status::START:
             if (isdigit(static_cast<unsigned char>(*it)))
-                state = DIGIT;
+                state = Status::DIGIT;
             else
                 return false;
             break;
-        case DIGIT:
+        case Status::DIGIT:
             if (isdigit(static_cast<unsigned char>(*it)))
-                state = DIGIT;
+                state = Status::DIGIT;
             else
                 return isValidIntegerSuffixIt(it,str.end());
             break;
         }
     }
-    return state == DIGIT;
+    return state == Status::DIGIT;
 }
 
 bool MathLib::isInt(const std::string & str)
@@ -1161,7 +1032,7 @@ std::string MathLib::subtract(const std::string &first, const std::string &secon
     }
 
     if (first == second)
-        return "0.0" ;
+        return "0.0";
 
     double d1 = toDoubleNumber(first);
     double d2 = toDoubleNumber(second);
@@ -1310,16 +1181,19 @@ bool MathLib::isNotEqual(const std::string &first, const std::string &second)
     return !isEqual(first, second);
 }
 
+// cppcheck-suppress unusedFunction
 bool MathLib::isGreater(const std::string &first, const std::string &second)
 {
     return toDoubleNumber(first) > toDoubleNumber(second);
 }
 
+// cppcheck-suppress unusedFunction
 bool MathLib::isGreaterEqual(const std::string &first, const std::string &second)
 {
     return toDoubleNumber(first) >= toDoubleNumber(second);
 }
 
+// cppcheck-suppress unusedFunction
 bool MathLib::isLess(const std::string &first, const std::string &second)
 {
     return toDoubleNumber(first) < toDoubleNumber(second);
