@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2020 Cppcheck team.
+ * Copyright (C) 2007-2022 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,13 +27,15 @@
 #include "templatesimplifier.h"
 #include "utils.h"
 
-#include <algorithm>
+#include <cstdint>
 #include <cstddef>
 #include <functional>
 #include <list>
 #include <memory>
 #include <ostream>
+#include <set>
 #include <string>
+#include <utility>
 #include <vector>
 
 class Enumerator;
@@ -43,6 +45,9 @@ class Settings;
 class Type;
 class ValueType;
 class Variable;
+class TokenList;
+class ConstTokenRange;
+class Token;
 
 /**
  * @brief This struct stores pointers to the front and back tokens of the list this token is in.
@@ -50,6 +55,7 @@ class Variable;
 struct TokensFrontBack {
     Token *front;
     Token *back;
+    const TokenList* list;
 };
 
 struct ScopeInfo2 {
@@ -64,6 +70,7 @@ struct TokenImpl {
     nonneg int mFileIndex;
     nonneg int mLineNumber;
     nonneg int mColumn;
+    nonneg int mExprId;
 
     // AST..
     Token *mAstOperand1;
@@ -100,11 +107,8 @@ struct TokenImpl {
     std::list<ValueFlow::Value>* mValues;
     static const std::list<ValueFlow::Value> mEmptyValueList;
 
-    /** Bitfield bit count. */
-    unsigned char mBits;
-
     // Pointer to a template in the template simplifier
-    std::set<TemplateSimplifier::TokenAndName*> mTemplateSimplifierPointers;
+    std::set<TemplateSimplifier::TokenAndName*>* mTemplateSimplifierPointers;
 
     // Pointer to the object representing this token's scope
     std::shared_ptr<ScopeInfo2> mScopeInfo;
@@ -120,6 +124,9 @@ struct TokenImpl {
     // For memoization, to speed up parsing of huge arrays #8897
     enum class Cpp11init {UNKNOWN, CPP11INIT, NOINIT} mCpp11init;
 
+    /** Bitfield bit count. */
+    unsigned char mBits;
+
     void setCppcheckAttribute(CppcheckAttributes::Type type, MathLib::bigint value);
     bool getCppcheckAttribute(CppcheckAttributes::Type type, MathLib::bigint *value) const;
 
@@ -128,6 +135,7 @@ struct TokenImpl {
         , mFileIndex(0)
         , mLineNumber(0)
         , mColumn(0)
+        , mExprId(0)
         , mAstOperand1(nullptr)
         , mAstOperand2(nullptr)
         , mAstParent(nullptr)
@@ -138,11 +146,11 @@ struct TokenImpl {
         , mOriginalName(nullptr)
         , mValueType(nullptr)
         , mValues(nullptr)
-        , mBits(0)
-        , mTemplateSimplifierPointers()
+        , mTemplateSimplifierPointers(nullptr)
         , mScopeInfo(nullptr)
         , mCppcheckAttributes(nullptr)
         , mCpp11init(Cpp11init::UNKNOWN)
+        , mBits(0)
     {}
 
     ~TokenImpl();
@@ -184,6 +192,8 @@ public:
     explicit Token(TokensFrontBack *tokensFrontBack = nullptr);
     ~Token();
 
+    ConstTokenRange until(const Token * t) const;
+
     template<typename T>
     void str(T&& s) {
         mStr = s;
@@ -208,8 +218,8 @@ public:
     void deleteNext(nonneg int count = 1);
 
     /**
-    * Unlink and delete the previous 'count' tokens.
-    */
+     * Unlink and delete the previous 'count' tokens.
+     */
     void deletePrevious(nonneg int count = 1);
 
     /**
@@ -261,7 +271,12 @@ public:
      * @return true if given token matches with given pattern
      *         false if given token does not match with given pattern
      */
-    static bool simpleMatch(const Token *tok, const char pattern[]);
+    template<size_t count>
+    static bool simpleMatch(const Token *tok, const char (&pattern)[count]) {
+        return simpleMatch(tok, pattern, count-1);
+    }
+
+    static bool simpleMatch(const Token *tok, const char pattern[], size_t pattern_len);
 
     /**
      * Match given token (or list of tokens) to a pattern list.
@@ -334,7 +349,7 @@ public:
      * @param tok token with C-string
      * @param settings Settings
      **/
-    static nonneg int getStrSize(const Token *tok, const Settings *const);
+    static nonneg int getStrSize(const Token *tok, const Settings *const settings);
 
     /**
      * @return char of C-string at index (possible escaped "\\n")
@@ -373,12 +388,6 @@ public:
                                         mTokType == eBoolean || mTokType == eLiteral || mTokType == eEnumerator);
         setFlag(fIsLiteral, memoizedIsLiteral);
     }
-    void isKeyword(const bool kwd) {
-        if (kwd)
-            tokType(eKeyword);
-        else if (mTokType == eKeyword)
-            tokType(eName);
-    }
     bool isKeyword() const {
         return mTokType == eKeyword;
     }
@@ -425,6 +434,9 @@ public:
     bool isBoolean() const {
         return mTokType == eBoolean;
     }
+    bool isIncDecOp() const {
+        return mTokType == eIncDecOp;
+    }
     bool isBinaryOp() const {
         return astOperand1() != nullptr && astOperand2() != nullptr;
     }
@@ -541,11 +553,11 @@ public:
     void isAttributeNodiscard(const bool value) {
         setFlag(fIsAttributeNodiscard, value);
     }
-    bool isMaybeUnused() const {
-        return getFlag(fIsMaybeUnused);
+    bool isAttributeMaybeUnused() const {
+        return getFlag(fIsAttributeMaybeUnused);
     }
-    void isMaybeUnused(const bool value) {
-        setFlag(fIsMaybeUnused, value);
+    void isAttributeMaybeUnused(const bool value) {
+        setFlag(fIsAttributeMaybeUnused, value);
     }
     void setCppcheckAttribute(TokenImpl::CppcheckAttributes::Type type, MathLib::bigint value) {
         mImpl->setCppcheckAttribute(type, value);
@@ -553,6 +565,9 @@ public:
     bool getCppcheckAttribute(TokenImpl::CppcheckAttributes::Type type, MathLib::bigint *value) const {
         return mImpl->getCppcheckAttribute(type, value);
     }
+    bool hasCppcheckAttributes() const {
+        return nullptr != mImpl->mCppcheckAttributes;
+    }
     bool isControlFlowKeyword() const {
         return getFlag(fIsControlFlowKeyword);
     }
@@ -587,6 +602,13 @@ public:
         setFlag(fIncompleteVar, b);
     }
 
+    bool isIncompleteConstant() const {
+        return getFlag(fIsIncompleteConstant);
+    }
+    void isIncompleteConstant(bool b) {
+        setFlag(fIsIncompleteConstant, b);
+    }
+
     bool isConstexpr() const {
         return getFlag(fConstexpr);
     }
@@ -601,6 +623,54 @@ public:
         setFlag(fExternC, b);
     }
 
+    bool isSplittedVarDeclComma() const {
+        return getFlag(fIsSplitVarDeclComma);
+    }
+    void isSplittedVarDeclComma(bool b) {
+        setFlag(fIsSplitVarDeclComma, b);
+    }
+
+    bool isSplittedVarDeclEq() const {
+        return getFlag(fIsSplitVarDeclEq);
+    }
+    void isSplittedVarDeclEq(bool b) {
+        setFlag(fIsSplitVarDeclEq, b);
+    }
+
+    bool isImplicitInt() const {
+        return getFlag(fIsImplicitInt);
+    }
+    void isImplicitInt(bool b) {
+        setFlag(fIsImplicitInt, b);
+    }
+
+    bool isInline() const {
+        return getFlag(fIsInline);
+    }
+    void isInline(bool b) {
+        setFlag(fIsInline, b);
+    }
+
+    bool isRemovedVoidParameter() const {
+        return getFlag(fIsRemovedVoidParameter);
+    }
+    void setRemovedVoidParameter(bool b) {
+        setFlag(fIsRemovedVoidParameter, b);
+    }
+
+    bool isTemplate() const {
+        return getFlag(fIsTemplate);
+    }
+    void isTemplate(bool b) {
+        setFlag(fIsTemplate, b);
+    }
+
+    bool isSimplifiedScope() const {
+        return getFlag(fIsSimplifedScope);
+    }
+    void isSimplifiedScope(bool b) {
+        setFlag(fIsSimplifedScope, b);
+    }
 
     bool isBitfield() const {
         return mImpl->mBits > 0;
@@ -608,11 +678,13 @@ public:
     unsigned char bits() const {
         return mImpl->mBits;
     }
-    std::set<TemplateSimplifier::TokenAndName*> &templateSimplifierPointers() const {
+    std::set<TemplateSimplifier::TokenAndName*>* templateSimplifierPointers() const {
         return mImpl->mTemplateSimplifierPointers;
     }
     void templateSimplifierPointer(TemplateSimplifier::TokenAndName* tokenAndName) {
-        mImpl->mTemplateSimplifierPointers.insert(tokenAndName);
+        if (!mImpl->mTemplateSimplifierPointers)
+            mImpl->mTemplateSimplifierPointers = new std::set<TemplateSimplifier::TokenAndName*>;
+        mImpl->mTemplateSimplifierPointers->insert(tokenAndName);
     }
     void setBits(const unsigned char b) {
         mImpl->mBits = b;
@@ -666,16 +738,36 @@ public:
         setFlag(fIsTemplateArg, value);
     }
 
-    static const Token *findsimplematch(const Token * const startTok, const char pattern[]);
-    static const Token *findsimplematch(const Token * const startTok, const char pattern[], const Token * const end);
+    template<size_t count>
+    static const Token *findsimplematch(const Token * const startTok, const char (&pattern)[count]) {
+        return findsimplematch(startTok, pattern, count-1);
+    }
+    static const Token *findsimplematch(const Token * const startTok, const char pattern[], size_t pattern_len);
+
+    template<size_t count>
+    static const Token *findsimplematch(const Token * const startTok, const char (&pattern)[count], const Token * const end) {
+        return findsimplematch(startTok, pattern, count-1, end);
+    }
+    static const Token *findsimplematch(const Token * const startTok, const char pattern[], size_t pattern_len, const Token * const end);
+
     static const Token *findmatch(const Token * const startTok, const char pattern[], const nonneg int varId = 0);
     static const Token *findmatch(const Token * const startTok, const char pattern[], const Token * const end, const nonneg int varId = 0);
-    static Token *findsimplematch(Token * const startTok, const char pattern[]) {
-        return const_cast<Token *>(findsimplematch(const_cast<const Token *>(startTok), pattern));
+
+    template<size_t count>
+    static Token *findsimplematch(Token * const startTok, const char (&pattern)[count]) {
+        return findsimplematch(startTok, pattern, count-1);
     }
-    static Token *findsimplematch(Token * const startTok, const char pattern[], const Token * const end) {
-        return const_cast<Token *>(findsimplematch(const_cast<const Token *>(startTok), pattern, end));
+    static Token *findsimplematch(Token * const startTok, const char pattern[], size_t pattern_len) {
+        return const_cast<Token *>(findsimplematch(const_cast<const Token *>(startTok), pattern, pattern_len));
     }
+    template<size_t count>
+    static Token *findsimplematch(Token * const startTok, const char (&pattern)[count], const Token * const end) {
+        return findsimplematch(startTok, pattern, count-1, end);
+    }
+    static Token *findsimplematch(Token * const startTok, const char pattern[], size_t pattern_len, const Token * const end) {
+        return const_cast<Token *>(findsimplematch(const_cast<const Token *>(startTok), pattern, pattern_len, end));
+    }
+
     static Token *findmatch(Token * const startTok, const char pattern[], const nonneg int varId = 0) {
         return const_cast<Token *>(findmatch(const_cast<const Token *>(startTok), pattern, varId));
     }
@@ -742,7 +834,12 @@ public:
      * @param prepend Insert the new token before this token when it's not
      * the first one on the tokens list.
      */
-    void insertToken(const std::string &tokenStr, const std::string &originalNameStr=emptyString, bool prepend=false);
+    Token* insertToken(const std::string& tokenStr, const std::string& originalNameStr = emptyString, bool prepend = false);
+
+    Token* insertTokenBefore(const std::string& tokenStr, const std::string& originalNameStr = emptyString)
+    {
+        return insertToken(tokenStr, originalNameStr, true);
+    }
 
     Token *previous() const {
         return mPrevious;
@@ -762,6 +859,15 @@ public:
         }
     }
 
+    nonneg int exprId() const {
+        if (mImpl->mExprId)
+            return mImpl->mExprId;
+        return mImpl->mVarId;
+    }
+    void exprId(nonneg int id) {
+        mImpl->mExprId = id;
+    }
+
     /**
      * For debugging purposes, prints token and all tokens
      * followed by it.
@@ -780,6 +886,11 @@ public:
      */
     void printOut(const char *title, const std::vector<std::string> &fileNames) const;
 
+    /**
+     * print out tokens - used for debugging
+     */
+    void printLines(int lines=5) const;
+
     /**
      * Replace token replaceThis with tokens between start and end,
      * including start and end. The replaceThis token is deleted.
@@ -789,18 +900,60 @@ public:
      */
     static void replace(Token *replaceThis, Token *start, Token *end);
 
+    struct stringifyOptions {
+        bool varid = false;
+        bool exprid = false;
+        bool idtype = false; // distinguish varid / exprid
+        bool attributes = false;
+        bool macro = false;
+        bool linenumbers = false;
+        bool linebreaks = false;
+        bool files = false;
+        static stringifyOptions forDebug() {
+            stringifyOptions options;
+            options.attributes = true;
+            options.macro = true;
+            options.linenumbers = true;
+            options.linebreaks = true;
+            options.files = true;
+            return options;
+        }
+        static stringifyOptions forDebugVarId() {
+            stringifyOptions options = forDebug();
+            options.varid = true;
+            return options;
+        }
+        static stringifyOptions forDebugExprId() {
+            stringifyOptions options = forDebug();
+            options.exprid = true;
+            return options;
+        }
+        static stringifyOptions forPrintOut() {
+            stringifyOptions options = forDebug();
+            options.exprid = true;
+            options.varid = true;
+            options.idtype = true;
+            return options;
+        }
+    };
+
+    std::string stringify(const stringifyOptions& options) const;
+
     /**
      * Stringify a token
-     * @param os The result is shifted into that output stream
-     * @param varid Print varids. (Style: "varname@id")
+     * @param varid Print varids. (Style: "varname\@id")
      * @param attributes Print attributes of tokens like "unsigned" in front of it.
      * @param macro Prints $ in front of the token if it was expanded from a macro.
      */
-    void stringify(std::ostream& os, bool varid, bool attributes, bool macro) const;
+    std::string stringify(bool varid, bool attributes, bool macro) const;
+
+    std::string stringifyList(const stringifyOptions& options, const std::vector<std::string>* fileNames = nullptr, const Token* end = nullptr) const;
+    std::string stringifyList(const Token* end, bool attributes = true) const;
+    std::string stringifyList(bool varid = false) const;
 
     /**
      * Stringify a list of token, from current instance on.
-     * @param varid Print varids. (Style: "varname@id")
+     * @param varid Print varids. (Style: "varname\@id")
      * @param attributes Print attributes of tokens like "unsigned" in front of it.
      * @param linenumbers Print line number in front of each line
      * @param linebreaks Insert "\\n" into string when line number changes
@@ -810,8 +963,6 @@ public:
      * @return Stringified token list as a string
      */
     std::string stringifyList(bool varid, bool attributes, bool linenumbers, bool linebreaks, bool files, const std::vector<std::string>* fileNames = nullptr, const Token* end = nullptr) const;
-    std::string stringifyList(const Token* end, bool attributes = true) const;
-    std::string stringifyList(bool varid = false) const;
 
     /**
      * Remove the contents for this token from the token list.
@@ -895,27 +1046,27 @@ public:
     }
 
     /**
-    * Associate this token with given type
-    * @param t Type to be associated
-    */
+     * Associate this token with given type
+     * @param t Type to be associated
+     */
     void type(const ::Type *t);
 
     /**
-    * @return a pointer to the type associated with this token.
-    */
+     * @return a pointer to the type associated with this token.
+     */
     const ::Type *type() const {
         return mTokType == eType ? mImpl->mType : nullptr;
     }
 
-    static const ::Type *typeOf(const Token *tok);
+    static const ::Type* typeOf(const Token* tok, const Token** typeTok = nullptr);
 
     static std::pair<const Token*, const Token*> typeDecl(const Token * tok);
 
     static std::string typeStr(const Token* tok);
 
     /**
-    * @return a pointer to the Enumerator associated with this token.
-    */
+     * @return a pointer to the Enumerator associated with this token.
+     */
     const Enumerator *enumerator() const {
         return mTokType == eEnumerator ? mImpl->mEnumerator : nullptr;
     }
@@ -977,10 +1128,10 @@ public:
     Token* nextArgumentBeforeCreateLinks2() const;
 
     /**
-    * @return the first token of the next template argument. Does only work on template argument
-    * lists. Requires that Tokenizer::createLinks2() has been called before.
-    * Returns 0, if there is no next argument.
-    */
+     * @return the first token of the next template argument. Does only work on template argument
+     * lists. Requires that Tokenizer::createLinks2() has been called before.
+     * Returns 0, if there is no next argument.
+     */
     Token* nextTemplateArgument() const;
 
     /**
@@ -1016,90 +1167,32 @@ public:
             *mImpl->mOriginalName = name;
     }
 
-    bool hasKnownIntValue() const {
-        if (!mImpl->mValues)
-            return false;
-        return std::any_of(mImpl->mValues->begin(), mImpl->mValues->end(), [](const ValueFlow::Value &value) {
-            return value.isKnown() && value.isIntValue();
-        });
-    }
-
-    bool hasKnownValue() const {
-        return mImpl->mValues && std::any_of(mImpl->mValues->begin(), mImpl->mValues->end(), std::mem_fn(&ValueFlow::Value::isKnown));
-    }
+    bool hasKnownIntValue() const;
+    bool hasKnownValue() const;
+    bool hasKnownValue(ValueFlow::Value::ValueType t) const;
+    bool hasKnownSymbolicValue(const Token* tok) const;
 
+    const ValueFlow::Value* getKnownValue(ValueFlow::Value::ValueType t) const;
     MathLib::bigint getKnownIntValue() const {
         return mImpl->mValues->front().intvalue;
     }
 
-    bool isImpossibleIntValue(const MathLib::bigint val) const {
-        if (!mImpl->mValues)
-            return false;
-        for (const auto &v: *mImpl->mValues) {
-            if (v.isIntValue() && v.isImpossible() && v.intvalue == val)
-                return true;
-            if (v.isIntValue() && v.bound == ValueFlow::Value::Bound::Lower && val > v.intvalue)
-                return true;
-            if (v.isIntValue() && v.bound == ValueFlow::Value::Bound::Upper && val < v.intvalue)
-                return true;
-        }
-        return false;
-    }
+    const ValueFlow::Value* getValue(const MathLib::bigint val) const;
 
-    const ValueFlow::Value * getValue(const MathLib::bigint val) const {
-        if (!mImpl->mValues)
-            return nullptr;
-        const auto it = std::find_if(mImpl->mValues->begin(), mImpl->mValues->end(), [=](const ValueFlow::Value& value) {
-            return value.isIntValue() && !value.isImpossible() && value.intvalue == val;
-        });
-        return it == mImpl->mValues->end() ? nullptr : &*it;
-    }
+    const ValueFlow::Value* getMaxValue(bool condition, MathLib::bigint path = 0) const;
 
-    const ValueFlow::Value * getMaxValue(bool condition) const {
-        if (!mImpl->mValues)
-            return nullptr;
-        const ValueFlow::Value *ret = nullptr;
-        for (const ValueFlow::Value &value : *mImpl->mValues) {
-            if (!value.isIntValue())
-                continue;
-            if (value.isImpossible())
-                continue;
-            if ((!ret || value.intvalue > ret->intvalue) &&
-                ((value.condition != nullptr) == condition))
-                ret = &value;
-        }
-        return ret;
-    }
-
-    const ValueFlow::Value * getMovedValue() const {
-        if (!mImpl->mValues)
-            return nullptr;
-        const auto it = std::find_if(mImpl->mValues->begin(), mImpl->mValues->end(), [](const ValueFlow::Value& value) {
-            return value.isMovedValue() && !value.isImpossible() &&
-                   value.moveKind != ValueFlow::Value::MoveKind::NonMovedVariable;
-        });
-        return it == mImpl->mValues->end() ? nullptr : &*it;
-    }
+    const ValueFlow::Value* getMovedValue() const;
 
     const ValueFlow::Value * getValueLE(const MathLib::bigint val, const Settings *settings) const;
     const ValueFlow::Value * getValueGE(const MathLib::bigint val, const Settings *settings) const;
 
     const ValueFlow::Value * getInvalidValue(const Token *ftok, nonneg int argnr, const Settings *settings) const;
 
-    const ValueFlow::Value * getContainerSizeValue(const MathLib::bigint val) const {
-        if (!mImpl->mValues)
-            return nullptr;
-        const auto it = std::find_if(mImpl->mValues->begin(), mImpl->mValues->end(), [=](const ValueFlow::Value& value) {
-            return value.isContainerSizeValue() && !value.isImpossible() && value.intvalue == val;
-        });
-        return it == mImpl->mValues->end() ? nullptr : &*it;
-    }
+    const ValueFlow::Value* getContainerSizeValue(const MathLib::bigint val) const;
 
     const Token *getValueTokenMaxStrLength() const;
     const Token *getValueTokenMinStrSize(const Settings *settings) const;
 
-    const Token *getValueTokenDeadPointer() const;
-
     /** Add token value. Return true if value is added. */
     bool addValue(const ValueFlow::Value &value);
 
@@ -1146,7 +1239,7 @@ private:
     Token *mPrevious;
     Token *mLink;
 
-    enum {
+    enum : uint64_t {
         fIsUnsigned             = (1 << 0),
         fIsSigned               = (1 << 1),
         fIsPointerCompare       = (1 << 2),
@@ -1163,24 +1256,32 @@ private:
         fIsAttributeNothrow     = (1 << 13), // __attribute__((nothrow)), __declspec(nothrow)
         fIsAttributeUsed        = (1 << 14), // __attribute__((used))
         fIsAttributePacked      = (1 << 15), // __attribute__((packed))
-        fIsControlFlowKeyword   = (1 << 16), // if/switch/while/...
-        fIsOperatorKeyword      = (1 << 17), // operator=, etc
-        fIsComplex              = (1 << 18), // complex/_Complex type
-        fIsEnumType             = (1 << 19), // enumeration type
-        fIsName                 = (1 << 20),
-        fIsLiteral              = (1 << 21),
-        fIsTemplateArg          = (1 << 22),
-        fIsAttributeNodiscard   = (1 << 23), // __attribute__ ((warn_unused_result)), [[nodiscard]]
-        fAtAddress              = (1 << 24), // @ 0x4000
-        fIncompleteVar          = (1 << 25),
-        fConstexpr              = (1 << 26),
-        fExternC                = (1 << 27),
-        fIsMaybeUnused          = (1 << 28), // [[maybe_unsed]]
+        fIsAttributeMaybeUnused = (1 << 16), // [[maybe_unsed]]
+        fIsControlFlowKeyword   = (1 << 17), // if/switch/while/...
+        fIsOperatorKeyword      = (1 << 18), // operator=, etc
+        fIsComplex              = (1 << 19), // complex/_Complex type
+        fIsEnumType             = (1 << 20), // enumeration type
+        fIsName                 = (1 << 21),
+        fIsLiteral              = (1 << 22),
+        fIsTemplateArg          = (1 << 23),
+        fIsAttributeNodiscard   = (1 << 24), // __attribute__ ((warn_unused_result)), [[nodiscard]]
+        fAtAddress              = (1 << 25), // @ 0x4000
+        fIncompleteVar          = (1 << 26),
+        fConstexpr              = (1 << 27),
+        fExternC                = (1 << 28),
+        fIsSplitVarDeclComma    = (1 << 29), // set to true when variable declarations are split up ('int a,b;' => 'int a; int b;')
+        fIsSplitVarDeclEq       = (1 << 30), // set to true when variable declaration with initialization is split up ('int a=5;' => 'int a; a=5;')
+        fIsImplicitInt          = (1U << 31),   // Is "int" token implicitly added?
+        fIsInline               = (1ULL << 32), // Is this a inline type
+        fIsTemplate             = (1ULL << 33),
+        fIsSimplifedScope       = (1ULL << 34), // scope added when simplifying e.g. if (int i = ...; ...)
+        fIsRemovedVoidParameter = (1ULL << 35), // A void function parameter has been removed
+        fIsIncompleteConstant   = (1ULL << 36),
     };
 
     Token::Type mTokType;
 
-    unsigned int mFlags;
+    uint64_t mFlags;
 
     TokenImpl *mImpl;
 
@@ -1189,7 +1290,7 @@ private:
      * @param flag_ flag to get state of
      * @return true if flag set or false in flag not set
      */
-    bool getFlag(unsigned int flag_) const {
+    bool getFlag(uint64_t flag_) const {
         return ((mFlags & flag_) != 0);
     }
 
@@ -1198,7 +1299,7 @@ private:
      * @param flag_ flag to set state
      * @param state_ new state of flag
      */
-    void setFlag(unsigned int flag_, bool state_) {
+    void setFlag(uint64_t flag_, bool state_) {
         mFlags = state_ ? mFlags | flag_ : mFlags & ~flag_;
     }
 
@@ -1218,6 +1319,7 @@ private:
 public:
     void astOperand1(Token *tok);
     void astOperand2(Token *tok);
+    void astParent(Token* tok);
 
     Token * astOperand1() {
         return mImpl->mAstOperand1;
@@ -1237,6 +1339,26 @@ public:
     const Token * astParent() const {
         return mImpl->mAstParent;
     }
+    Token * astSibling() {
+        if (!astParent())
+            return nullptr;
+        if (this == astParent()->astOperand1())
+            return astParent()->astOperand2();
+        else if (this == astParent()->astOperand2())
+            return astParent()->astOperand1();
+        return nullptr;
+
+    }
+    const Token * astSibling() const {
+        if (!astParent())
+            return nullptr;
+        if (this == astParent()->astOperand1())
+            return astParent()->astOperand2();
+        else if (this == astParent()->astOperand2())
+            return astParent()->astOperand1();
+        return nullptr;
+
+    }
     Token *astTop() {
         Token *ret = this;
         while (ret->mImpl->mAstParent)
@@ -1282,9 +1404,11 @@ public:
 
     std::string astStringVerbose() const;
 
+    std::string astStringZ3() const;
+
     std::string expressionString() const;
 
-    void printAst(bool verbose, bool xml, std::ostream &out) const;
+    void printAst(bool verbose, bool xml, const std::vector<std::string> &fileNames, std::ostream &out) const;
 
     void printValueFlow(bool xml, std::ostream &out) const;
 
@@ -1299,6 +1423,11 @@ public:
     }
 };
 
+Token* findTypeEnd(Token* tok);
+const Token* findTypeEnd(const Token* tok);
+Token* findLambdaEndScope(Token* tok);
+const Token* findLambdaEndScope(const Token* tok);
+
 /// @}
 //---------------------------------------------------------------------------
 #endif // tokenH
