@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2016 Cppcheck team.
+ * Copyright (C) 2007-2022 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,21 +16,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "tokenize.h"
-#include "check64bit.h"
-#include "testsuite.h"
 
+#include "check64bit.h"
+#include "errortypes.h"
+#include "settings.h"
+#include "testsuite.h"
+#include "tokenize.h"
+
+#include <iosfwd>
 
 class Test64BitPortability : public TestFixture {
 public:
-    Test64BitPortability() : TestFixture("Test64BitPortability") {
-    }
+    Test64BitPortability() : TestFixture("Test64BitPortability") {}
 
 private:
     Settings settings;
 
-    void run() {
-        settings.addEnabled("portability");
+    void run() override {
+        settings.severity.enable(Severity::portability);
 
         TEST_CASE(novardecl);
         TEST_CASE(functionpar);
@@ -38,22 +41,34 @@ private:
         TEST_CASE(ptrcompare);
         TEST_CASE(ptrarithmetic);
         TEST_CASE(returnIssues);
+        TEST_CASE(assignment);
     }
 
-    void check(const char code[]) {
+#define check(code) check_(code, __FILE__, __LINE__)
+    void check_(const char code[], const char* file, int line) {
         // Clear the error buffer..
         errout.str("");
 
         // Tokenize..
         Tokenizer tokenizer(&settings, this);
+        LOAD_LIB_2(settings.library, "std.cfg");
         std::istringstream istr(code);
-        tokenizer.tokenize(istr, "test.cpp");
+        ASSERT_LOC(tokenizer.tokenize(istr, "test.cpp"), file, line);
 
         // Check char variable usage..
         Check64BitPortability check64BitPortability(&tokenizer, &settings, this);
         check64BitPortability.pointerassignment();
     }
 
+    void assignment() {
+        // #8631
+        check("using CharArray = char[16];\n"
+              "void f() {\n"
+              "    CharArray foo = \"\";\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+    }
+
     void novardecl() {
         // if the variable declarations can't be seen then skip the warning
         check("void foo()\n"
@@ -102,6 +117,41 @@ private:
               "    return a;\n"
               "}");
         ASSERT_EQUALS("", errout.str());
+
+        check("std::array<int,2> f();\n"
+              "void g() {\n"
+              "    std::array<int, 2> a = f();\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        check("std::array<int,2> f(int x);\n"
+              "void g(int i) {\n"
+              "    std::array<int, 2> a = f(i);\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        check("typedef std::array<int, 2> Array;\n"
+              "Array f(int x);\n"
+              "void g(int i) {\n"
+              "    Array a = f(i);\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        check("typedef std::array<int, 2> Array;\n"
+              "Array f();\n"
+              "void g(int i) {\n"
+              "    Array a = f();\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        check("struct S {\n" // #9951
+              "    enum E { E0 };\n"
+              "    std::array<double, 1> g(S::E);\n"
+              "};\n"
+              "void f() {\n"
+              "    std::array<double, 1> a = S::g(S::E::E0);\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
     }
 
     void structmember() {
@@ -110,6 +160,16 @@ private:
               "    int i = foo->p;\n"
               "}");
         ASSERT_EQUALS("[test.cpp:3]: (portability) Assigning a pointer to an integer is not portable.\n", errout.str());
+
+        check("struct S {\n" // #10145
+              "    enum class E { e1, e2 };\n"
+              "    E e;\n"
+              "    char* e1;\n"
+              "};\n"
+              "void f(S& s) {\n"
+              "    s.e = S::E::e1;\n"
+              "}\n");
+        ASSERT_EQUALS("", errout.str());
     }
 
     void ptrcompare() {
@@ -168,6 +228,13 @@ private:
               "}");
         ASSERT_EQUALS("", errout.str());
 
+        check("struct Foo {};\n"
+              "\n"
+              "int* dostuff(Foo foo) {\n"
+              "  return foo;\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
         check("int foo(char* c) {\n"
               "    return c;\n"
               "}");
@@ -202,7 +269,7 @@ private:
               "}");
         ASSERT_EQUALS("", errout.str());
 
-        // #7247: dont check return statements in nested functions..
+        // #7247: don't check return statements in nested functions..
         check("int foo() {\n"
               "  struct {\n"
               "    const char * name() { return \"abc\"; }\n"
@@ -212,7 +279,7 @@ private:
 
         // #7451: Lambdas
         check("const int* test(std::vector<int> outputs, const std::string& text) {\n"
-              "  auto it = std::find_if(outputs.begin(), outputs.end(), \n"
+              "  auto it = std::find_if(outputs.begin(), outputs.end(),\n"
               "     [&](int ele) { return \"test\" == text; });\n"
               "  return nullptr;\n"
               "}");
