@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2018 Cppcheck team.
+ * Copyright (C) 2007-2022 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,21 +19,24 @@
 #include "config.h"
 #include "cppcheck.h"
 #include "errorlogger.h"
+#include "errortypes.h"
 #include "suppressions.h"
 #include "testsuite.h"
 
+#include <iosfwd>
 #include <list>
+#include <memory>
 #include <string>
 
+#include <tinyxml2.h>
 
 class TestErrorLogger : public TestFixture {
 public:
-    TestErrorLogger() : TestFixture("TestErrorLogger"), fooCpp5("foo.cpp", 5), barCpp8("bar.cpp", 8) {
-    }
+    TestErrorLogger() : TestFixture("TestErrorLogger"), fooCpp5("foo.cpp", 5, 1), barCpp8("bar.cpp", 8, 1) {}
 
 private:
-    const ErrorLogger::ErrorMessage::FileLocation fooCpp5;
-    const ErrorLogger::ErrorMessage::FileLocation barCpp8;
+    const ErrorMessage::FileLocation fooCpp5;
+    const ErrorMessage::FileLocation barCpp8;
 
     void run() override {
         TEST_CASE(PatternSearchReplace);
@@ -49,6 +52,7 @@ private:
         TEST_CASE(ToXmlV2);
         TEST_CASE(ToXmlV2Locations);
         TEST_CASE(ToXmlV2Encoding);
+        TEST_CASE(FromXmlV2);
 
         // Inconclusive results in xml reports..
         TEST_CASE(InconclusiveXml);
@@ -57,6 +61,7 @@ private:
         TEST_CASE(SerializeInconclusiveMessage);
         TEST_CASE(DeserializeInvalidInput);
         TEST_CASE(SerializeSanitize);
+        TEST_CASE(SerializeFileLocation);
 
         TEST_CASE(suppressUnmatchedSuppressions);
     }
@@ -64,8 +69,8 @@ private:
     void TestPatternSearchReplace(const std::string& idPlaceholder, const std::string& id) const {
         const std::string plainText = "text";
 
-        ErrorLogger::ErrorMessage message;
-        message._id = id;
+        ErrorMessage message;
+        message.id = id;
 
         std::string serialized = message.toString(true, idPlaceholder + plainText + idPlaceholder);
         ASSERT_EQUALS(id + plainText + id, serialized);
@@ -97,22 +102,22 @@ private:
     }
 
     void FileLocationDefaults() const {
-        ErrorLogger::ErrorMessage::FileLocation loc;
+        ErrorMessage::FileLocation loc;
         ASSERT_EQUALS("", loc.getfile());
         ASSERT_EQUALS(0, loc.line);
     }
 
     void FileLocationSetFile() const {
-        ErrorLogger::ErrorMessage::FileLocation loc;
+        ErrorMessage::FileLocation loc;
         loc.setfile("foo.cpp");
         ASSERT_EQUALS("foo.cpp", loc.getfile());
         ASSERT_EQUALS(0, loc.line);
     }
 
     void ErrorMessageConstruct() const {
-        std::list<ErrorLogger::ErrorMessage::FileLocation> locs(1, fooCpp5);
-        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error.", "errorId", false);
-        ASSERT_EQUALS(1, (int)msg._callStack.size());
+        std::list<ErrorMessage::FileLocation> locs(1, fooCpp5);
+        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error.", "errorId", Certainty::normal);
+        ASSERT_EQUALS(1, msg.callStack.size());
         ASSERT_EQUALS("Programming error.", msg.shortMessage());
         ASSERT_EQUALS("Programming error.", msg.verboseMessage());
         ASSERT_EQUALS("[foo.cpp:5]: (error) Programming error.", msg.toString(false));
@@ -120,9 +125,9 @@ private:
     }
 
     void ErrorMessageConstructLocations() const {
-        std::list<ErrorLogger::ErrorMessage::FileLocation> locs = { fooCpp5, barCpp8 };
-        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error.", "errorId", false);
-        ASSERT_EQUALS(2, (int)msg._callStack.size());
+        std::list<ErrorMessage::FileLocation> locs = { fooCpp5, barCpp8 };
+        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error.", "errorId", Certainty::normal);
+        ASSERT_EQUALS(2, msg.callStack.size());
         ASSERT_EQUALS("Programming error.", msg.shortMessage());
         ASSERT_EQUALS("Programming error.", msg.verboseMessage());
         ASSERT_EQUALS("[foo.cpp:5] -> [bar.cpp:8]: (error) Programming error.", msg.toString(false));
@@ -130,9 +135,9 @@ private:
     }
 
     void ErrorMessageVerbose() const {
-        std::list<ErrorLogger::ErrorMessage::FileLocation> locs(1, fooCpp5);
-        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error.\nVerbose error", "errorId", false);
-        ASSERT_EQUALS(1, (int)msg._callStack.size());
+        std::list<ErrorMessage::FileLocation> locs(1, fooCpp5);
+        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error.\nVerbose error", "errorId", Certainty::normal);
+        ASSERT_EQUALS(1, msg.callStack.size());
         ASSERT_EQUALS("Programming error.", msg.shortMessage());
         ASSERT_EQUALS("Verbose error", msg.verboseMessage());
         ASSERT_EQUALS("[foo.cpp:5]: (error) Programming error.", msg.toString(false));
@@ -140,9 +145,9 @@ private:
     }
 
     void ErrorMessageVerboseLocations() const {
-        std::list<ErrorLogger::ErrorMessage::FileLocation> locs = { fooCpp5, barCpp8 };
-        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error.\nVerbose error", "errorId", false);
-        ASSERT_EQUALS(2, (int)msg._callStack.size());
+        std::list<ErrorMessage::FileLocation> locs = { fooCpp5, barCpp8 };
+        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error.\nVerbose error", "errorId", Certainty::normal);
+        ASSERT_EQUALS(2, msg.callStack.size());
         ASSERT_EQUALS("Programming error.", msg.shortMessage());
         ASSERT_EQUALS("Verbose error", msg.verboseMessage());
         ASSERT_EQUALS("[foo.cpp:5] -> [bar.cpp:8]: (error) Programming error.", msg.toString(false));
@@ -150,9 +155,9 @@ private:
     }
 
     void CustomFormat() const {
-        std::list<ErrorLogger::ErrorMessage::FileLocation> locs(1, fooCpp5);
-        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error.\nVerbose error", "errorId", false);
-        ASSERT_EQUALS(1, (int)msg._callStack.size());
+        std::list<ErrorMessage::FileLocation> locs(1, fooCpp5);
+        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error.\nVerbose error", "errorId", Certainty::normal);
+        ASSERT_EQUALS(1, msg.callStack.size());
         ASSERT_EQUALS("Programming error.", msg.shortMessage());
         ASSERT_EQUALS("Verbose error", msg.verboseMessage());
         ASSERT_EQUALS("foo.cpp:5,error,errorId,Programming error.", msg.toString(false, "{file}:{line},{severity},{id},{message}"));
@@ -160,9 +165,9 @@ private:
     }
 
     void CustomFormat2() const {
-        std::list<ErrorLogger::ErrorMessage::FileLocation> locs(1, fooCpp5);
-        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error.\nVerbose error", "errorId", false);
-        ASSERT_EQUALS(1, (int)msg._callStack.size());
+        std::list<ErrorMessage::FileLocation> locs(1, fooCpp5);
+        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error.\nVerbose error", "errorId", Certainty::normal);
+        ASSERT_EQUALS(1, msg.callStack.size());
         ASSERT_EQUALS("Programming error.", msg.shortMessage());
         ASSERT_EQUALS("Verbose error", msg.verboseMessage());
         ASSERT_EQUALS("Programming error. - foo.cpp(5):(error,errorId)", msg.toString(false, "{message} - {file}({line}):({severity},{id})"));
@@ -171,9 +176,9 @@ private:
 
     void CustomFormatLocations() const {
         // Check that first location from location stack is used in template
-        std::list<ErrorLogger::ErrorMessage::FileLocation> locs = { fooCpp5, barCpp8 };
-        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error.\nVerbose error", "errorId", false);
-        ASSERT_EQUALS(2, (int)msg._callStack.size());
+        std::list<ErrorMessage::FileLocation> locs = { fooCpp5, barCpp8 };
+        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error.\nVerbose error", "errorId", Certainty::normal);
+        ASSERT_EQUALS(2, msg.callStack.size());
         ASSERT_EQUALS("Programming error.", msg.shortMessage());
         ASSERT_EQUALS("Verbose error", msg.verboseMessage());
         ASSERT_EQUALS("Programming error. - bar.cpp(8):(error,errorId)", msg.toString(false, "{message} - {file}({line}):({severity},{id})"));
@@ -181,75 +186,111 @@ private:
     }
 
     void ToXmlV2() const {
-        std::list<ErrorLogger::ErrorMessage::FileLocation> locs(1, fooCpp5);
-        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error.\nVerbose error", "errorId", false);
+        std::list<ErrorMessage::FileLocation> locs(1, fooCpp5);
+        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error.\nVerbose error", "errorId", Certainty::normal);
         std::string header("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<results version=\"2\">\n");
         header += "    <cppcheck version=\"";
         header += CppCheck::version();
         header += "\"/>\n    <errors>";
-        ASSERT_EQUALS(header, ErrorLogger::ErrorMessage::getXMLHeader());
-        ASSERT_EQUALS("    </errors>\n</results>", ErrorLogger::ErrorMessage::getXMLFooter());
+        ASSERT_EQUALS(header, ErrorMessage::getXMLHeader());
+        ASSERT_EQUALS("    </errors>\n</results>", ErrorMessage::getXMLFooter());
         std::string message("        <error id=\"errorId\" severity=\"error\"");
         message += " msg=\"Programming error.\" verbose=\"Verbose error\">\n";
-        message += "            <location file=\"foo.cpp\" line=\"5\"/>\n        </error>";
+        message += "            <location file=\"foo.cpp\" line=\"5\" column=\"1\"/>\n        </error>";
         ASSERT_EQUALS(message, msg.toXML());
     }
 
     void ToXmlV2Locations() const {
-        std::list<ErrorLogger::ErrorMessage::FileLocation> locs = { fooCpp5, barCpp8 };
-        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error.\nVerbose error", "errorId", false);
+        std::list<ErrorMessage::FileLocation> locs = { fooCpp5, barCpp8 };
+        locs.back().setinfo("ä");
+        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error.\nVerbose error", "errorId", Certainty::normal);
         std::string header("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<results version=\"2\">\n");
         header += "    <cppcheck version=\"";
         header += CppCheck::version();
         header += "\"/>\n    <errors>";
-        ASSERT_EQUALS(header, ErrorLogger::ErrorMessage::getXMLHeader());
-        ASSERT_EQUALS("    </errors>\n</results>", ErrorLogger::ErrorMessage::getXMLFooter());
+        ASSERT_EQUALS(header, ErrorMessage::getXMLHeader());
+        ASSERT_EQUALS("    </errors>\n</results>", ErrorMessage::getXMLFooter());
         std::string message("        <error id=\"errorId\" severity=\"error\"");
         message += " msg=\"Programming error.\" verbose=\"Verbose error\">\n";
-        message += "            <location file=\"bar.cpp\" line=\"8\"/>\n";
-        message += "            <location file=\"foo.cpp\" line=\"5\"/>\n        </error>";
+        message += "            <location file=\"bar.cpp\" line=\"8\" column=\"1\" info=\"\\303\\244\"/>\n";
+        message += "            <location file=\"foo.cpp\" line=\"5\" column=\"1\"/>\n        </error>";
         ASSERT_EQUALS(message, msg.toXML());
     }
 
     void ToXmlV2Encoding() const {
         {
-            std::list<ErrorLogger::ErrorMessage::FileLocation> locs;
-            ErrorMessage msg(locs, emptyString, Severity::error, "Programming error.\nComparing \"\203\" with \"\003\"", "errorId", false);
+            std::list<ErrorMessage::FileLocation> locs;
+            ErrorMessage msg(locs, emptyString, Severity::error, "Programming error.\nComparing \"\203\" with \"\003\"", "errorId", Certainty::normal);
             const std::string expected("        <error id=\"errorId\" severity=\"error\" msg=\"Programming error.\" verbose=\"Comparing &quot;\\203&quot; with &quot;\\003&quot;\"/>");
             ASSERT_EQUALS(expected, msg.toXML());
         }
         {
             const char code1[]="äöü";
             const char code2[]="\x12\x00\x00\x01";
-            std::list<ErrorLogger::ErrorMessage::FileLocation> locs;
-            ErrorMessage msg1(locs, emptyString, Severity::error, std::string("Programming error.\nReading \"")+code1+"\"", "errorId", false);
+            std::list<ErrorMessage::FileLocation> locs;
+            ErrorMessage msg1(locs, emptyString, Severity::error, std::string("Programming error.\nReading \"")+code1+"\"", "errorId", Certainty::normal);
             ASSERT_EQUALS("        <error id=\"errorId\" severity=\"error\" msg=\"Programming error.\" verbose=\"Reading &quot;\\303\\244\\303\\266\\303\\274&quot;\"/>", msg1.toXML());
-            ErrorMessage msg2(locs, emptyString, Severity::error, std::string("Programming error.\nReading \"")+code2+"\"", "errorId", false);
+            ErrorMessage msg2(locs, emptyString, Severity::error, std::string("Programming error.\nReading \"")+code2+"\"", "errorId", Certainty::normal);
             ASSERT_EQUALS("        <error id=\"errorId\" severity=\"error\" msg=\"Programming error.\" verbose=\"Reading &quot;\\022&quot;\"/>", msg2.toXML());
         }
     }
 
+    void FromXmlV2() const {
+        const char xmldata[] = "<?xml version=\"1.0\"?>\n"
+                               "<error id=\"errorId\""
+                               " severity=\"error\""
+                               " cwe=\"123\""
+                               " inconclusive=\"true\""
+                               " msg=\"Programming error.\""
+                               " verbose=\"Verbose error\""
+                               " hash=\"456\""
+                               ">\n"
+                               "  <location file=\"bar.cpp\" line=\"8\" column=\"1\"/>\n"
+                               "  <location file=\"foo.cpp\" line=\"5\" column=\"2\"/>\n"
+                               "</error>";
+        tinyxml2::XMLDocument doc;
+        doc.Parse(xmldata, sizeof(xmldata));
+        ErrorMessage msg(doc.FirstChildElement());
+        ASSERT_EQUALS("errorId", msg.id);
+        ASSERT_EQUALS(Severity::error, msg.severity);
+        ASSERT_EQUALS(123u, msg.cwe.id);
+        ASSERT_EQUALS(Certainty::inconclusive, msg.certainty);
+        ASSERT_EQUALS("Programming error.", msg.shortMessage());
+        ASSERT_EQUALS("Verbose error", msg.verboseMessage());
+        ASSERT_EQUALS(456u, msg.hash);
+        ASSERT_EQUALS(2u, msg.callStack.size());
+        ASSERT_EQUALS("foo.cpp", msg.callStack.front().getfile());
+        ASSERT_EQUALS(5, msg.callStack.front().line);
+        ASSERT_EQUALS(2u, msg.callStack.front().column);
+        ASSERT_EQUALS("bar.cpp", msg.callStack.back().getfile());
+        ASSERT_EQUALS(8, msg.callStack.back().line);
+        ASSERT_EQUALS(1u, msg.callStack.back().column);
+    }
+
     void InconclusiveXml() const {
         // Location
-        std::list<ErrorLogger::ErrorMessage::FileLocation> locs(1, fooCpp5);
+        std::list<ErrorMessage::FileLocation> locs(1, fooCpp5);
 
         // Inconclusive error message
-        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error", "errorId", true);
+        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error", "errorId", Certainty::inconclusive);
 
         // xml version 2 error message
         ASSERT_EQUALS("        <error id=\"errorId\" severity=\"error\" msg=\"Programming error\" verbose=\"Programming error\" inconclusive=\"true\">\n"
-                      "            <location file=\"foo.cpp\" line=\"5\"/>\n"
+                      "            <location file=\"foo.cpp\" line=\"5\" column=\"1\"/>\n"
                       "        </error>",
                       msg.toXML());
     }
 
     void SerializeInconclusiveMessage() const {
         // Inconclusive error message
-        std::list<ErrorLogger::ErrorMessage::FileLocation> locs;
-        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error", "errorId", true);
+        std::list<ErrorMessage::FileLocation> locs;
+        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error", "errorId", Certainty::inconclusive);
+        msg.file0 = "test.cpp";
         ASSERT_EQUALS("7 errorId"
                       "5 error"
                       "1 0"
+                      "1 0"
+                      "8 test.cpp"
                       "12 inconclusive"
                       "17 Programming error"
                       "17 Programming error"
@@ -257,9 +298,10 @@ private:
 
         ErrorMessage msg2;
         msg2.deserialize(msg.serialize());
-        ASSERT_EQUALS("errorId", msg2._id);
-        ASSERT_EQUALS(Severity::error, msg2._severity);
-        ASSERT_EQUALS(true, msg2._inconclusive);
+        ASSERT_EQUALS("errorId", msg2.id);
+        ASSERT_EQUALS(Severity::error, msg2.severity);
+        ASSERT_EQUALS("test.cpp", msg2.file0);
+        ASSERT_EQUALS(Certainty::inconclusive, msg2.certainty);
         ASSERT_EQUALS("Programming error", msg2.shortMessage());
         ASSERT_EQUALS("Programming error", msg2.verboseMessage());
     }
@@ -270,24 +312,46 @@ private:
     }
 
     void SerializeSanitize() const {
-        std::list<ErrorLogger::ErrorMessage::FileLocation> locs;
-        ErrorMessage msg(locs, emptyString, Severity::error, std::string("Illegal character in \"foo\001bar\""), "errorId", false);
+        std::list<ErrorMessage::FileLocation> locs;
+        ErrorMessage msg(locs, emptyString, Severity::error, std::string("Illegal character in \"foo\001bar\""), "errorId", Certainty::normal);
+        msg.file0 = "1.c";
 
         ASSERT_EQUALS("7 errorId"
                       "5 error"
                       "1 0"
+                      "1 0"
+                      "3 1.c"
                       "33 Illegal character in \"foo\\001bar\""
                       "33 Illegal character in \"foo\\001bar\""
                       "0 ", msg.serialize());
 
         ErrorMessage msg2;
         msg2.deserialize(msg.serialize());
-        ASSERT_EQUALS("errorId", msg2._id);
-        ASSERT_EQUALS(Severity::error, msg2._severity);
+        ASSERT_EQUALS("errorId", msg2.id);
+        ASSERT_EQUALS(Severity::error, msg2.severity);
+        ASSERT_EQUALS("1.c", msg2.file0);
         ASSERT_EQUALS("Illegal character in \"foo\\001bar\"", msg2.shortMessage());
         ASSERT_EQUALS("Illegal character in \"foo\\001bar\"", msg2.verboseMessage());
     }
 
+    void SerializeFileLocation() const {
+        ErrorMessage::FileLocation loc1(":/,;", 654, 33);
+        loc1.setfile("[]:;,()");
+        loc1.setinfo("abcd:/,");
+
+        std::list<ErrorMessage::FileLocation> locs{loc1};
+
+        ErrorMessage msg(locs, emptyString, Severity::error, "Programming error", "errorId", Certainty::inconclusive);
+
+        ErrorMessage msg2;
+        msg2.deserialize(msg.serialize());
+        ASSERT_EQUALS("[]:;,()", msg2.callStack.front().getfile(false));
+        ASSERT_EQUALS(":/,;", msg2.callStack.front().getOrigFile(false));
+        ASSERT_EQUALS(654, msg2.callStack.front().line);
+        ASSERT_EQUALS(33, msg2.callStack.front().column);
+        ASSERT_EQUALS("abcd:/,", msg2.callStack.front().getinfo());
+    }
+
     void suppressUnmatchedSuppressions() {
         std::list<Suppressions::Suppression> suppressions;
 
@@ -301,7 +365,15 @@ private:
         errout.str("");
         suppressions.clear();
         suppressions.emplace_back("abc", "a.c", 10U);
-        suppressions.emplace_back("unmatchedSuppression", "*", 0U);
+        suppressions.emplace_back("unmatchedSuppression", "*", Suppressions::Suppression::NO_LINE);
+        reportUnmatchedSuppressions(suppressions);
+        ASSERT_EQUALS("", errout.str());
+
+        // suppress all unmatchedSuppression (corresponds to "--suppress=unmatchedSuppression")
+        errout.str("");
+        suppressions.clear();
+        suppressions.emplace_back("abc", "a.c", 10U);
+        suppressions.emplace_back("unmatchedSuppression", "", Suppressions::Suppression::NO_LINE);
         reportUnmatchedSuppressions(suppressions);
         ASSERT_EQUALS("", errout.str());
 
@@ -309,7 +381,7 @@ private:
         errout.str("");
         suppressions.clear();
         suppressions.emplace_back("abc", "a.c", 10U);
-        suppressions.emplace_back("unmatchedSuppression", "a.c", 0U);
+        suppressions.emplace_back("unmatchedSuppression", "a.c", Suppressions::Suppression::NO_LINE);
         reportUnmatchedSuppressions(suppressions);
         ASSERT_EQUALS("", errout.str());
 
@@ -325,7 +397,7 @@ private:
         errout.str("");
         suppressions.clear();
         suppressions.emplace_back("abc", "a.c", 10U);
-        suppressions.emplace_back("unmatchedSuppression", "b.c", 0U);
+        suppressions.emplace_back("unmatchedSuppression", "b.c", Suppressions::Suppression::NO_LINE);
         reportUnmatchedSuppressions(suppressions);
         ASSERT_EQUALS("[a.c:10]: (information) Unmatched suppression: abc\n", errout.str());
 
