@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2016 Cppcheck team.
+ * Copyright (C) 2007-2022 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,40 +25,40 @@
 
 class TestInternal : public TestFixture {
 public:
-    TestInternal() : TestFixture("TestInternal") {
-    }
+    TestInternal() : TestFixture("TestInternal") {}
 
 private:
     Settings settings;
 
-    void run() {
+    void run() override {
         settings.addEnabled("internal");
 
-        TEST_CASE(simplePatternInTokenMatch)
-        TEST_CASE(complexPatternInTokenSimpleMatch)
-        TEST_CASE(simplePatternSquareBrackets)
-        TEST_CASE(simplePatternAlternatives)
-        TEST_CASE(missingPercentCharacter)
-        TEST_CASE(unknownPattern)
-        TEST_CASE(redundantNextPrevious)
+        TEST_CASE(simplePatternInTokenMatch);
+        TEST_CASE(complexPatternInTokenSimpleMatch);
+        TEST_CASE(simplePatternSquareBrackets);
+        TEST_CASE(simplePatternAlternatives);
+        TEST_CASE(missingPercentCharacter);
+        TEST_CASE(unknownPattern);
+        TEST_CASE(redundantNextPrevious);
         TEST_CASE(internalError);
         TEST_CASE(orInComplexPattern);
         TEST_CASE(extraWhitespace);
+        TEST_CASE(checkRedundantTokCheck);
     }
 
-    void check(const char code[]) {
+#define check(code) check_(code, __FILE__, __LINE__)
+    void check_(const char code[], const char* file, int line) {
         // Clear the error buffer..
         errout.str("");
 
         // Tokenize..
         Tokenizer tokenizer(&settings, this);
         std::istringstream istr(code);
-        tokenizer.tokenize(istr, "test.cpp");
-        tokenizer.simplifyTokenList2();
+        ASSERT_LOC(tokenizer.tokenize(istr, "test.cpp"), file, line);
 
         // Check..
         CheckInternal checkInternal;
-        checkInternal.runSimplifiedChecks(&tokenizer, &settings, this);
+        checkInternal.runChecks(&tokenizer, &settings, this);
     }
 
     void simplePatternInTokenMatch() {
@@ -309,6 +309,12 @@ private:
               "    return tok->next()->next()->str();\n"
               "}");
         ASSERT_EQUALS("[test.cpp:2]: (style) Call to 'Token::next()' followed by 'Token::str()' can be simplified.\n", errout.str());
+
+        check("void f() {\n"
+              "    return tok->previous()->next()->str();\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (style) Call to 'Token::previous()' followed by 'Token::next()' can be simplified.\n", errout.str());
+
     }
 
     void internalError() {
@@ -402,6 +408,123 @@ private:
               "}");
         ASSERT_EQUALS("[test.cpp:3]: (warning) Found extra whitespace inside Token::findsimplematch() call: \"foobar \"\n", errout.str());
     }
+
+    void checkRedundantTokCheck() {
+        // findsimplematch
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(tok && Token::findsimplematch(tok, \"foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (style) Unnecessary check of \"tok\", match-function already checks if it is null.\n", errout.str());
+
+        // findmatch
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(tok && Token::findmatch(tok, \"%str% foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (style) Unnecessary check of \"tok\", match-function already checks if it is null.\n", errout.str());
+
+        // Match
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(tok && Token::Match(tok, \"5str% foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (style) Unnecessary check of \"tok\", match-function already checks if it is null.\n", errout.str());
+
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(a && tok && Token::Match(tok, \"5str% foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (style) Unnecessary check of \"tok\", match-function already checks if it is null.\n", errout.str());
+
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(a && b && tok && Token::Match(tok, \"5str% foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (style) Unnecessary check of \"tok\", match-function already checks if it is null.\n", errout.str());
+
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(a && b && c && tok && Token::Match(tok, \"5str% foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (style) Unnecessary check of \"tok\", match-function already checks if it is null.\n", errout.str());
+
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(a && b && c && tok && d && Token::Match(tok, \"5str% foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        // simpleMatch
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(tok && Token::simpleMatch(tok, \"foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (style) Unnecessary check of \"tok\", match-function already checks if it is null.\n", errout.str());
+
+        // Match
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(tok->previous() && Token::Match(tok->previous(), \"5str% foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (style) Unnecessary check of \"tok->previous()\", match-function already checks if it is null.\n", errout.str());
+
+        // don't report:
+        // tok->previous() vs tok
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(tok->previous() && Token::Match(tok, \"5str% foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        // tok vs tok->previous())
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(tok && Token::Match(tok->previous(), \"5str% foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        // tok->previous() vs tok->previous()->previous())
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(tok->previous() && Token::Match(tok->previous()->previous(), \"5str% foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        // if a && fn(a) triggers, make sure !a || !fn(a) triggers as well!
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(!tok || !Token::simpleMatch(tok, \"foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (style) Unnecessary check of \"tok\", match-function already checks if it is null.\n", errout.str());
+
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(a || !tok || !Token::simpleMatch(tok, \"foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (style) Unnecessary check of \"tok\", match-function already checks if it is null.\n", errout.str());
+
+        // if tok || !Token::simpleMatch...
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(tok || !Token::simpleMatch(tok, \"foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        // if !tok || Token::simpleMatch...
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(!tok || Token::simpleMatch(tok, \"foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        // something more complex
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(!tok->previous()->previous() || !Token::simpleMatch(tok->previous()->previous(), \"foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (style) Unnecessary check of \"tok->previous()->previous()\", match-function already checks if it is null.\n", errout.str());
+    }
 };
 
 REGISTER_TEST(TestInternal)
