@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2016 Cppcheck team.
+ * Copyright (C) 2007-2022 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,23 +17,25 @@
  */
 
 
-#include "tokenize.h"
 #include "checkstring.h"
+#include "errortypes.h"
+#include "settings.h"
 #include "testsuite.h"
-#include "testutils.h"
+#include "tokenize.h"
+
+#include <iosfwd>
 
 
 class TestString : public TestFixture {
 public:
-    TestString() : TestFixture("TestString") {
-    }
+    TestString() : TestFixture("TestString") {}
 
 private:
     Settings settings;
 
-    void run() {
-        settings.addEnabled("warning");
-        settings.addEnabled("style");
+    void run() override {
+        settings.severity.enable(Severity::warning);
+        settings.severity.enable(Severity::style);
 
         TEST_CASE(stringLiteralWrite);
 
@@ -44,30 +46,36 @@ private:
         TEST_CASE(strPlusChar1);     // "/usr" + '/'
         TEST_CASE(strPlusChar2);     // "/usr" + ch
         TEST_CASE(strPlusChar3);     // ok: path + "/sub" + '/'
+        TEST_CASE(strPlusChar4);     // L"/usr" + L'/'
 
+        TEST_CASE(snprintf1);       // Dangerous usage of snprintf
         TEST_CASE(sprintf1);        // Dangerous usage of sprintf
         TEST_CASE(sprintf2);
         TEST_CASE(sprintf3);
         TEST_CASE(sprintf4);        // struct member
+        TEST_CASE(sprintf5);        // another struct member
+        TEST_CASE(sprintf6);        // (char*)
+        TEST_CASE(sprintf7);        // (char*)(void*)
+        TEST_CASE(wsprintf1);       // Dangerous usage of wsprintf
 
         TEST_CASE(incorrectStringCompare);
+
+        TEST_CASE(deadStrcmp);
     }
 
-    void check(const char code[], const char filename[] = "test.cpp") {
+#define check(...) check_(__FILE__, __LINE__, __VA_ARGS__)
+    void check_(const char* file, int line, const char code[], const char filename[] = "test.cpp") {
         // Clear the error buffer..
         errout.str("");
 
         // Tokenize..
         Tokenizer tokenizer(&settings, this);
         std::istringstream istr(code);
-        tokenizer.tokenize(istr, filename);
+        ASSERT_LOC(tokenizer.tokenize(istr, filename), file, line);
 
         // Check char variable usage..
         CheckString checkString(&tokenizer, &settings, this);
         checkString.runChecks(&tokenizer, &settings, this);
-
-        tokenizer.simplifyTokenList2();
-        checkString.runSimplifiedChecks(&tokenizer, &settings, this);
     }
 
     void stringLiteralWrite() {
@@ -83,11 +91,59 @@ private:
               "}");
         ASSERT_EQUALS("[test.cpp:3] -> [test.cpp:2]: (error) Modifying string literal \"abc\" directly or indirectly is undefined behaviour.\n", errout.str());
 
+        check("void f() {\n"
+              "  char *abc = \"A very long string literal\";\n"
+              "  abc[0] = 'a';\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3] -> [test.cpp:2]: (error) Modifying string literal \"A very long stri..\" directly or indirectly is undefined behaviour.\n", errout.str());
+
         check("void f() {\n"
               "  QString abc = \"abc\";\n"
               "  abc[0] = 'a';\n"
               "}");
         ASSERT_EQUALS("", errout.str());
+
+        check("void foo_FP1(char *p) {\n"
+              "  p[1] = 'B';\n"
+              "}\n"
+              "void foo_FP2(void) {\n"
+              "  char* s = \"Y\";\n"
+              "  foo_FP1(s);\n"
+              "}");
+        ASSERT_EQUALS(
+            "[test.cpp:2] -> [test.cpp:5]: (error) Modifying string literal \"Y\" directly or indirectly is undefined behaviour.\n",
+            errout.str());
+
+        check("void foo_FP1(char *p) {\n"
+              "  p[1] = 'B';\n"
+              "}\n"
+              "void foo_FP2(void) {\n"
+              "  char s[10] = \"Y\";\n"
+              "  foo_FP1(s);\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        check("void f() {\n"
+              "  wchar_t *abc = L\"abc\";\n"
+              "  abc[0] = u'a';\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3] -> [test.cpp:2]: (error) Modifying string literal L\"abc\" directly or indirectly is undefined behaviour.\n", errout.str());
+
+        check("void f() {\n"
+              "  char16_t *abc = u\"abc\";\n"
+              "  abc[0] = 'a';\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3] -> [test.cpp:2]: (error) Modifying string literal u\"abc\" directly or indirectly is undefined behaviour.\n", errout.str());
+
+        check("void foo() {\n" // #8332
+              "    int i;\n"
+              "    char *p  = \"string literal\";\n"
+              "    for( i = 0; i < strlen(p); i++) {\n"
+              "        p[i] = \'X\';\n" // <<
+              "    }\n"
+              "    printf(\"%s\\n\", p);\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:5] -> [test.cpp:3]: (error) Modifying string literal \"string literal\" directly or indirectly is undefined behaviour.\n", errout.str());
     }
 
     void alwaysTrueFalseStringCompare() {
@@ -149,7 +205,7 @@ private:
               "{\n"
               "  if (strcmp(\"00FF00\", \"00FF00\") == 0)"
               "  {"
-              "    std::cout << \"Equal\n\""
+              "    std::cout << \"Equal\";"
               "  }"
               "}");
         ASSERT_EQUALS("[test.cpp:3]: (warning) Unnecessary comparison of static strings.\n", errout.str());
@@ -168,7 +224,7 @@ private:
               "{\n"
               "  if (stricmp(\"hotdog\",\"HOTdog\") == 0)"
               "  {"
-              "    std::cout << \"Equal\n\""
+              "    std::cout << \"Equal\";"
               "  }"
               "}");
         ASSERT_EQUALS("[test.cpp:3]: (warning) Unnecessary comparison of static strings.\n", errout.str());
@@ -177,7 +233,7 @@ private:
               "{\n"
               "  if (QString::compare(\"Hamburger\", \"Hotdog\") == 0)"
               "  {"
-              "    std::cout << \"Equal\n\""
+              "    std::cout << \"Equal\";"
               "  }"
               "}");
         ASSERT_EQUALS("[test.cpp:3]: (warning) Unnecessary comparison of static strings.\n", errout.str());
@@ -186,7 +242,7 @@ private:
               "{\n"
               "  if (QString::compare(argv[2], \"hotdog\") == 0)"
               "  {"
-              "    std::cout << \"Equal\n\""
+              "    std::cout << \"Equal\";"
               "  }"
               "}");
         ASSERT_EQUALS("", errout.str());
@@ -195,7 +251,7 @@ private:
               "{\n"
               "  if (strncmp(\"hotdog\",\"hotdog\", 6) == 0)"
               "  {"
-              "    std::cout << \"Equal\n\""
+              "    std::cout << \"Equal\";"
               "  }"
               "}");
         ASSERT_EQUALS("[test.cpp:3]: (warning) Unnecessary comparison of static strings.\n", errout.str());
@@ -204,7 +260,7 @@ private:
               "{\n"
               "  if (strcmp(buf, buf) == 0)"
               "  {"
-              "    std::cout << \"Equal\n\""
+              "    std::cout << \"Equal\";"
               "  }"
               "}");
         ASSERT_EQUALS("[test.cpp:3]: (warning) Comparison of identical string variables.\n", errout.str());
@@ -213,28 +269,28 @@ private:
               "{\n"
               "  if (stricmp(buf.c_str(), buf.c_str()) == 0)"
               "  {"
-              "    std::cout << \"Equal\n\""
+              "    std::cout << \"Equal\";"
               "  }"
               "}");
         ASSERT_EQUALS("[test.cpp:3]: (warning) Comparison of identical string variables.\n", errout.str());
 
         check("int main() {\n"
               "  if (\"str\" == \"str\") {\n"
-              "    std::cout << \"Equal\n\"\n"
+              "    std::cout << \"Equal\";\n"
               "  }\n"
               "}");
         ASSERT_EQUALS("[test.cpp:2]: (warning) Unnecessary comparison of static strings.\n", errout.str());
 
         check("int main() {\n"
               "  if (\"str\" != \"str\") {\n"
-              "    std::cout << \"Equal\n\"\n"
+              "    std::cout << \"Equal\";\n"
               "  }\n"
               "}");
         ASSERT_EQUALS("[test.cpp:2]: (warning) Unnecessary comparison of static strings.\n", errout.str());
 
         check("int main() {\n"
               "  if (a+\"str\" != \"str\"+b) {\n"
-              "    std::cout << \"Equal\n\"\n"
+              "    std::cout << \"Equal\";\n"
               "  }\n"
               "}");
         ASSERT_EQUALS("", errout.str());
@@ -246,6 +302,16 @@ private:
               "}");
         ASSERT_EQUALS("[test.cpp:2]: (warning) String literal compared with variable 'c'. Did you intend to use strcmp() instead?\n", errout.str());
 
+        check("bool foo(char** c) {\n"
+              "    return c[3] == \"x\";\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (warning) String literal compared with variable 'c[3]'. Did you intend to use strcmp() instead?\n", errout.str());
+
+        check("bool foo(wchar_t* c) {\n"
+              "    return c == L\"x\";\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (warning) String literal compared with variable 'c'. Did you intend to use wcscmp() instead?\n", errout.str());
+
         check("bool foo(const char* c) {\n"
               "    return \"x\" == c;\n"
               "}");
@@ -264,7 +330,7 @@ private:
         check("bool foo(char* c) {\n"
               "    return \"x\" == c+foo;\n"
               "}", "test.c");
-        ASSERT_EQUALS("[test.c:2]: (warning) String literal compared with variable 'c'. Did you intend to use strcmp() instead?\n", errout.str());
+        ASSERT_EQUALS("[test.c:2]: (warning) String literal compared with variable 'c+foo'. Did you intend to use strcmp() instead?\n", errout.str());
 
         check("bool foo(Foo c) {\n"
               "    return \"x\" == c.foo;\n"
@@ -301,18 +367,14 @@ private:
         // Ticket #4257
         check("bool foo() {\n"
               "MyString **str=OtherGetter();\n"
-              "return *str==\"bug\"; }");
-        TODO_ASSERT_EQUALS("[test.cpp:2]: (warning) String literal compared with variable 'c'. Did you intend to use strcmp() instead?\n",
-                           "",
-                           errout.str());
+              "return *str==\"bug\"; }", "test.c");
+        ASSERT_EQUALS("[test.c:3]: (warning) String literal compared with variable '*str'. Did you intend to use strcmp() instead?\n", errout.str());
 
         // Ticket #4257
         check("bool foo() {\n"
               "MyString str=OtherGetter2();\n"
-              "return &str==\"bug\"; }");
-        TODO_ASSERT_EQUALS("[test.cpp:2]: (warning) String literal compared with variable 'c'. Did you intend to use strcmp() instead?\n",
-                           "",
-                           errout.str());
+              "return &str==\"bug\"; }", "test.c");
+        ASSERT_EQUALS("[test.c:3]: (warning) String literal compared with variable '&str'. Did you intend to use strcmp() instead?\n", errout.str());
 
         // Ticket #5734
         check("int foo(char c) {\n"
@@ -345,11 +407,23 @@ private:
               "   if (example.buffer == \"test\") ;\n"
               "}\n", "test.c");
         ASSERT_EQUALS("[test.c:6]: (warning) String literal compared with variable 'example.buffer'. Did you intend to use strcmp() instead?\n", errout.str());
+
+        // #9726
+        check("void f(std::vector<std::string> theArgs) {\n"
+              "   std::string arg1(*theArgs.begin());\n"
+              "   if(arg1 == \"aaa\") {}\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
     }
 
     void suspiciousStringCompare_char() {
         check("bool foo(char* c) {\n"
-              "    return c == '\\0';\n"
+              "    return c == 'x';\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (warning) Char literal compared with pointer 'c'. Did you intend to dereference it?\n", errout.str());
+
+        check("bool foo(wchar_t* c) {\n"
+              "    return c == L'x';\n"
               "}");
         ASSERT_EQUALS("[test.cpp:2]: (warning) Char literal compared with pointer 'c'. Did you intend to dereference it?\n", errout.str());
 
@@ -363,6 +437,21 @@ private:
               "}");
         ASSERT_EQUALS("", errout.str());
 
+        check("bool foo(char* c) {\n"
+              "    return c[0] == '\\0';\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        check("bool foo(char** c) {\n"
+              "    return c[0] == '\\0';\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (warning) Char literal compared with pointer 'c[0]'. Did you intend to dereference it?\n", errout.str());
+
+        check("bool foo(char** c) {\n"
+              "    return *c == '\\0';\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (warning) Char literal compared with pointer '*c'. Did you intend to dereference it?\n", errout.str());
+
         check("bool foo(char c) {\n"
               "    return c == 0;\n"
               "}");
@@ -402,13 +491,22 @@ private:
     }
 
 
+    void snprintf1() {
+        check("void foo()\n"
+              "{\n"
+              "    char buf[100];\n"
+              "    snprintf(buf,100,\"%s\",buf);\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:4]: (error) Undefined behavior: Variable 'buf' is used as parameter and destination in snprintf().\n", errout.str());
+    }
+
     void sprintf1() {
         check("void foo()\n"
               "{\n"
               "    char buf[100];\n"
               "    sprintf(buf,\"%s\",buf);\n"
               "}");
-        ASSERT_EQUALS("[test.cpp:4]: (error) Undefined behavior: Variable 'buf' is used as parameter and destination in s[n]printf().\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:4]: (error) Undefined behavior: Variable 'buf' is used as parameter and destination in sprintf().\n", errout.str());
     }
 
     void sprintf2() {
@@ -445,6 +543,46 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
+    void sprintf5() {
+        check("struct A\n"
+              "{\n"
+              "    char filename[128];\n"
+              "};\n"
+              "\n"
+              "void foo(struct A *a)\n"
+              "{\n"
+              "    snprintf(a->filename, 128, \"%s\", a->filename);\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:8]: (error) Undefined behavior: Variable 'a->filename' is used as parameter and destination in snprintf().\n", errout.str());
+    }
+
+    void sprintf6() {
+        check("void foo()\n"
+              "{\n"
+              "    char buf[100];\n"
+              "    sprintf((char*)buf,\"%s\",(char*)buf);\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:4]: (error) Undefined behavior: Variable 'buf' is used as parameter and destination in sprintf().\n", errout.str());
+    }
+
+    void sprintf7() {
+        check("void foo()\n"
+              "{\n"
+              "    char buf[100];\n"
+              "    sprintf((char*)(void*)buf,\"%s\",(void*)(char*)buf);\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:4]: (error) Undefined behavior: Variable 'buf' is used as parameter and destination in sprintf().\n", errout.str());
+    }
+
+    void wsprintf1() {
+        check("void foo()\n"
+              "{\n"
+              "    wchar_t buf[100];\n"
+              "    swprintf(buf,10, \"%s\",buf);\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:4]: (error) Undefined behavior: Variable 'buf' is used as parameter and destination in swprintf().\n", errout.str());
+    }
+
     void strPlusChar1() {
         // Strange looking pointer arithmetic..
         check("void foo()\n"
@@ -483,30 +621,49 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
+    void strPlusChar4() {
+        // Strange looking pointer arithmetic, wide char..
+        check("void foo()\n"
+              "{\n"
+              "    const wchar_t *p = L\"/usr\" + L'/';\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (error) Unusual pointer arithmetic. A value of type 'wchar_t' is added to a string literal.\n", errout.str());
+
+        check("void foo(wchar_t c)\n"
+              "{\n"
+              "    const wchar_t *p = L\"/usr\" + c;\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (error) Unusual pointer arithmetic. A value of type 'wchar_t' is added to a string literal.\n", errout.str());
+    }
 
     void incorrectStringCompare() {
         check("int f() {\n"
-              "    return test.substr( 0 , 4 ) == \"Hello\" ? : 0 : 1 ;\n"
+              "    return test.substr( 0 , 4 ) == \"Hello\" ? 0 : 1 ;\n"
               "}");
         ASSERT_EQUALS("[test.cpp:2]: (warning) String literal \"Hello\" doesn't match length argument for substr().\n", errout.str());
 
         check("int f() {\n"
-              "    return test.substr( 0 , 5 ) == \"Hello\" ? : 0 : 1 ;\n"
+              "    return test.substr( 0 , 4 ) == L\"Hello\" ? 0 : 1 ;\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (warning) String literal L\"Hello\" doesn't match length argument for substr().\n", errout.str());
+
+        check("int f() {\n"
+              "    return test.substr( 0 , 5 ) == \"Hello\" ? 0 : 1 ;\n"
               "}");
         ASSERT_EQUALS("", errout.str());
 
         check("int f() {\n"
-              "    return \"Hello\" == test.substr( 0 , 4 ) ? : 0 : 1 ;\n"
+              "    return \"Hello\" == test.substr( 0 , 4 ) ? 0 : 1 ;\n"
               "}");
         ASSERT_EQUALS("[test.cpp:2]: (warning) String literal \"Hello\" doesn't match length argument for substr().\n", errout.str());
 
         check("int f() {\n"
-              "    return \"Hello\" == foo.bar<int>().z[1].substr(i+j*4, 4) ? : 0 : 1 ;\n"
+              "    return \"Hello\" == foo.bar<int>().z[1].substr(i+j*4, 4) ? 0 : 1 ;\n"
               "}");
         ASSERT_EQUALS("[test.cpp:2]: (warning) String literal \"Hello\" doesn't match length argument for substr().\n", errout.str());
 
         check("int f() {\n"
-              "    return \"Hello\" == test.substr( 0 , 5 ) ? : 0 : 1 ;\n"
+              "    return \"Hello\" == test.substr( 0 , 5 ) ? 0 : 1 ;\n"
               "}");
         ASSERT_EQUALS("", errout.str());
 
@@ -530,6 +687,11 @@ private:
               "}");
         ASSERT_EQUALS("[test.cpp:2]: (warning) Conversion of string literal \"Hello\" to bool always evaluates to true.\n", errout.str());
 
+        check("int f() {\n"
+              "    return \"Hello\" ? 1 : 2;\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (warning) Conversion of string literal \"Hello\" to bool always evaluates to true.\n", errout.str());
+
         check("int f() {\n"
               "    assert (test || \"Hello\");\n"
               "}");
@@ -559,6 +721,53 @@ private:
               "    return f2(\"Hello\");\n"
               "}");
         ASSERT_EQUALS("", errout.str());
+
+        // #7750 warn about char literals in boolean expressions
+        check("void f() {\n"
+              "  if('a'){}\n"
+              "  if(L'b'){}\n"
+              "  if(1 && 'c'){}\n"
+              "  int x = 'd' ? 1 : 2;\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (warning) Conversion of char literal 'a' to bool always evaluates to true.\n"
+                      "[test.cpp:3]: (warning) Conversion of char literal L'b' to bool always evaluates to true.\n"
+                      "[test.cpp:4]: (warning) Conversion of char literal 'c' to bool always evaluates to true.\n"
+                      "[test.cpp:5]: (warning) Conversion of char literal 'd' to bool always evaluates to true.\n"
+                      , errout.str());
+
+        check("void f() {\n"
+              "  if('\\0'){}\n"
+              "  if(L'\\0'){}\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        check("void f() {\n"
+              "  if('\\0' || cond){}\n"
+              "  if(L'\\0' || cond){}\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (warning) Conversion of char literal '\\0' to bool always evaluates to false.\n"
+                      "[test.cpp:3]: (warning) Conversion of char literal L'\\0' to bool always evaluates to false.\n", errout.str());
+    }
+
+    void deadStrcmp() {
+        check("void f(const char *str) {\n"
+              "  if (strcmp(str, \"abc\") == 0 || strcmp(str, \"def\")) {}\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (warning) The expression 'strcmp(str,\"def\") != 0' is suspicious. It overlaps 'strcmp(str,\"abc\") == 0'.\n", errout.str());
+
+        check("void f(const wchar_t *str) {\n"
+              "  if (wcscmp(str, L\"abc\") == 0 || wcscmp(str, L\"def\")) {}\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (warning) The expression 'wcscmp(str,L\"def\") != 0' is suspicious. It overlaps 'wcscmp(str,L\"abc\") == 0'.\n", errout.str());
+
+        check("struct X {\n"
+              "  char *str;\n"
+              "};\n"
+              "\n"
+              "void f(const struct X *x) {\n"
+              "  if (strcmp(x->str, \"abc\") == 0 || strcmp(x->str, \"def\")) {}\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:6]: (warning) The expression 'strcmp(x->str,\"def\") != 0' is suspicious. It overlaps 'strcmp(x->str,\"abc\") == 0'.\n", errout.str());
     }
 };
 
