@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2020 Cppcheck team.
+ * Copyright (C) 2007-2022 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,26 +18,32 @@
 
 #include "config.h"
 #include "cppcheck.h"
+#include "errortypes.h"
 #include "settings.h"
 #include "suppressions.h"
 #include "testsuite.h"
+#include "testutils.h"
 #include "threadexecutor.h"
 
+#include <algorithm>
 #include <cstddef>
+#include <cstring>
+#include <functional>
+#include <iosfwd>
 #include <list>
 #include <map>
+#include <memory>
 #include <string>
 #include <utility>
 #include <vector>
 
 class TestSuppressions : public TestFixture {
 public:
-    TestSuppressions() : TestFixture("TestSuppressions") {
-    }
+    TestSuppressions() : TestFixture("TestSuppressions") {}
 
 private:
 
-    void run() OVERRIDE {
+    void run() override {
         TEST_CASE(suppressionsBadId1);
         TEST_CASE(suppressionsDosFormat);     // Ticket #1836
         TEST_CASE(suppressionsFileNameWithColon);    // Ticket #1919 - filename includes colon
@@ -80,13 +86,16 @@ private:
         ASSERT_EQUALS("", suppressions.parseFile(s2));
     }
 
-    Suppressions::ErrorMessage errorMessage(const std::string &errorId) const {
+    static Suppressions::ErrorMessage errorMessage(const std::string &errorId) {
         Suppressions::ErrorMessage ret;
         ret.errorId = errorId;
+        ret.hash = 0;
+        ret.lineNumber = 0;
+        ret.certainty = Certainty::CertaintyLevel::normal;
         return ret;
     }
 
-    Suppressions::ErrorMessage errorMessage(const std::string &errorId, const std::string &file, int line) const {
+    static Suppressions::ErrorMessage errorMessage(const std::string &errorId, const std::string &file, int line) {
         Suppressions::ErrorMessage ret;
         ret.errorId = errorId;
         ret.setFileName(file);
@@ -179,13 +188,13 @@ private:
         // Clear the error log
         errout.str("");
 
-        CppCheck cppCheck(*this, true);
+        CppCheck cppCheck(*this, true, nullptr);
         Settings& settings = cppCheck.settings();
         settings.exitCode = 1;
         settings.inlineSuppressions = true;
         if (suppression == "unusedFunction")
-            settings.addEnabled("unusedFunction");
-        settings.addEnabled("information");
+            settings.checks.setEnabled(Checks::unusedFunction, true);
+        settings.severity.enable(Severity::information);
         settings.jointSuppressionReport = true;
         if (!suppression.empty()) {
             std::string r = settings.nomsg.addSuppressionLine(suppression);
@@ -209,18 +218,20 @@ private:
         output.str("");
 
         std::map<std::string, std::size_t> files;
-        files["test.cpp"] = 1;
+        files["test.cpp"] = strlen(code);
 
         Settings settings;
         settings.jobs = 1;
         settings.inlineSuppressions = true;
-        settings.addEnabled("information");
+        settings.severity.enable(Severity::information);
         if (!suppression.empty()) {
             EXPECT_EQ("", settings.nomsg.addSuppressionLine(suppression));
         }
         ThreadExecutor executor(files, settings, *this);
+        std::vector<ScopedFile> scopedfiles;
+        scopedfiles.reserve(files.size());
         for (std::map<std::string, std::size_t>::const_iterator i = files.begin(); i != files.end(); ++i)
-            executor.addFileContent(i->first, code);
+            scopedfiles.emplace_back(i->first, code);
 
         const unsigned int exitCode = executor.check();
 
@@ -477,13 +488,13 @@ private:
         ASSERT_EQUALS(true, s2.isSuppressed(errorMessage("abc", "include/1.h", 142)));
     }
 
-    void suppressionsLine0() {
+    void suppressionsLine0() const {
         Suppressions suppressions;
         suppressions.addSuppressionLine("syntaxError:*:0");
         ASSERT_EQUALS(true, suppressions.isSuppressed(errorMessage("syntaxError", "test.cpp", 0)));
     }
 
-    void suppressionsFileComment() {
+    void suppressionsFileComment() const {
         std::istringstream file1("# comment\nabc");
         Suppressions suppressions1;
         suppressions1.parseFile(file1);
@@ -493,6 +504,26 @@ private:
         Suppressions suppressions2;
         suppressions2.parseFile(file2);
         ASSERT_EQUALS(true, suppressions2.isSuppressed(errorMessage("abc", "test.cpp", 123)));
+
+        std::istringstream file3("abc // comment");
+        Suppressions suppressions3;
+        suppressions3.parseFile(file3);
+        ASSERT_EQUALS(true, suppressions3.isSuppressed(errorMessage("abc", "test.cpp", 123)));
+
+        std::istringstream file4("abc\t\t # comment");
+        Suppressions suppressions4;
+        suppressions4.parseFile(file4);
+        ASSERT_EQUALS(true, suppressions4.isSuppressed(errorMessage("abc", "test.cpp", 123)));
+
+        std::istringstream file5("abc:test.cpp\t\t # comment");
+        Suppressions suppressions5;
+        suppressions5.parseFile(file5);
+        ASSERT_EQUALS(true, suppressions5.isSuppressed(errorMessage("abc", "test.cpp", 123)));
+
+        std::istringstream file6("abc:test.cpp:123\t\t # comment with . inside");
+        Suppressions suppressions6;
+        suppressions6.parseFile(file6);
+        ASSERT_EQUALS(true, suppressions6.isSuppressed(errorMessage("abc", "test.cpp", 123)));
     }
 
     void inlinesuppress() {
@@ -510,8 +541,6 @@ private:
     }
 
     void inlinesuppress_symbolname() {
-        Suppressions suppressions;
-
         checkSuppression("void f() {\n"
                          "    int a;\n"
                          "    /* cppcheck-suppress uninitvar symbolName=a */\n"
@@ -541,33 +570,32 @@ private:
     }
 
     void multi_inlinesuppress() {
-        Suppressions ss;
         std::vector<Suppressions::Suppression> suppressions;
         std::string errMsg;
 
         errMsg = "";
-        suppressions=ss.parseMultiSuppressComment("// cppcheck-suppress[errorId]", &errMsg);
+        suppressions=Suppressions::parseMultiSuppressComment("// cppcheck-suppress[errorId]", &errMsg);
         ASSERT_EQUALS(1, suppressions.size());
         ASSERT_EQUALS("errorId", suppressions[0].errorId);
         ASSERT_EQUALS("", suppressions[0].symbolName);
         ASSERT_EQUALS("", errMsg);
 
         errMsg = "";
-        suppressions=ss.parseMultiSuppressComment("// cppcheck-suppress[errorId symbolName=arr]", &errMsg);
+        suppressions=Suppressions::parseMultiSuppressComment("// cppcheck-suppress[errorId symbolName=arr]", &errMsg);
         ASSERT_EQUALS(1, suppressions.size());
         ASSERT_EQUALS("errorId", suppressions[0].errorId);
         ASSERT_EQUALS("arr", suppressions[0].symbolName);
         ASSERT_EQUALS("", errMsg);
 
         errMsg = "";
-        suppressions=ss.parseMultiSuppressComment("// cppcheck-suppress[errorId symbolName=]", &errMsg);
+        suppressions=Suppressions::parseMultiSuppressComment("// cppcheck-suppress[errorId symbolName=]", &errMsg);
         ASSERT_EQUALS(1, suppressions.size());
         ASSERT_EQUALS("errorId", suppressions[0].errorId);
         ASSERT_EQUALS("", suppressions[0].symbolName);
         ASSERT_EQUALS("", errMsg);
 
         errMsg = "";
-        suppressions=ss.parseMultiSuppressComment("// cppcheck-suppress[errorId1, errorId2 symbolName=arr]", &errMsg);
+        suppressions=Suppressions::parseMultiSuppressComment("// cppcheck-suppress[errorId1, errorId2 symbolName=arr]", &errMsg);
         ASSERT_EQUALS(2, suppressions.size());
         ASSERT_EQUALS("errorId1", suppressions[0].errorId);
         ASSERT_EQUALS("", suppressions[0].symbolName);
@@ -576,53 +604,52 @@ private:
         ASSERT_EQUALS("", errMsg);
 
         errMsg = "";
-        suppressions=ss.parseMultiSuppressComment("// cppcheck-suppress[]", &errMsg);
+        suppressions=Suppressions::parseMultiSuppressComment("// cppcheck-suppress[]", &errMsg);
         ASSERT_EQUALS(0, suppressions.size());
         ASSERT_EQUALS(true, errMsg.empty());
 
         errMsg = "";
-        suppressions=ss.parseMultiSuppressComment("// cppcheck-suppress[errorId", &errMsg);
+        suppressions=Suppressions::parseMultiSuppressComment("// cppcheck-suppress[errorId", &errMsg);
         ASSERT_EQUALS(0, suppressions.size());
         ASSERT_EQUALS(false, errMsg.empty());
 
         errMsg = "";
-        suppressions=ss.parseMultiSuppressComment("// cppcheck-suppress errorId", &errMsg);
+        suppressions=Suppressions::parseMultiSuppressComment("// cppcheck-suppress errorId", &errMsg);
         ASSERT_EQUALS(0, suppressions.size());
         ASSERT_EQUALS(false, errMsg.empty());
 
         errMsg = "";
-        suppressions=ss.parseMultiSuppressComment("// cppcheck-suppress[errorId1 errorId2 symbolName=arr]", &errMsg);
+        suppressions=Suppressions::parseMultiSuppressComment("// cppcheck-suppress[errorId1 errorId2 symbolName=arr]", &errMsg);
         ASSERT_EQUALS(0, suppressions.size());
         ASSERT_EQUALS(false, errMsg.empty());
 
         errMsg = "";
-        suppressions=ss.parseMultiSuppressComment("// cppcheck-suppress[errorId1, errorId2 symbol=arr]", &errMsg);
+        suppressions=Suppressions::parseMultiSuppressComment("// cppcheck-suppress[errorId1, errorId2 symbol=arr]", &errMsg);
         ASSERT_EQUALS(0, suppressions.size());
         ASSERT_EQUALS(false, errMsg.empty());
 
         errMsg = "";
-        suppressions=ss.parseMultiSuppressComment("// cppcheck-suppress[errorId1, errorId2 symbolName]", &errMsg);
+        suppressions=Suppressions::parseMultiSuppressComment("// cppcheck-suppress[errorId1, errorId2 symbolName]", &errMsg);
         ASSERT_EQUALS(0, suppressions.size());
         ASSERT_EQUALS(false, errMsg.empty());
     }
 
     void multi_inlinesuppress_comment() {
-        Suppressions ss;
         std::vector<Suppressions::Suppression> suppressions;
         std::string errMsg;
 
         errMsg = "";
-        suppressions=ss.parseMultiSuppressComment("//cppcheck-suppress[errorId1, errorId2 symbolName=arr]", &errMsg);
+        suppressions=Suppressions::parseMultiSuppressComment("//cppcheck-suppress[errorId1, errorId2 symbolName=arr]", &errMsg);
         ASSERT_EQUALS(2, suppressions.size());
         ASSERT_EQUALS(true, errMsg.empty());
 
         errMsg = "";
-        suppressions=ss.parseMultiSuppressComment("//cppcheck-suppress[errorId1, errorId2 symbolName=arr] some text", &errMsg);
+        suppressions=Suppressions::parseMultiSuppressComment("//cppcheck-suppress[errorId1, errorId2 symbolName=arr] some text", &errMsg);
         ASSERT_EQUALS(2, suppressions.size());
         ASSERT_EQUALS(true, errMsg.empty());
 
         errMsg = "";
-        suppressions=ss.parseMultiSuppressComment("/*cppcheck-suppress[errorId1, errorId2 symbolName=arr]*/", &errMsg);
+        suppressions=Suppressions::parseMultiSuppressComment("/*cppcheck-suppress[errorId1, errorId2 symbolName=arr]*/", &errMsg);
         ASSERT_EQUALS(2, suppressions.size());
         ASSERT_EQUALS(true, errMsg.empty());
     }
@@ -630,7 +657,7 @@ private:
     void globalSuppressions() { // Testing that Cppcheck::useGlobalSuppressions works (#8515)
         errout.str("");
 
-        CppCheck cppCheck(*this, false); // <- do not "use global suppressions". pretend this is a thread that just checks a file.
+        CppCheck cppCheck(*this, false, nullptr); // <- do not "use global suppressions". pretend this is a thread that just checks a file.
         Settings& settings = cppCheck.settings();
         settings.nomsg.addSuppressionLine("uninitvar");
         settings.exitCode = 1;
@@ -642,7 +669,9 @@ private:
 
     void inlinesuppress_unusedFunction() const { // #4210, #4946 - wrong report of "unmatchedSuppression" for "unusedFunction"
         Suppressions suppressions;
-        suppressions.addSuppression(Suppressions::Suppression("unusedFunction", "test.c", 3));
+        auto suppression = Suppressions::Suppression("unusedFunction", "test.c", 3);
+        suppression.checked = true; // have to do this because fixes for #5704
+        suppressions.addSuppression(suppression);
         ASSERT_EQUALS(true, !suppressions.getUnmatchedLocalSuppressions("test.c", true).empty());
         ASSERT_EQUALS(false, !suppressions.getUnmatchedGlobalSuppressions(true).empty());
         ASSERT_EQUALS(false, !suppressions.getUnmatchedLocalSuppressions("test.c", false).empty());
@@ -662,9 +691,9 @@ private:
         // Clear the error log
         errout.str("");
 
-        CppCheck cppCheck(*this, true);
+        CppCheck cppCheck(*this, true, nullptr);
         Settings& settings = cppCheck.settings();
-        settings.addEnabled("style");
+        settings.severity.enable(Severity::style);
         settings.inlineSuppressions = true;
         settings.relativePaths = true;
         settings.basePaths.emplace_back("/somewhere");
@@ -694,7 +723,7 @@ private:
                             "_asm\n"
                             "{\n"
                             "   // cppcheck-suppress syntaxError\n"
-                            "   push  EAX               ; save EAX for callers \n"
+                            "   push  EAX               ; save EAX for callers\n"
                             "   mov   EAX,Real10        ; get the address pointed to by Real10\n"
                             "   fld   TBYTE PTR [EAX]   ; load an extended real (10 bytes)\n"
                             "   fstp  QWORD PTR result  ; store a double (8 bytes)\n"
@@ -723,7 +752,7 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
-    void symbol() {
+    void symbol() const {
         Suppressions::Suppression s;
         s.errorId = "foo";
         s.symbolName = "array*";
