@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2016 Cppcheck team.
+ * Copyright (C) 2007-2022 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,28 +16,28 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "timer.h"
 #include "testsuite.h"
-#include <cmath>
+#include "timer.h"
 
+#include <cmath>
+#include <ctime>
 
 class TestTimer : public TestFixture {
 public:
-    TestTimer() : TestFixture("TestTimer") {
-    }
+    TestTimer() : TestFixture("TestTimer") {}
 
 private:
 
-    void run() {
+    void run() override {
         TEST_CASE(result);
     }
 
     void result() const {
         TimerResultsData t1;
-        t1._clocks = ~(std::clock_t)0;
+        t1.mClocks = ~(std::clock_t)0;
         ASSERT(t1.seconds() > 100.0);
 
-        t1._clocks = CLOCKS_PER_SEC * 5 / 2;
+        t1.mClocks = CLOCKS_PER_SEC * 5 / 2;
         ASSERT(std::fabs(t1.seconds()-2.5) < 0.01);
     }
 };
