@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2016 Cppcheck team.
+ * Copyright (C) 2007-2022 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,10 +16,14 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "testsuite.h"
+
 #include "checkvaarg.h"
+#include "errortypes.h"
+#include "settings.h"
+#include "testsuite.h"
 #include "tokenize.h"
 
+#include <iosfwd>
 
 class TestVaarg : public TestFixture {
 public:
@@ -28,22 +32,23 @@ public:
 private:
     Settings settings;
 
-    void check(const char code[]) {
+#define check(code) check_(code, __FILE__, __LINE__)
+    void check_(const char code[], const char* file, int line) {
         // Clear the error buffer..
         errout.str("");
 
         // Tokenize..
         Tokenizer tokenizer(&settings, this);
         std::istringstream istr(code);
-        tokenizer.tokenize(istr, "test.cpp");
+        ASSERT_LOC(tokenizer.tokenize(istr, "test.cpp"), file, line);
 
         // Check..
-        CheckVaarg checkVaarg(&tokenizer, &settings, this);
-        checkVaarg.runSimplifiedChecks(&tokenizer, &settings, this);
+        CheckVaarg checkVaarg;
+        checkVaarg.runChecks(&tokenizer, &settings, this);
     }
 
-    void run() {
-        settings.addEnabled("warning");
+    void run() override {
+        settings.severity.enable(Severity::warning);
 
         TEST_CASE(wrongParameterTo_va_start);
         TEST_CASE(referenceAs_va_start);
@@ -153,6 +158,32 @@ private:
               "    va_end(arg_ptr);\n"
               "}");
         ASSERT_EQUALS("[test.cpp:4]: (error) va_list 'arg_ptr' was opened but not closed by va_end().\n", errout.str());
+
+        // #8124
+        check("void f(int n, ...)\n"
+              "{\n"
+              "    va_list ap;\n"
+              "    va_start(ap, n);\n"
+              "    std::vector<std::string> v(n);\n"
+              "    std::generate_n(v.begin(), n, [&ap]()\n"
+              "    {\n"
+              "        return va_arg(ap, const char*);\n"
+              "    });\n"
+              "    va_end(ap);\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        check("void f(int n, ...)\n"
+              "{\n"
+              "    va_list ap;\n"
+              "    va_start(ap, n);\n"
+              "    std::vector<std::string> v(n);\n"
+              "    std::generate_n(v.begin(), n, [&ap]()\n"
+              "    {\n"
+              "        return va_arg(ap, const char*);\n"
+              "    });\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:10]: (error) va_list 'ap' was opened but not closed by va_end().\n", errout.str());
     }
 
     void va_list_usedBeforeStarted() {
@@ -182,6 +213,13 @@ private:
               "}");
         ASSERT_EQUALS("[test.cpp:5]: (error) va_list 'arg_ptr' used before va_start() was called.\n", errout.str());
 
+        check("void Format(char* szFormat, char (*szBuffer)[_Size], ...) {\n"
+              "    va_list arg_ptr;\n"
+              "    va_end(arg_ptr);\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (error) va_list 'arg_ptr' used before va_start() was called.\n", errout.str());
+
+
         check("void Format(char* szFormat, char (*szBuffer)[_Size], ...) {\n"
               "    va_list arg_ptr;\n"
               "    va_start(arg_ptr, szBuffer);\n"
@@ -216,6 +254,68 @@ private:
               "    }\n"
               "}");
         ASSERT_EQUALS("", errout.str());
+
+        // #7533
+        check("void action_push(int type, ...) {\n"
+              "    va_list args;\n"
+              "    va_start(args, type);\n"
+              "    switch (push_mode) {\n"
+              "    case UNDO:\n"
+              "        list_add(&act->node, &to_redo);\n"
+              "        break;\n"
+              "    case REDO:\n"
+              "        list_add(&act->node, &to_undo);\n"
+              "        break;\n"
+              "    }\n"
+              "    va_end(args);\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        check("void action_push(int type, ...) {\n"
+              "    va_list args;\n"
+              "    va_start(args, type);\n"
+              "    switch (push_mode) {\n"
+              "    case UNDO:\n"
+              "        list_add(&act->node, &to_redo);\n"
+              "        va_end(args);\n"
+              "        break;\n"
+              "    case REDO:\n"
+              "        list_add(&act->node, &to_undo);\n"
+              "        va_end(args);\n"
+              "        break;\n"
+              "    }\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        check("void action_push(int type, ...) {\n"
+              "    va_list args;\n"
+              "    va_start(args, type);\n"
+              "    switch (push_mode) {\n"
+              "    case UNDO:\n"
+              "        list_add(&act->node, &to_redo);\n"
+              "        break;\n"
+              "    case REDO:\n"
+              "        list_add(&act->node, &to_undo);\n"
+              "        va_end(args);\n"
+              "        break;\n"
+              "    }\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:13]: (error) va_list 'args' was opened but not closed by va_end().\n", errout.str());
+
+        // #8043
+        check("void redisvFormatCommand(char *format, va_list ap, bool flag) {\n"
+              "    va_list _cpy;\n"
+              "    va_copy(_cpy, ap);\n"
+              "    if (flag)\n"
+              "        goto fmt_valid;\n"
+              "    va_end(_cpy);\n"
+              "    goto format_err;\n"
+              "fmt_valid:\n"
+              "    sdscatvprintf(curarg, _format, _cpy);\n"
+              "    va_end(_cpy);\n"
+              "format_err:\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
     }
 
     void va_start_subsequentCalls() {
@@ -252,6 +352,17 @@ private:
               "  BG_TString result(f);\n"
               "}");
         ASSERT_EQUALS("", errout.str());
+
+        // #7559
+        check("void mowgli_object_message_broadcast(mowgli_object_t *self, const char *name, ...) {\n"
+              "  va_list va;\n"
+              "  MOWGLI_LIST_FOREACH(n, self->klass->message_handlers.head) {\n"
+              "    if (!strcasecmp(sig2->name, name))\n"
+              "      break;\n"
+              "  }\n"
+              "  va_start(va, name);\n"
+              "  va_end(va);\n"
+              "}");
     }
 };
 
